@@ -31,10 +31,11 @@ DO_CONFIG=1
 DO_BUILD=1
 DO_INSTALL=0
 SUDO=""
+CMAKE_OPTIONS_FROM_CMDLINE=""
 
 PRINT_HELP=0
 
-FEATURES="CAMERA COLORD FLICKR GRAPHICSMAGICK IMAGEMAGICK KWALLET LIBSECRET LUA MAP MAC_INTEGRATION NLS OPENCL OPENEXR OPENMP UNITY WEBP GAME"
+FEATURES="CAMERA COLORD GAME GRAPHICSMAGICK IMAGEMAGICK JXL KWALLET LIBSECRET LUA MAC_INTEGRATION MAP NLS OPENCL OPENEXR OPENMP UNITY WEBP"
 
 # prepare a lowercase version with a space before and after
 # it's very important for parse_feature, has no impact in for loop expansions
@@ -120,6 +121,10 @@ parse_args()
 		-h|--help)
 			PRINT_HELP=1
 			;;
+		--)
+			shift
+			CMAKE_OPTIONS_FROM_CMDLINE="$@"
+			;;
 		*)
 			echo "warning: ignoring unknown option $option"
 			;;
@@ -135,7 +140,7 @@ parse_args()
 print_help()
 {
 	cat <<EOF
-$(basename $0) [OPTIONS]
+$(basename $0) [OPTIONS] [-- [additional cmake configuration options...]]
 
 Options:
 Installation:
@@ -170,6 +175,9 @@ Cleanup actions:
 -f --force                    Force clean-build to perform removal
                               ignoring any errors
 
+Additional cmake configuration options:
+Options passed as is to the cmake configure command
+
 Features:
 By default cmake will enable the features it autodetects on the build machine.
 Specifying the option on the command line forces the feature on or off.
@@ -362,7 +370,7 @@ if [ $ADDRESS_SANITIZER -ne 0 ] ; then
 fi
 
 
-cmd_config="${ASAN_FLAGS}cmake -G \"$BUILD_GENERATOR\" -DCMAKE_INSTALL_PREFIX=${INSTALL_PREFIX} -DCMAKE_BUILD_TYPE=${BUILD_TYPE} ${CMAKE_MORE_OPTIONS} \"$DT_SRC_DIR\""
+cmd_config="${ASAN_FLAGS}cmake -G \"$BUILD_GENERATOR\" -DCMAKE_INSTALL_PREFIX=${INSTALL_PREFIX} -DCMAKE_BUILD_TYPE=${BUILD_TYPE} ${CMAKE_MORE_OPTIONS} ${CMAKE_OPTIONS_FROM_CMDLINE} \"$DT_SRC_DIR\""
 cmd_build="cmake --build "$BUILD_DIR" -- -j$MAKE_TASKS"
 cmd_install="${SUDO}cmake --build \"$BUILD_DIR\" --target install -- -j$MAKE_TASKS"
 
