@@ -143,18 +143,15 @@ typedef struct dt_collection_t
   int clone;
   gchar *query, *query_no_group;
   gchar **where_ext;
-  unsigned int count, count_no_group;
-  unsigned int tagid;
+  uint32_t count, count_no_group;
+  uint32_t tagid;
   dt_collection_params_t params;
   dt_collection_params_t store;
 } dt_collection_t;
 
 /* returns the name for the given collection property */
-const char *dt_collection_name(dt_collection_properties_t prop);
-const char *dt_collection_name_untranslated(dt_collection_properties_t prop);
-/* returns the name for the given collection sort property */
-const char *dt_collection_sort_name(dt_collection_sort_t sort);
-const char *dt_collection_sort_name_untranslated(dt_collection_sort_t sort);
+const char *dt_collection_name(const dt_collection_properties_t prop);
+const char *dt_collection_name_untranslated(const dt_collection_properties_t prop);
 
 /** instantiates a collection context, if clone equals NULL default query is constructed. */
 const dt_collection_t *dt_collection_new(const dt_collection_t *clone);
@@ -165,7 +162,8 @@ const dt_collection_params_t *dt_collection_params(const dt_collection_t *collec
 /** get the filtered map between sanitized makermodel and exif maker/model **/
 void dt_collection_get_makermodels(const gchar *filter, GList **sanitized, GList **exif);
 /** get the sanitized makermodel for exif maker/model **/
-gchar *dt_collection_get_makermodel(const char *exif_maker, const char *exif_model);
+gchar *dt_collection_get_makermodel(const char *exif_maker,
+                                    const char *exif_model);
 /** get the generated query for collection */
 const gchar *dt_collection_get_query(const dt_collection_t *collection);
 /** get the generated query for collection including the images hidden in groups */
@@ -175,22 +173,27 @@ int dt_collection_update(const dt_collection_t *collection);
 /** reset collection to default dummy selection */
 void dt_collection_reset(const dt_collection_t *collection);
 /** gets an extended where part */
-gchar *dt_collection_get_extended_where(const dt_collection_t *collection, int exclude);
+gchar *dt_collection_get_extended_where(const dt_collection_t *collection,
+                                        const int exclude);
 /** sets an extended where part */
-void dt_collection_set_extended_where(const dt_collection_t *collection, gchar **extended_where);
+void dt_collection_set_extended_where(const dt_collection_t *collection,
+                                      gchar **extended_where);
 
 /** get filter flags for collection */
 uint32_t dt_collection_get_filter_flags(const dt_collection_t *collection);
 /** set filter flags for collection */
-void dt_collection_set_filter_flags(const dt_collection_t *collection, uint32_t flags);
+void dt_collection_set_filter_flags(const dt_collection_t *collection,
+                                    const uint32_t flags);
 
 /** get filter flags for collection */
 uint32_t dt_collection_get_query_flags(const dt_collection_t *collection);
 /** set filter flags for collection */
-void dt_collection_set_query_flags(const dt_collection_t *collection, uint32_t flags);
+void dt_collection_set_query_flags(const dt_collection_t *collection,
+                                   const uint32_t flags);
 
 /** set the film_id of collection */
-void dt_collection_set_film_id(const dt_collection_t *collection, const int32_t film_id);
+void dt_collection_set_film_id(const dt_collection_t *collection,
+                               const int32_t film_id);
 /** set the tagid of collection */
 void dt_collection_set_tag_id(dt_collection_t *collection, const uint32_t tagid);
 
@@ -205,48 +208,66 @@ uint32_t dt_collection_get_count(const dt_collection_t *collection);
 /** get the count of query including the images hidden in groups */
 uint32_t dt_collection_get_count_no_group(const dt_collection_t *collection);
 /** get the nth image in the query */
-int dt_collection_get_nth(const dt_collection_t *collection, int nth);
-/** get all image ids order as current selection. no more than limit many images are returned, <0 ==
- * unlimited */
-GList *dt_collection_get_all(const dt_collection_t *collection, int limit);
-/** get selected image ids order as current selection. no more than limit many images are returned, <0 ==
- * unlimited */
-GList *dt_collection_get_selected(const dt_collection_t *collection, int limit);
+int dt_collection_get_nth(const dt_collection_t *collection, const int nth);
+/** get all image ids order as current selection. no more than limit
+ * many images are returned, <0 == unlimited */
+GList *dt_collection_get_all(const dt_collection_t *collection, const int limit);
+/** get selected image ids order as current selection. no more than
+ * limit many images are returned, <0 == unlimited */
+GList *dt_collection_get_selected(const dt_collection_t *collection, const int limit);
+
 /** get the count of selected images */
-uint32_t dt_collection_get_selected_count(const dt_collection_t *collection);
+uint32_t dt_collection_get_selected_count(void);
+/** get the count of collected images */
+uint32_t dt_collection_get_collected_count(void);
 
 /** update query by conf vars */
-void dt_collection_update_query(const dt_collection_t *collection, dt_collection_change_t query_change,
-                                dt_collection_properties_t changed_property, GList *list);
+void dt_collection_update_query(const dt_collection_t *collection,
+                                const dt_collection_change_t query_change,
+                                const dt_collection_properties_t changed_property,
+                                GList *list);
 
 /** updates the hint message for collection */
 void dt_collection_hint_message(const dt_collection_t *collection);
 
 /** returns the image offset in the collection */
-int dt_collection_image_offset(int imgid);
+int dt_collection_image_offset(dt_imgid_t imgid);
 
 /* serialize and deserialize into a string. */
 void dt_collection_deserialize(const char *buf, gboolean filtering);
 int dt_collection_serialize(char *buf, int bufsize, gboolean filtering);
 
 /* splits an input string into a number part and an optional operator part */
-void dt_collection_split_operator_number(const gchar *input, char **number1, char **number2, char **op);
-void dt_collection_split_operator_datetime(const gchar *input, char **number1, char **number2, char **op);
-void dt_collection_split_operator_exposure(const gchar *input, char **number1, char **number2, char **op);
+void dt_collection_split_operator_number(const gchar *input,
+                                         char **number1,
+                                         char **number2,
+                                         char **op);
+void dt_collection_split_operator_datetime(const gchar *input,
+                                           char **number1,
+                                           char **number2,
+                                           char **op);
+void dt_collection_split_operator_exposure(const gchar *input,
+                                           char **number1,
+                                           char **number2,
+                                           char **op);
 
-int64_t dt_collection_get_image_position(const int32_t image_id, const int32_t tagid);
-void dt_collection_shift_image_positions(const unsigned int length, const int64_t image_position,
+int64_t dt_collection_get_image_position(const dt_imgid_t image_id,
+                                         const int32_t tagid);
+void dt_collection_shift_image_positions(const unsigned int length,
+                                         const int64_t image_position,
                                          const int32_t tagid);
 
 /* move images with drag and drop */
-void dt_collection_move_before(const int32_t image_id, GList * selected_images);
+void dt_collection_move_before(const dt_imgid_t image_id, GList * selected_images);
 
 /* initialize memory table */
 void dt_collection_memory_update();
 
+/** save the current collection for recentcollect module and collect history */
+void dt_collection_history_save();
+
 // clang-format off
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.py
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
 // clang-format on
-
