@@ -20,6 +20,10 @@
 #include <glib.h>
 #include "common/image.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #define DT_DATETIME_EXIF_LENGTH 20  // exif format string length
 
 // The GTimeSpan saved in db is an offset to datetime_origin (0001:01:01 00:00:00)
@@ -114,6 +118,10 @@ GTimeSpan dt_datetime_gtimespan_add_numbers(const GTimeSpan dt, const dt_datetim
 gboolean dt_datetime_exif_add_numbers(const gchar *exif, const dt_datetime_t numbers, const gboolean add,
                                       gchar **result);
 
+#ifdef __cplusplus
+} // extern "C"
+#endif /* __cplusplus */
+
 // clang-format off
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.py
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
