@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2017-2021 darktable developers.
+    Copyright (C) 2017-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,22 +41,6 @@
 // width, if greater) to keep memory use under control.
 #define GF_TILE_SIZE 512
 
-// some shorthand to make code more legible
-// if we have OpenMP simd enabled, declare a vectorizable for loop;
-// otherwise, just leave it a plain for()
-#if defined(_OPENMP) && defined(OPENMP_SIMD_)
-#define SIMD_FOR \
-  _Pragma("omp simd") \
-  for
-#else
-#define SIMD_FOR for
-#endif
-
-// avoid cluttering the scalar codepath with #ifdefs by hiding the dependency on SSE2
-#ifndef __SSE2__
-# define _mm_prefetch(where,hint)
-#endif
-
 // the filter does internal tiling to keep memory requirements reasonable, so this structure
 // defines the position of the tile being processed
 typedef struct tile
