@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2019-2021 darktable developers.
+    Copyright (C) 2019-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "common/calculator.h"
 #include "common/darktable.h"
 #include "common/file_location.h"
+#include "common/math.h"
 #include "control/conf.h"
 
 #include <glib.h>
@@ -148,14 +149,14 @@ int dt_conf_get_int_fast(const char *name)
 {
   const char *str = dt_conf_get_var(name);
   float new_value = dt_calculator_solve(1, str);
-  if(isnan(new_value))
+  if(dt_isnan(new_value))
   {
     //we've got garbage, check default
     const char *def_val = dt_confgen_get(name, DT_DEFAULT);
     if(def_val)
     {
       new_value = dt_calculator_solve(1, def_val);
-      if(isnan(new_value))
+      if(dt_isnan(new_value))
         new_value = 0.0;
       else
       {
@@ -191,14 +192,14 @@ int64_t dt_conf_get_int64_fast(const char *name)
 {
   const char *str = dt_conf_get_var(name);
   float new_value = dt_calculator_solve(1, str);
-  if(isnan(new_value))
+  if(dt_isnan(new_value))
   {
     //we've got garbage, check default
     const char *def_val = dt_confgen_get(name, DT_DEFAULT);
     if(def_val)
     {
       new_value = dt_calculator_solve(1, def_val);
-      if(isnan(new_value))
+      if(dt_isnan(new_value))
         new_value = 0.0;
       else
       {
@@ -234,14 +235,14 @@ float dt_conf_get_float_fast(const char *name)
 {
   const char *str = dt_conf_get_var(name);
   float new_value = dt_calculator_solve(1, str);
-  if(isnan(new_value))
+  if(dt_isnan(new_value))
   {
     //we've got garbage, check default
     const char *def_val = dt_confgen_get(name, DT_DEFAULT);
     if(def_val)
     {
       new_value = dt_calculator_solve(1, def_val);
-      if(isnan(new_value))
+      if(dt_isnan(new_value))
         new_value = 0.0;
       else
       {
@@ -342,6 +343,9 @@ gboolean dt_conf_is_equal(const char *name, const char *value)
 
 static char *_sanitize_confgen(const char *name, const char *value)
 {
+  if(!darktable.conf->x_confgen)
+    return g_strdup(value);
+
   const dt_confgen_value_t *item = g_hash_table_lookup(darktable.conf->x_confgen, name);
 
   if(!item) return g_strdup(value);
@@ -357,7 +361,7 @@ static char *_sanitize_confgen(const char *name, const char *value)
       const int min = item->min ? (int)dt_calculator_solve(1, item->min) : INT_MIN;
       const int max = item->max ? (int)dt_calculator_solve(1, item->max) : INT_MAX;
       // if garbage, use default
-      const int val = isnan(v) ? dt_confgen_get_int(name, DT_DEFAULT) : (int)v;
+      const int val = dt_isnan(v) ? dt_confgen_get_int(name, DT_DEFAULT) : (int)v;
       result = g_strdup_printf("%d", CLAMP(val, min, max));
     }
     break;
@@ -368,7 +372,7 @@ static char *_sanitize_confgen(const char *name, const char *value)
       const int64_t min = item->min ? (int64_t)dt_calculator_solve(1, item->min) : INT64_MIN;
       const int64_t max = item->max ? (int64_t)dt_calculator_solve(1, item->max) : INT64_MAX;
       // if garbage, use default
-      const int64_t val = isnan(v) ? dt_confgen_get_int64(name, DT_DEFAULT) : (int64_t)v;
+      const int64_t val = dt_isnan(v) ? dt_confgen_get_int64(name, DT_DEFAULT) : (int64_t)v;
       result = g_strdup_printf("%"PRId64, CLAMP(val, min, max));
     }
     break;
@@ -379,7 +383,7 @@ static char *_sanitize_confgen(const char *name, const char *value)
       const float min = item->min ? (float)dt_calculator_solve(1, item->min) : -FLT_MAX;
       const float max = item->max ? (float)dt_calculator_solve(1, item->max) : FLT_MAX;
       // if garbage, use default
-      const float val = isnan(v) ? dt_confgen_get_float(name, DT_DEFAULT) : v;
+      const float val = dt_isnan(v) ? dt_confgen_get_float(name, DT_DEFAULT) : v;
       result = g_strdup_printf("%f", CLAMP(val, min, max));
     }
     break;
@@ -393,13 +397,17 @@ static char *_sanitize_confgen(const char *name, const char *value)
     break;
     case DT_ENUM:
     {
-      char *v = g_strdup_printf("[%s]", value);
-      if(!strstr(item->enum_values, v))
-        result = g_strdup(dt_confgen_get(name, DT_DEFAULT));
-      else
-        result = g_strdup(value);
+      size_t n = strlen(value);
+      char *v = item->enum_values;
+      while(v++)
+      {
+        if(!g_ascii_strncasecmp(value, v, n) && v[n] == ']')
+          return g_strndup(v, n);
 
-      g_free(v);
+        v = strchr(v, '[');
+      }
+
+      result = g_strdup(dt_confgen_get(name, DT_DEFAULT));
     }
     break;
     default:
@@ -410,15 +418,9 @@ static char *_sanitize_confgen(const char *name, const char *value)
   return result;
 }
 
-void dt_conf_init(dt_conf_t *cf, const char *filename, GSList *override_entries)
+gchar *dt_conf_read_values(const char *filename,
+                           gchar* (*callback)(const gchar *key, const gchar *value))
 {
-  cf->table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-  cf->override_entries = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-  dt_pthread_mutex_init(&darktable.conf->mutex, NULL);
-
-  // init conf filename
-  g_strlcpy(darktable.conf->filename, filename, sizeof(darktable.conf->filename));
-
 #define LINE_SIZE 1023
 
   char line[LINE_SIZE + 1];
@@ -447,19 +449,22 @@ void dt_conf_init(dt_conf_t *cf, const char *filename, GSList *override_entries)
         {
           *c = '\0';
 
-          char *name = g_strdup(line);
+          const char *name = line;
           // ensure that numbers are properly clamped if min/max
           // defined and if not and garbage is read then the default
           // value is returned.
           char *value = _sanitize_confgen(name, (const char *)(c + 1));
 
-          g_hash_table_insert(darktable.conf->table, name, value);
+          gchar *v = callback(name, value);
+          g_free(value);
+          if(v)
+            return v;
         }
       }
     }
     fclose(f);
   }
-  else
+  else if(darktable.conf->x_confgen)
   {
     // we initialize the conf table with default values
     GHashTableIter iter;
@@ -470,10 +475,32 @@ void dt_conf_init(dt_conf_t *cf, const char *filename, GSList *override_entries)
     {
       const char *name = (const char *)key;
       const dt_confgen_value_t *entry = (dt_confgen_value_t *)value;
-      g_hash_table_insert(darktable.conf->table, g_strdup(name), g_strdup(entry->def));
+      gchar *v = callback(name, entry->def);
+      if(v)
+        return v;
     }
   }
 
+  return NULL;
+}
+
+gchar *_conf_insert_value(const gchar *key, const gchar *value)
+{
+  g_hash_table_insert(darktable.conf->table, g_strdup(key), g_strdup(value));
+  return NULL;
+}
+
+void dt_conf_init(dt_conf_t *cf, const char *filename, GSList *override_entries)
+{
+  cf->table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+  cf->override_entries = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+  dt_pthread_mutex_init(&darktable.conf->mutex, NULL);
+
+  // init conf filename
+  g_strlcpy(darktable.conf->filename, filename, sizeof(darktable.conf->filename));
+
+  dt_conf_read_values(filename, _conf_insert_value);
+
   if(override_entries)
   {
     for(GSList *p = override_entries; p; p = g_slist_next(p))
@@ -638,13 +665,13 @@ int dt_confgen_get_int(const char *name, dt_confgen_value_kind_t kind)
   switch(kind)
   {
   case DT_MIN:
-    return isnan(value) ? INT_MIN : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? INT_MIN : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   case DT_MAX:
-    return isnan(value) ? INT_MAX : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? INT_MAX : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   default:
-    return isnan(value) ? 0.0f : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? 0.0f : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   }
   return (int)value;
@@ -676,13 +703,13 @@ int64_t dt_confgen_get_int64(const char *name, dt_confgen_value_kind_t kind)
   switch(kind)
   {
   case DT_MIN:
-    return isnan(value) ? INT64_MIN : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? INT64_MIN : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   case DT_MAX:
-    return isnan(value) ? INT64_MAX : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? INT64_MAX : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   default:
-    return isnan(value) ? 0.0f : (value > 0 ? value + 0.5f : value - 0.5f);
+    return dt_isnan(value) ? 0.0f : (value > 0 ? value + 0.5f : value - 0.5f);
     break;
   }
   return (int64_t)value;
@@ -722,15 +749,15 @@ float dt_confgen_get_float(const char *name, dt_confgen_value_kind_t kind)
   {
   case DT_MIN:
     // to anyone askig FLT_MIN is superclose to 0, not furthest value from 0 possible in float
-    return isnan(value) ? -FLT_MAX : value;
+    return dt_isnan(value) ? -FLT_MAX : value;
     break;
   case DT_MAX:
-    return isnan(value) ? FLT_MAX : value;
+    return dt_isnan(value) ? FLT_MAX : value;
     break;
   default:
     break;
   }
-  return isnan(value) ? 0.0f : value;
+  return dt_isnan(value) ? 0.0f : value;
 }
 
 gboolean dt_conf_is_default(const char *name)
