@@ -25,10 +25,10 @@
 #include "common/Common.h"                     // for roundUp
 #include "common/RawImage.h"                   // for RawImage, RawImageData
 #include <array>                               // for array
-#include <benchmark/benchmark.h>               // for State, Benchmark, Cou...
 #include <cstdint>                             // for uint16_t
 #include <tuple>                               // for array
 #include <type_traits>                         // for integral_constant
+#include <benchmark/benchmark.h>               // for State, Benchmark, Cou...
 
 using rawspeed::Cr2sRawInterpolator;
 using rawspeed::iPoint2D;
@@ -83,6 +83,13 @@ static inline void BM_Cr2sRawInterpolator(benchmark::State& state) {
 static inline void CustomArguments(benchmark::internal::Benchmark* b) {
   b->MeasureProcessCPUTime();
   b->UseRealTime();
+
+  if (benchmarkDryRun()) {
+    static constexpr int L2dByteSize = 512U * (1U << 10U);
+    b->Arg((L2dByteSize / (16 / 8)) / (3 * 2));
+    return;
+  }
+
   b->RangeMultiplier(2);
 #if 1
   b->Arg(2 * 3 * 2 * 1'000'000);
