@@ -19,6 +19,7 @@
 */
 
 #include "decompressors/CrwDecompressor.h" // for CrwDecompressor
+#include "MemorySanitizer.h"               // for MSan
 #include "common/RawImage.h"               // for RawImage, RawImageData
 #include "common/RawspeedException.h"      // for RawspeedException
 #include "fuzz/Common.h"                   // for CreateRawImage
@@ -49,7 +50,8 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* Data, size_t Size) {
     mRaw->createData();
     c.decompress();
 
-    mRaw->checkMemIsInitialized();
+    rawspeed::MSan::CheckMemIsInitialized(
+        mRaw->getByteDataAsUncroppedArray2DRef());
   } catch (const rawspeed::RawspeedException&) {
     // Exceptions are good, crashes are bad.
   }
