@@ -19,6 +19,7 @@
 */
 
 #include "decompressors/PentaxDecompressor.h" // for PentaxDecompressor
+#include "MemorySanitizer.h"                  // for MSan
 #include "common/RawImage.h"                  // for RawImage, RawImageData
 #include "common/RawspeedException.h"         // for RawspeedException
 #include "fuzz/Common.h"                      // for CreateRawImage
@@ -55,7 +56,8 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* Data, size_t Size) {
     mRaw->createData();
     p.decompress(rawData);
 
-    mRaw->checkMemIsInitialized();
+    rawspeed::MSan::CheckMemIsInitialized(
+        mRaw->getByteDataAsUncroppedArray2DRef());
   } catch (const rawspeed::RawspeedException&) {
     // Exceptions are good, crashes are bad.
   }
