@@ -18,10 +18,11 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include "common/Common.h" // for rawspeed_get_number_of_processor_cores
+#include "rawspeedconfig.h" // for RAWSPEED_READNONE
+#include "common/Common.h"  // for rawspeed_get_number_of_processor_cores
 
 // define this function, it is only declared in rawspeed:
 // for fuzzing, do not want any threading.
-extern "C" int __attribute__((const)) rawspeed_get_number_of_processor_cores() {
+extern "C" int RAWSPEED_READNONE rawspeed_get_number_of_processor_cores() {
   return 1;
 }
