@@ -20,7 +20,9 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
+#include "rawspeedconfig.h" // for RAWSPEED_READONLY
 #include "common/DngOpcodes.h"
+#include "adt/CroppedArray1DRef.h"        // for CroppedArray1DRef
 #include "adt/CroppedArray2DRef.h"        // for CroppedArray2DRef
 #include "adt/Mutex.h"                    // for MutexLocker
 #include "adt/Point.h"                    // for iRectangle2D, iPoint2D
@@ -31,13 +33,12 @@
 #include "io/Endianness.h"                // for Endianness, Endianness::big
 #include <algorithm>                      // for generate_n, clamp, fill_n
 #include <cassert>                        // for assert
-#include <cmath>                          // for abs, isfinite, pow
+#include <cmath>                          // for isfinite, pow
 #include <cstdlib>                        // for abs
 #include <initializer_list>               // for initializer_list
 #include <iterator>                       // for back_insert_iterator, back...
 #include <limits>                         // for numeric_limits
-#include <optional>
-#include <stdexcept>                      // for out_of_range
+#include <optional>                       // for optional, nullopt
 #include <tuple>                          // for tie, tuple
 // IWYU pragma: no_include <ext/alloc_traits.h>
 // IWYU pragma: no_include <type_traits>
@@ -46,16 +47,16 @@
 #include <exception>
 #endif
 
-using std::vector;
 using std::fill_n;
 using std::make_pair;
+using std::vector;
 
 namespace rawspeed {
 
 namespace {
 
 template <typename T>
-iRectangle2D getImageCropAsRectangle(const CroppedArray2DRef<T>& img) {
+iRectangle2D getImageCropAsRectangle(CroppedArray2DRef<T> img) {
   return {{img.offsetCols, img.offsetRows},
           {img.croppedWidth, img.croppedHeight}};
 }
@@ -202,7 +203,7 @@ protected:
     assert(roi.isThisInside(subImage));
   }
 
-  [[nodiscard]] const iRectangle2D& __attribute__((pure)) getRoi() const {
+  [[nodiscard]] const iRectangle2D& RAWSPEED_READONLY getRoi() const {
     return roi;
   }
 };
@@ -217,9 +218,7 @@ public:
     DummyROIOpcode::setup(ri);
   }
 
-  [[nodiscard]] const iRectangle2D& __attribute__((pure)) getRoi() const {
-    return ROIOpcode::getRoi();
-  }
+  using ROIOpcode::getRoi;
 
   [[noreturn]] void apply(const RawImage& ri) override {
     assert(false && "You should not be calling this.");
@@ -333,7 +332,7 @@ protected:
       ThrowRDE("Invalid pitch");
   }
 
-  [[nodiscard]] iPoint2D __attribute__((pure)) getPitch() const {
+  [[nodiscard]] iPoint2D RAWSPEED_READONLY getPitch() const {
     return {static_cast<int>(colPitch), static_cast<int>(rowPitch)};
   }
 
