@@ -62,18 +62,19 @@ void TableLookUp::setTable(int ntable, const std::vector<uint16_t>& table) {
     int center = table[i];
     int lower = i > 0 ? table[i - 1] : center;
     int upper = i < (nfilled - 1) ? table[i + 1] : center;
+    // Non-monotonic LUT handling: don't interpolate across the cross-over.
+    lower = std::min(lower, center);
+    upper = std::max(upper, center);
     int delta = upper - lower;
+    invariant(delta >= 0);
     t[i * 2] = clampBits(center - ((upper - lower + 2) / 4), 16);
-    t[i * 2 + 1] = uint16_t(delta);
-    // FIXME: this is completely broken when the curve is non-monotonic.
+    t[i * 2 + 1] = delta;
   }
 
   for (int i = nfilled; i < TABLE_MAX_ELTS; i++) {
     t[i * 2] = table[nfilled - 1];
     t[i * 2 + 1] = 0;
   }
-  t[0] = t[1];
-  t[TABLE_SIZE - 1] = t[TABLE_SIZE - 2];
 }
 
 uint16_t* TableLookUp::getTable(int n) {
