@@ -20,6 +20,8 @@
 */
 
 #include "decoders/DcsDecoder.h"
+#include "adt/Point.h"                              // for iPoint2D, iRecta...
+#include "common/Common.h"                          // for BitOrder, BitOrd...
 #include "decoders/RawDecoderException.h"           // for ThrowException
 #include "decompressors/UncompressedDecompressor.h" // for UncompressedDeco...
 #include "io/Buffer.h"                              // for Buffer, DataBuffer
@@ -37,7 +39,7 @@ namespace rawspeed {
 class CameraMetaData;
 
 bool DcsDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
-                                      [[maybe_unused]] const Buffer& file) {
+                                      [[maybe_unused]] Buffer file) {
   const auto id = rootIFD->getID();
   const std::string& make = id.make;
 
@@ -67,12 +69,14 @@ RawImage DcsDecoder::decodeRawInternal() {
 
   UncompressedDecompressor u(
       ByteStream(DataBuffer(mFile.getSubView(off, c2), Endianness::little)),
-      mRaw);
+      mRaw, iRectangle2D({0, 0}, iPoint2D(width, height)), 8 * width / 8, 8,
+      BitOrder::LSB);
+  mRaw->createData();
 
   if (uncorrectedRawValues)
-    u.decode8BitRaw<true>(width, height);
+    u.decode8BitRaw<true>();
   else
-    u.decode8BitRaw<false>(width, height);
+    u.decode8BitRaw<false>();
 
   return mRaw;
 }
