@@ -21,8 +21,10 @@
 
 #pragma once
 
+#include "rawspeedconfig.h"             // for RAWSPEED_READONLY
 #include "common/RawImage.h"            // for RawImage
 #include "decoders/SimpleTiffDecoder.h" // for SimpleTiffDecoder
+#include "io/Buffer.h"                  // for Buffer
 #include "tiff/TiffIFD.h"               // for TiffRootIFD (ptr only), Tiff...
 #include <utility>                      // for move
 
@@ -35,9 +37,9 @@ class DcsDecoder final : public SimpleTiffDecoder {
   void checkImageDimensions() override;
 
 public:
-  static bool __attribute__((pure))
-  isAppropriateDecoder(const TiffRootIFD* rootIFD, const Buffer& file);
-  DcsDecoder(TiffRootIFDOwner&& root, const Buffer& file)
+  static bool RAWSPEED_READONLY isAppropriateDecoder(const TiffRootIFD* rootIFD,
+                                                     Buffer file);
+  DcsDecoder(TiffRootIFDOwner&& root, Buffer file)
       : SimpleTiffDecoder(std::move(root), file) {}
 
   RawImage decodeRawInternal() override;
