@@ -20,6 +20,8 @@
 */
 
 #include "decoders/ErfDecoder.h"
+#include "adt/Point.h"                              // for iPoint2D, iRecta...
+#include "common/Common.h"                          // for BitOrder, BitOrd...
 #include "decoders/RawDecoderException.h"           // for ThrowException
 #include "decompressors/UncompressedDecompressor.h" // for UncompressedDeco...
 #include "io/Buffer.h"                              // for Buffer, DataBuffer
@@ -29,7 +31,7 @@
 #include "tiff/TiffIFD.h"                           // for TiffRootIFD, TiffID
 #include "tiff/TiffTag.h"                           // for TiffTag, TiffTag...
 #include <array>                                    // for array
-#include <memory>                                   // for unique_ptr, allo...
+#include <memory>                                   // for allocator, uniqu...
 #include <string>                                   // for operator==, string
 
 namespace rawspeed {
@@ -37,7 +39,7 @@ namespace rawspeed {
 class CameraMetaData;
 
 bool ErfDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
-                                      [[maybe_unused]] const Buffer& file) {
+                                      [[maybe_unused]] Buffer file) {
   const auto id = rootIFD->getID();
   const std::string& make = id.make;
 
@@ -56,9 +58,11 @@ RawImage ErfDecoder::decodeRawInternal() {
 
   UncompressedDecompressor u(
       ByteStream(DataBuffer(mFile.getSubView(off, c2), Endianness::little)),
-      mRaw);
+      mRaw, iRectangle2D({0, 0}, iPoint2D(width, height)),
+      (12 * width / 8) + ((width + 2) / 10), 12, BitOrder::MSB);
+  mRaw->createData();
 
-  u.decode12BitRaw<Endianness::big, false, true>(width, height);
+  u.decode12BitRawWithControl<Endianness::big>();
 
   return mRaw;
 }
