@@ -40,26 +40,26 @@
 #include <array>                                // for array, array<>::valu...
 #include <cassert>                              // for assert
 #include <cinttypes>                            // for PRIu64
-#include <cmath>                                // for lround, abs
+#include <cmath>                                // for lround
 #include <cstdlib>                              // for abs
 #include <functional>                           // for greater_equal
 #include <iterator>                             // for advance, next, begin
 #include <memory>                               // for unique_ptr
+#include <optional>                             // for optional, nullopt
 #include <string>                               // for operator==, string
 #include <utility>                              // for move
 #include <vector>                               // for vector, allocator
 
 namespace rawspeed {
 
-bool IiqDecoder::isAppropriateDecoder(const Buffer& file) {
+bool IiqDecoder::isAppropriateDecoder(Buffer file) {
   const DataBuffer db(file, Endianness::little);
 
   // The IIQ magic. Is present for all IIQ raws.
   return db.get<uint32_t>(8) == 0x49494949;
 }
 
-bool IiqDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
-                                      const Buffer& file) {
+bool IiqDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD, Buffer file) {
   const auto id = rootIFD->getID();
   const std::string& make = id.make;
 
@@ -69,8 +69,8 @@ bool IiqDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
 
 // FIXME: this is very close to SamsungV0Decompressor::computeStripes()
 std::vector<PhaseOneStrip>
-IiqDecoder::computeSripes(const Buffer& raw_data,
-                          std::vector<IiqOffset> offsets, uint32_t height) {
+IiqDecoder::computeSripes(Buffer raw_data, std::vector<IiqOffset> offsets,
+                          uint32_t height) {
   assert(height > 0);
   assert(offsets.size() == (1 + height));
 
@@ -111,6 +111,40 @@ IiqDecoder::computeSripes(const Buffer& raw_data,
   return slices;
 }
 
+namespace {
+
+enum class IIQFormat {
+  RAW_1,
+  RAW_2,
+  IIQ_L,
+  IIQ_S,
+  IIQ_Sv2,
+  IIQ_L16
+
+};
+
+std::optional<IIQFormat> getAsIIQFormat(uint32_t v) {
+  switch (v) {
+  case 1:
+    return IIQFormat::RAW_1;
+  case 2:
+    return IIQFormat::RAW_2;
+  case 3:
+    return IIQFormat::IIQ_L;
+  case 5:
+    return IIQFormat::IIQ_S;
+  case 6:
+    return IIQFormat::IIQ_Sv2;
+  case 8:
+    return IIQFormat::IIQ_L16;
+  default:
+    return std::nullopt;
+  }
+}
+} // namespace
+
+enum class IiqDecoder::IiqCorr { LUMA, CHROMA };
+
 RawImage IiqDecoder::decodeRawInternal() {
   const Buffer buf(mFile.getSubView(8));
   const DataBuffer db(buf, Endianness::little);
@@ -138,6 +172,7 @@ RawImage IiqDecoder::decodeRawInternal() {
   uint32_t split_row = 0;
   uint32_t split_col = 0;
 
+  std::optional<IIQFormat> format;
   Buffer raw_data;
   ByteStream block_offsets;
   ByteStream wb;
@@ -159,6 +194,13 @@ RawImage IiqDecoder::decodeRawInternal() {
     case 0x109:
       height = data;
       break;
+    case 0x10e: // RawFormat
+      if (format)
+        ThrowRDE("Duplicate RawFormat tag.");
+      format = getAsIIQFormat(data);
+      if (!format || *format != IIQFormat::IIQ_L)
+        ThrowRDE("Unsupported RawFormat: %u", data);
+      break;
     case 0x10f:
       raw_data = bs.getSubView(data, len);
       break;
@@ -170,6 +212,7 @@ RawImage IiqDecoder::decodeRawInternal() {
       block_offsets = bs.getSubStream(data, len);
       break;
     case 0x21d:
+      // 16-bit black level adapted to 14-bit raw data (IIQFormat::IIQ_L)
       black_level = data >> 2;
       break;
     case 0x222:
@@ -188,6 +231,9 @@ RawImage IiqDecoder::decodeRawInternal() {
   if (width == 0 || height == 0 || width > 11976 || height > 8854)
     ThrowRDE("Unexpected image dimensions found: (%u; %u)", width, height);
 
+  if (!format)
+    ThrowRDE("Unspecified RawFormat");
+
   if (split_col > width || split_row > height)
     ThrowRDE("Invalid sensor quadrant split values (%u, %u)", split_row,
              split_col);
@@ -249,6 +295,12 @@ void IiqDecoder::CorrectPhaseOneC(ByteStream meta_data, uint32_t split_row,
       correctSensorDefects(meta_data.getSubStream(offset, len));
       SensorDefectsSeen = true;
       break;
+    case 0x40b: // Chroma calibration
+      PhaseOneFlatField(meta_data.getSubStream(offset, len), IiqCorr::CHROMA);
+      break;
+    case 0x410: // Luma calibration
+      PhaseOneFlatField(meta_data.getSubStream(offset, len), IiqCorr::LUMA);
+      break;
     case 0x431:
       if (QuadrantMultipliersSeen)
         ThrowRDE("Second quadrant multipliers entry seen. Unexpected.");
@@ -341,6 +393,82 @@ void IiqDecoder::CorrectQuadrantMultipliersCombined(ByteStream data,
   }
 }
 
+// Luma and chroma calibration to eliminate remaining paneling artefacts,
+// needed in addition to CorrectQuadrantMultipliersCombined().
+// -- Based on phase_one_flat_field() in dcraw.c by Dave Coffin
+void IiqDecoder::PhaseOneFlatField(ByteStream data, IiqCorr corr) const {
+  const Array2DRef<uint16_t> img(mRaw->getU16DataAsUncroppedArray2DRef());
+
+  int nc;
+  switch (corr) {
+  case IiqCorr::LUMA:
+    nc = 2;
+    break;
+  case IiqCorr::CHROMA:
+    nc = 4;
+    break;
+  default:
+    ThrowRDE("Unsupported IIQ correction");
+  }
+
+  std::array<uint16_t, 8> head;
+  for (int i = 0; i < 8; i++)
+    head[i] = data.getU16();
+
+  if (head[2] == 0 || head[3] == 0 || head[4] == 0 || head[5] == 0)
+    return;
+
+  int wide = roundUpDivision(head[2], head[4]);
+  int high = roundUpDivision(head[3], head[5]);
+
+  std::vector<float> mrow_storage;
+  Array2DRef<float> mrow = Array2DRef<float>::create(
+      mrow_storage, /*width=*/wide * nc, /*height=*/1);
+  mrow = Array2DRef<float>(mrow_storage.data(), /*width=*/nc, /*height=*/wide);
+
+  for (int y = 0; y < high; y++) {
+    for (int x = 0; x < wide; x++) {
+      for (int c = 0; c < nc; c += 2) {
+        float num = data.getU16() / 32768.0;
+        if (y == 0)
+          mrow(x, c) = num;
+        else
+          mrow(x, c + 1) = (num - mrow(x, c)) / head[5];
+      }
+    }
+    if (y == 0)
+      continue;
+    for (int rend = head[1] + y * head[5], row = rend - head[5];
+         row < mRaw->dim.y && row < rend && row < (head[1] + head[3] - head[5]);
+         row++) {
+      for (int x = 1; x < wide; x++) {
+        std::array<float, 4> mult;
+        for (int c = 0; c < nc; c += 2) {
+          mult[c] = mrow(x - 1, c);
+          mult[c + 1] = (mrow(x, c) - mult[c]) / head[4];
+        }
+        for (int cend = head[0] + x * head[4], col = cend - head[4];
+             col < mRaw->dim.x && col < cend &&
+             col < head[0] + head[2] - head[4];
+             col++) {
+          if (int c =
+                  nc > 2 ? static_cast<unsigned>(mRaw->cfa.getColorAt(row, col))
+                         : 0;
+              !(c & 1)) {
+            unsigned val = img(row, col) * mult[c];
+            img(row, col) = std::min(val, 0xFFFFU);
+          }
+          for (int c = 0; c < nc; c += 2)
+            mult[c] += mult[c + 1];
+        }
+      }
+      for (int x = 0; x < wide; x++)
+        for (int c = 0; c < nc; c += 2)
+          mrow(x, c) += mrow(x, c + 1);
+    }
+  }
+}
+
 void IiqDecoder::checkSupportInternal(const CameraMetaData* meta) {
   checkCameraSupported(meta, mRootIFD->getID(), "");
 
