@@ -54,7 +54,7 @@ using std::ostringstream;
 namespace rawspeed {
 
 bool NefDecoder::isAppropriateDecoder(const TiffRootIFD* rootIFD,
-                                      [[maybe_unused]] const Buffer& file) {
+                                      [[maybe_unused]] Buffer file) {
   const auto id = rootIFD->getID();
   const std::string& make = id.make;
 
@@ -259,10 +259,10 @@ void NefDecoder::DecodeUncompressed() const {
   assert(height == offY);
   assert(slices.size() == counts->count);
 
-  mRaw->createData();
-  if (bitPerPixel == 14 && width*slices[0].h*2 == slices[0].count)
+  if (bitPerPixel == 14 && width * slices[0].h * 2 == slices[0].count)
     bitPerPixel = 16; // D3 & D810
 
+  mRaw->createData();
   bitPerPixel = hints.get("real_bpp", bitPerPixel);
 
   switch (bitPerPixel) {
@@ -274,8 +274,6 @@ void NefDecoder::DecodeUncompressed() const {
     ThrowRDE("Invalid bpp found: %u", bitPerPixel);
   }
 
-  bool bitorder = ! hints.has("msb_override");
-
   offY = 0;
   for (const NefSlice& slice : slices) {
     ByteStream in(DataBuffer(mFile.getSubView(slice.offset, slice.count),
@@ -284,19 +282,23 @@ void NefDecoder::DecodeUncompressed() const {
     iPoint2D pos(0, offY);
 
     if (hints.has("coolpixmangled")) {
-      UncompressedDecompressor u(in, mRaw);
-      u.readUncompressedRaw(size, pos, width * bitPerPixel / 8, 12,
-                            BitOrder::MSB32);
+      UncompressedDecompressor u(in, mRaw, iRectangle2D(pos, size),
+                                 width * bitPerPixel / 8, 12, BitOrder::MSB32);
+      u.readUncompressedRaw();
     } else {
-      if (hints.has("coolpixsplit"))
+      if (hints.has("coolpixsplit")) {
         readCoolpixSplitRaw(in, size, pos, width * bitPerPixel / 8);
-      else {
-        UncompressedDecompressor u(in, mRaw);
+      } else {
         if (in.getSize() % size.y != 0)
           ThrowRDE("Inconsistent row size");
         const auto inputPitchBytes = in.getSize() / size.y;
-        u.readUncompressedRaw(size, pos, inputPitchBytes, bitPerPixel,
-                              bitorder ? BitOrder::MSB : BitOrder::LSB);
+        BitOrder bo = (mRootIFD->rootBuffer.getByteOrder() == Endianness::big) ^
+                              hints.has("msb_override")
+                          ? BitOrder::MSB
+                          : BitOrder::LSB;
+        UncompressedDecompressor u(in, mRaw, iRectangle2D(pos, size),
+                                   inputPitchBytes, bitPerPixel, bo);
+        u.readUncompressedRaw();
       }
     }
 
@@ -351,7 +353,6 @@ void NefDecoder::DecodeD100Uncompressed() const {
   uint32_t height = 2024;
 
   mRaw->dim = iPoint2D(width, height);
-  mRaw->createData();
 
   if (ByteStream bs(DataBuffer(mFile.getSubView(offset), Endianness::little));
       bs.getRemainSize() == 0)
@@ -359,9 +360,11 @@ void NefDecoder::DecodeD100Uncompressed() const {
 
   UncompressedDecompressor u(
       ByteStream(DataBuffer(mFile.getSubView(offset), Endianness::little)),
-      mRaw);
+      mRaw, iRectangle2D({0, 0}, iPoint2D(width, height)),
+      (12 * width / 8) + ((width + 2) / 10), 12, BitOrder::MSB);
+  mRaw->createData();
 
-  u.decode12BitRaw<Endianness::big, false, true>(width, height);
+  u.decode12BitRawWithControl<Endianness::big>();
 }
 
 void NefDecoder::DecodeSNefUncompressed() const {
@@ -532,9 +535,9 @@ void NefDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
         uint32_t serialno = 0;
         for (unsigned char c : serial) {
           if (c >= '0' && c <= '9')
-            serialno = serialno*10 + c-'0';
+            serialno = serialno * 10 + c - '0';
           else
-            serialno = serialno*10 + c%10;
+            serialno = serialno * 10 + c % 10;
         }
 
         // Get the decryption key
@@ -597,8 +600,8 @@ void NefDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
   }
 
   if (hints.has("nikon_wb_adjustment")) {
-    mRaw->metadata.wbCoeffs[0] *= 256/527.0;
-    mRaw->metadata.wbCoeffs[2] *= 256/317.0;
+    mRaw->metadata.wbCoeffs[0] *= 256 / 527.0;
+    mRaw->metadata.wbCoeffs[2] *= 256 / 317.0;
   }
 
   auto id = mRootIFD->getID();
@@ -635,12 +638,11 @@ void NefDecoder::decodeMetaDataInternal(const CameraMetaData* meta) {
     mRaw->blackLevel = black;
 }
 
-
-// DecodeNikonYUY2 decodes 12 bit data in an YUY2-like pattern (2 Luma, 1 Chroma per 2 pixels).
-// We un-apply the whitebalance, so output matches lossless.
-// Note that values are scaled. See comment below on details.
-// OPTME: It would be trivial to run this multithreaded.
-void NefDecoder::DecodeNikonSNef(const ByteStream& input) const {
+// DecodeNikonYUY2 decodes 12 bit data in an YUY2-like pattern (2 Luma, 1 Chroma
+// per 2 pixels). We un-apply the whitebalance, so output matches lossless. Note
+// that values are scaled. See comment below on details. OPTME: It would be
+// trivial to run this multithreaded.
+void NefDecoder::DecodeNikonSNef(ByteStream input) const {
   if (mRaw->dim.x < 6)
     ThrowIOE("got a %u wide sNEF, aborting", mRaw->dim.x);
 
@@ -672,7 +674,7 @@ void NefDecoder::DecodeNikonSNef(const ByteStream& input) const {
   auto curve = gammaCurve(1 / 2.4, 12.92, 1, 4095);
 
   // Scale output values to 16 bits.
-  for (int i = 0 ; i < 4096; i++) {
+  for (int i = 0; i < 4096; i++) {
     curve[i] = clampBits(static_cast<int>(curve[i]) << 2, 16);
   }
 
@@ -696,7 +698,7 @@ void NefDecoder::DecodeNikonSNef(const ByteStream& input) const {
       uint32_t g5 = in[4];
       uint32_t g6 = in[5];
 
-      in+=6;
+      in += 6;
       auto y1 = static_cast<float>(g1 | ((g2 & 0x0f) << 8));
       auto y2 = static_cast<float>((g2 >> 4) | (g3 << 4));
       auto cb = static_cast<float>(g4 | ((g5 & 0x0f) << 8));
@@ -704,7 +706,8 @@ void NefDecoder::DecodeNikonSNef(const ByteStream& input) const {
 
       float cb2 = cb;
       float cr2 = cr;
-      // Interpolate right pixel. We assume the sample is aligned with left pixel.
+      // Interpolate right pixel. We assume the sample is aligned with left
+      // pixel.
       if ((col + 6) < out.width) {
         g4 = in[3];
         g5 = in[4];
@@ -746,7 +749,7 @@ void NefDecoder::DecodeNikonSNef(const ByteStream& input) const {
 }
 
 // From:  dcraw.c -- Dave Coffin's raw photo decoder
-#define SQR(x) ((x)*(x))
+#define SQR(x) ((x) * (x))
 std::vector<uint16_t> NefDecoder::gammaCurve(double pwr, double ts, int mode,
                                              int imax) {
   std::vector<uint16_t> curve(65536);
@@ -759,9 +762,9 @@ std::vector<uint16_t> NefDecoder::gammaCurve(double pwr, double ts, int mode,
   g[1] = ts;
   g[2] = g[3] = g[4] = 0;
   bnd[g[1] >= 1] = 1;
-  if (g[1] && (g[1]-1)*(g[0]-1) <= 0) {
-    for (i=0; i < 48; i++) {
-      g[2] = (bnd[0] + bnd[1])/2;
+  if (g[1] && (g[1] - 1) * (g[0] - 1) <= 0) {
+    for (i = 0; i < 48; i++) {
+      g[2] = (bnd[0] + bnd[1]) / 2;
       if (g[0])
         bnd[(pow(g[2] / g[1], -g[0]) - 1) / g[0] - 1 / g[2] > -1] = g[2];
       else
@@ -786,7 +789,7 @@ std::vector<uint16_t> NefDecoder::gammaCurve(double pwr, double ts, int mode,
 
   mode--;
 
-  for (i=0; i < 0x10000; i++) {
+  for (i = 0; i < 0x10000; i++) {
     curve[i] = 0xffff;
     if ((r = static_cast<double>(i) / imax) < 1) {
       curve[i] = static_cast<uint16_t>(
