@@ -23,6 +23,7 @@
 
 #include "common/RawImage.h"              // for RawImage
 #include "decoders/AbstractTiffDecoder.h" // for AbstractTiffDecoder
+#include "io/Buffer.h"                    // for Buffer
 #include "tiff/TiffIFD.h"                 // for TiffRootIFD (ptr only)
 #include <utility>                        // for move
 
@@ -32,21 +33,19 @@ class Buffer;
 class Camera;
 class CameraMetaData;
 
-class RafDecoder final : public AbstractTiffDecoder
-{
+class RafDecoder final : public AbstractTiffDecoder {
   bool alt_layout = false;
 
 public:
-  static bool isAppropriateDecoder(const TiffRootIFD* rootIFD,
-                                   const Buffer& file);
-  RafDecoder(TiffRootIFDOwner&& root, const Buffer& file)
+  static bool isAppropriateDecoder(const TiffRootIFD* rootIFD, Buffer file);
+  RafDecoder(TiffRootIFDOwner&& root, Buffer file)
       : AbstractTiffDecoder(std::move(root), file) {}
 
   RawImage decodeRawInternal() override;
   void applyCorrections(const Camera* cam);
   void decodeMetaDataInternal(const CameraMetaData* meta) override;
   void checkSupportInternal(const CameraMetaData* meta) override;
-  static bool isRAF(const Buffer& input);
+  static bool isRAF(Buffer input);
 
 protected:
   [[nodiscard]] int getDecoderVersion() const override { return 1; }
