@@ -19,11 +19,11 @@
 */
 
 #include "decompressors/Cr2Decompressor.h"     // for Cr2Decompressor
+#include "codes/PrefixCodeDecoder.h"           // for PrefixCodeDecoder
 #include "decompressors/Cr2DecompressorImpl.h" // IWYU pragma: keep
-#include "decompressors/HuffmanTable.h"        // for HuffmanTable
 
 namespace rawspeed {
 
-template class Cr2Decompressor<HuffmanTable>;
+template class Cr2Decompressor<PrefixCodeDecoder<>>;
 
 } // namespace rawspeed
