@@ -22,29 +22,28 @@
 
 #include "decompressors/KodakDecompressor.h"
 #include "adt/Array2DRef.h"               // for Array2DRef
+#include "adt/Invariant.h"                // for invariant
 #include "adt/Point.h"                    // for iPoint2D
+#include "codes/PrefixCodeDecoder.h"      // for PrefixCodeDecoder
 #include "common/Common.h"                // for extractHighBits, isIntN
 #include "common/RawImage.h"              // for RawImage, RawImageData
 #include "decoders/RawDecoderException.h" // for ThrowException, ThrowRDE
-#include "decompressors/HuffmanTable.h"   // for HuffmanTable
 #include "io/ByteStream.h"                // for ByteStream
-#include <algorithm>                      // for min
+#include <algorithm>                      // for min, fill_n
 #include <array>                          // for array
-#include <cassert>                        // for assert
 #include <cstdint>                        // for uint32_t, uint8_t, uint16_t
-#include <utility>                        // for move
 
 namespace rawspeed {
 
 KodakDecompressor::KodakDecompressor(const RawImage& img, ByteStream bs,
                                      int bps_, bool uncorrectedRawValues_)
-    : mRaw(img), input(std::move(bs)), bps(bps_),
+    : mRaw(img), input(bs), bps(bps_),
       uncorrectedRawValues(uncorrectedRawValues_) {
   if (mRaw->getCpp() != 1 || mRaw->getDataType() != RawImageType::UINT16 ||
       mRaw->getBpp() != sizeof(uint16_t))
     ThrowRDE("Unexpected component count / data type");
 
-  if (mRaw->dim.x == 0 || mRaw->dim.y == 0 || mRaw->dim.x % 4 != 0 ||
+  if (!mRaw->dim.hasPositiveArea() || mRaw->dim.x % 4 != 0 ||
       mRaw->dim.x > 4516 || mRaw->dim.y > 3012)
     ThrowRDE("Unexpected image dimensions found: (%u; %u)", mRaw->dim.x,
              mRaw->dim.y);
@@ -59,9 +58,9 @@ KodakDecompressor::KodakDecompressor(const RawImage& img, ByteStream bs,
 
 KodakDecompressor::segment
 KodakDecompressor::decodeSegment(const uint32_t bsize) {
-  assert(bsize > 0);
-  assert(bsize % 4 == 0);
-  assert(bsize <= segment_size);
+  invariant(bsize > 0);
+  invariant(bsize % 4 == 0);
+  invariant(bsize <= segment_size);
 
   segment out;
   static_assert(out.size() == segment_size, "Wrong segment size");
@@ -88,7 +87,7 @@ KodakDecompressor::decodeSegment(const uint32_t bsize) {
   }
   for (uint32_t i = 0; i < bsize; i++) {
     uint32_t len = blen[i];
-    assert(len < 16);
+    invariant(len < 16);
 
     if (bits < len) {
       for (uint32_t j = 0; j < 32; j += 8) {
@@ -103,7 +102,7 @@ KodakDecompressor::decodeSegment(const uint32_t bsize) {
     bitbuf >>= len;
     bits -= len;
 
-    out[i] = len != 0 ? HuffmanTable::extend(diff, len) : int(diff);
+    out[i] = len != 0 ? PrefixCodeDecoder<>::extend(diff, len) : int(diff);
   }
 
   return out;
