@@ -22,6 +22,7 @@
 
 #include "decompressors/AbstractSamsungDecompressor.h" // for AbstractSamsu...
 #include "io/BitPumpMSB.h"                             // for BitPumpMSB
+#include "io/ByteStream.h"                             // for ByteStream
 #include <cstdint>                                     // for int32_t
 #include <vector>                                      // for vector
 
@@ -37,11 +38,11 @@ class SamsungV1Decompressor final : public AbstractSamsungDecompressor {
   static inline int32_t samsungDiff(BitPumpMSB& pump,
                                     const std::vector<encTableItem>& tbl);
 
-  const ByteStream& bs;
+  ByteStream bs;
   static constexpr int bits = 12;
 
 public:
-  SamsungV1Decompressor(const RawImage& image, const ByteStream& bs_, int bit);
+  SamsungV1Decompressor(const RawImage& image, ByteStream bs_, int bit);
 
   void decompress() const;
 };
