@@ -20,7 +20,7 @@
 
 #pragma once
 
-#include "io/BitStream.h" // for BitStreamCacheRightInLeftOut, BitStream
+#include "io/BitStream.h"  // for BitStreamCacheRightInLeftOut, BitStream
 #include "io/Buffer.h"     // for Buffer::size_type
 #include "io/Endianness.h" // for getLE
 #include <cstdint>         // for uint32_t, uint8_t
@@ -35,7 +35,7 @@ struct MSB32BitPumpTag;
 using BitPumpMSB32 = BitStream<MSB32BitPumpTag, BitStreamCacheRightInLeftOut>;
 
 template <> struct BitStreamTraits<MSB32BitPumpTag> final {
-  static constexpr bool canUseWithHuffmanTable = true;
+  static constexpr bool canUseWithPrefixCodeDecoder = true;
 
   // How many bytes can we read from the input per each fillCache(), at most?
   static constexpr int MaxProcessBytes = 4;
