@@ -26,12 +26,11 @@ namespace rawspeed {
 
 class Buffer;
 
-class FileWriter
-{
+class FileWriter {
 public:
   explicit FileWriter(const char* filename);
 
-  void writeFile(const Buffer& fileMap, uint32_t size = 0) const;
+  void writeFile(Buffer fileMap, uint32_t size = 0) const;
   [[nodiscard]] const char* Filename() const { return mFilename; }
   //  void Filename(const char * val) { mFilename = val; }
 
