@@ -37,7 +37,7 @@
 
 namespace rawspeed {
 
-CiffParser::CiffParser(const Buffer& inputData) : RawParser(inputData) {}
+CiffParser::CiffParser(Buffer inputData) : RawParser(inputData) {}
 
 void CiffParser::parseData() {
   ByteStream bs(DataBuffer(mInput, Endianness::little));
