@@ -34,7 +34,7 @@ class FiffParser final : public RawParser {
   TiffRootIFDOwner rootIFD;
 
 public:
-  explicit FiffParser(const Buffer& input);
+  explicit FiffParser(Buffer input);
 
   void parseData();
   std::unique_ptr<RawDecoder>
