@@ -33,8 +33,8 @@ using std::vector;
 
 namespace rawspeed {
 
-CiffEntry::CiffEntry(NORangesSet<Buffer>* valueDatas,
-                     const ByteStream& valueData, ByteStream dirEntry) {
+CiffEntry::CiffEntry(NORangesSet<Buffer>* valueDatas, ByteStream valueData,
+                     ByteStream dirEntry) {
   uint16_t p = dirEntry.getU16();
 
   // NOLINTNEXTLINE cppcoreguidelines-prefer-member-initializer
@@ -70,7 +70,7 @@ CiffEntry::CiffEntry(NORangesSet<Buffer>* valueDatas,
   count = bytesize >> getElementShift();
 }
 
-uint32_t __attribute__((pure)) CiffEntry::getElementShift() const {
+uint32_t RAWSPEED_READONLY CiffEntry::getElementShift() const {
   switch (type) {
   case CiffDataType::SHORT:
     return 1;
@@ -85,7 +85,7 @@ uint32_t __attribute__((pure)) CiffEntry::getElementShift() const {
   }
 }
 
-uint32_t __attribute__((pure)) CiffEntry::getElementSize() const {
+uint32_t RAWSPEED_READONLY CiffEntry::getElementSize() const {
   switch (type) {
   case CiffDataType::BYTE:
   case CiffDataType::ASCII:
@@ -102,7 +102,7 @@ uint32_t __attribute__((pure)) CiffEntry::getElementSize() const {
   }
 }
 
-bool __attribute__((pure)) CiffEntry::isInt() const {
+bool RAWSPEED_READONLY CiffEntry::isInt() const {
   return (type == CiffDataType::LONG || type == CiffDataType::SHORT ||
           type == CiffDataType::BYTE);
 }
@@ -138,7 +138,7 @@ uint8_t CiffEntry::getByte(uint32_t num) const {
   return data.peek<uint8_t>(num);
 }
 
-std::string CiffEntry::getString() const {
+std::string_view CiffEntry::getString() const {
   if (type != CiffDataType::ASCII)
     ThrowCPE("Wrong type 0x%x encountered. Expected Ascii",
              static_cast<unsigned>(type));
@@ -171,7 +171,7 @@ vector<std::string> CiffEntry::getStrings() const {
   return strs;
 }
 
-bool __attribute__((pure)) CiffEntry::isString() const {
+bool RAWSPEED_READONLY CiffEntry::isString() const {
   return (type == CiffDataType::ASCII);
 }
 
