@@ -31,7 +31,8 @@
 #include <cassert>                       // for assert
 #include <map>                           // for map, operator!=, _Rb_tree_c...
 #include <memory>                        // for unique_ptr, make_unique
-#include <string>                        // for operator==, string
+#include <string>                        // for string, basic_string<>::__s...
+#include <string_view>                   // for operator==
 #include <utility>                       // for move, pair
 #include <vector>                        // for vector, vector<>::const_ite...
 
@@ -40,8 +41,7 @@ using std::vector;
 namespace rawspeed {
 
 void CiffIFD::parseIFDEntry(NORangesSet<Buffer>* valueDatas,
-                            const ByteStream& valueData,
-                            ByteStream& dirEntries) {
+                            ByteStream valueData, ByteStream& dirEntries) {
   assert(valueDatas);
 
   ByteStream dirEntry = dirEntries.getStream(10); // Entry is 10 bytes.
@@ -225,13 +225,13 @@ CiffIFD::getIFDsWithTagWhere(CiffTag tag, const std::string& isValue) const {
   });
 }
 
-bool __attribute__((pure)) CiffIFD::hasEntry(CiffTag tag) const {
+bool RAWSPEED_READONLY CiffIFD::hasEntry(CiffTag tag) const {
   assert(isIn(tag, CiffTagsWeCareAbout));
 
   return mEntry.count(tag) > 0;
 }
 
-bool __attribute__((pure)) CiffIFD::hasEntryRecursive(CiffTag tag) const {
+bool RAWSPEED_READONLY CiffIFD::hasEntryRecursive(CiffTag tag) const {
   assert(isIn(tag, CiffTagsWeCareAbout));
 
   if (mEntry.count(tag) > 0)
