@@ -18,17 +18,23 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include "RawSpeed-API.h"        // for RawDecoder, Buffer, FileReader, HAV...
-#include "common/ChecksumFile.h" // for ChecksumFileEntry, ReadChecksumFile
-#include <algorithm>             // for max
-#include <benchmark/benchmark.h> // for Counter, Counter::Flags, Counter::k...
-#include <chrono>                // for duration, high_resolution_clock
-#include <ctime>                 // for clock, clock_t, CLOCKS_PER_SEC
-#include <memory>                // for unique_ptr, allocator
-#include <ratio>                 // for ratio
-#include <string>                // for string, operator!=, to_string
-#include <utility>               // for move
-#include <vector>                // for vector
+#include "RawSpeed-API.h"                    // for Buffer, RawDecoder, Fil...
+#include "adt/AlignedAllocator.h"            // for AlignedAllocator
+#include "adt/DefaultInitAllocatorAdaptor.h" // for DefaultInitAllocatorAda...
+#include "common/ChecksumFile.h"             // for ChecksumFileEntry, Read...
+#include <algorithm>                         // for max
+#include <chrono>                            // for duration, high_resoluti...
+#include <cstdint>                           // for uint8_t
+#include <ctime>                             // for clock, clock_t, CLOCKS_...
+#include <memory>                            // for unique_ptr, allocator
+#include <ratio>                             // for ratio
+#include <string>                            // for string, to_string, char...
+#include <string_view>                       // for operator!=, string_view
+#include <tuple>                             // for tie, tuple
+#include <type_traits>                       // for __type_identity_t
+#include <utility>                           // for move
+#include <vector>                            // for vector
+#include <benchmark/benchmark.h>             // for Counter, Counter::Flags
 
 #ifdef HAVE_OPENMP
 #include <omp.h> // for omp_get_max_threads
@@ -94,20 +100,25 @@ template <typename Clock, typename period = std::ratio<1, 1>> struct Timer {
 // requested the first time.
 struct Entry {
   rawspeed::ChecksumFileEntry Name;
-  std::unique_ptr<const rawspeed::Buffer> Content;
+  std::unique_ptr<std::vector<
+      uint8_t, rawspeed::DefaultInitAllocatorAdaptor<
+                   uint8_t, rawspeed::AlignedAllocator<uint8_t, 16>>>>
+      Storage;
+  rawspeed::Buffer Content;
 
   const rawspeed::Buffer& getFileContents() {
-    if (Content)
-      return *Content;
+    if (Storage)
+      return Content;
 
-    Content = FileReader(Name.FullFileName.c_str()).readFile();
-    return *Content;
+    std::tie(Storage, Content) =
+        FileReader(Name.FullFileName.c_str()).readFile();
+    return Content;
   }
 };
 
 static int currThreadCount;
 
-extern "C" int __attribute__((pure)) rawspeed_get_number_of_processor_cores() {
+extern "C" int RAWSPEED_READONLY rawspeed_get_number_of_processor_cores() {
   return currThreadCount;
 }
 
@@ -174,8 +185,7 @@ static inline void BM_RawSpeed(benchmark::State& state, Entry* entry,
 static void addBench(Entry* entry, std::string tName, int threads) {
   tName += std::to_string(threads);
 
-  auto* b =
-      benchmark::RegisterBenchmark(tName.c_str(), &BM_RawSpeed, entry, threads);
+  auto* b = benchmark::RegisterBenchmark(tName, &BM_RawSpeed, entry, threads);
   b->Unit(benchmark::kMillisecond);
   b->UseRealTime();
   b->MeasureProcessCPUTime();
