@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2012-2021 darktable developers.
+    Copyright (C) 2012-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,9 +53,9 @@
 #include "bauhaus/bauhaus.h"
 #include "common/darktable.h"
 #include "common/exif.h"
-#include "common/imageio.h"
-#include "common/imageio_module.h"
 #include "control/conf.h"
+#include "imageio/imageio_common.h"
+#include "imageio/imageio_module.h"
 #include "imageio/format/imageio_format_api.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -220,10 +220,10 @@ static void cinema_setup_encoder(opj_cparameters_t *parameters, opj_image_t *ima
       }
       if(!((image->comps[0].w == 2048) | (image->comps[0].h == 1080)))
       {
-        fprintf(stdout,
-                "Image coordinates %d x %d is not 2K compliant.\nJPEG Digital Cinema Profile-3 "
-                "(2K profile) compliance requires that at least one of coordinates match 2048 x 1080\n",
-                image->comps[0].w, image->comps[0].h);
+        dt_print(DT_DEBUG_ALWAYS,
+                 "Image coordinates %d x %d is not 2K compliant.\nJPEG Digital Cinema Profile-3 "
+                 "(2K profile) compliance requires that at least one of coordinates match 2048 x 1080\n",
+                 image->comps[0].w, image->comps[0].h);
         parameters->cp_rsiz = OPJ_STD_RSIZ;
       }
       break;
@@ -240,10 +240,10 @@ static void cinema_setup_encoder(opj_cparameters_t *parameters, opj_image_t *ima
       }
       if(!((image->comps[0].w == 4096) | (image->comps[0].h == 2160)))
       {
-        fprintf(stdout,
-                "Image coordinates %d x %d is not 4K compliant.\nJPEG Digital Cinema Profile-4"
-                "(4K profile) compliance requires that at least one of coordinates match 4096 x 2160\n",
-                image->comps[0].w, image->comps[0].h);
+        dt_print(DT_DEBUG_ALWAYS,
+                 "Image coordinates %d x %d is not 4K compliant.\nJPEG Digital Cinema Profile-4 "
+                 "(4K profile) compliance requires that at least one of coordinates match 4096 x 2160\n",
+                 image->comps[0].w, image->comps[0].h);
         parameters->cp_rsiz = OPJ_STD_RSIZ;
       }
       parameters->numpocs = initialise_4K_poc(parameters->POC, parameters->numresolution);
@@ -320,7 +320,7 @@ static void cinema_setup_encoder(opj_cparameters_t *parameters, opj_image_t *ima
 
 int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const void *in_tmp,
                 dt_colorspaces_color_profile_type_t over_type, const char *over_filename,
-                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
+                void *exif, int exif_len, dt_imgid_t imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
                 const gboolean export_masks)
 {
   int rc = 1;
@@ -357,7 +357,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
   }
 
   /* Create comment for codestream */
-  parameters.cp_comment = g_strdup_printf("Created by %s", darktable_package_string);
+  parameters.cp_comment = g_strdup_printf("Created with %s", darktable_package_string);
 
   /*Converting the image to a format suitable for encoding*/
   {
@@ -382,7 +382,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
     image = opj_image_create(numcomps, &cmptparm[0], OPJ_CLRSPC_SRGB);
     if(!image)
     {
-      fprintf(stderr, "Error: opj_image_create() failed\n");
+      dt_print(DT_DEBUG_ALWAYS, "Error: opj_image_create() failed\n");
       free(rates);
       rc = 0;
       goto exit;
@@ -417,7 +417,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
 //        }
 //        break;
 //      default:
-//        fprintf(stderr, "Error: this shouldn't happen, there is no bit depth of %d for jpeg 2000 images.\n",
+//        dt_print(DT_DEBUG_ALWAYS, "Error: this shouldn't happen, there is no bit depth of %d for jpeg 2000 images.\n",
 //                prec);
 //        free(rates);
 //        opj_image_destroy(image);
@@ -465,7 +465,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
   {
     opj_destroy_codec(ccodec);
     opj_image_destroy(image);
-    fprintf(stderr, "failed to create output stream\n");
+    dt_print(DT_DEBUG_ALWAYS, "failed to create output stream\n");
     rc = 0;
     goto exit;
   }
@@ -475,7 +475,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
     opj_stream_destroy(cstream);
     opj_destroy_codec(ccodec);
     opj_image_destroy(image);
-    fprintf(stderr, "failed to encode image: opj_start_compress\n");
+    dt_print(DT_DEBUG_ALWAYS, "failed to encode image: opj_start_compress\n");
     rc = 0;
     goto exit;
   }
@@ -486,7 +486,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
     opj_stream_destroy(cstream);
     opj_destroy_codec(ccodec);
     opj_image_destroy(image);
-    fprintf(stderr, "failed to encode image: opj_encode\n");
+    dt_print(DT_DEBUG_ALWAYS, "failed to encode image: opj_encode\n");
     rc = 0;
     goto exit;
   }
@@ -497,7 +497,7 @@ int write_image(dt_imageio_module_data_t *j2k_tmp, const char *filename, const v
     opj_stream_destroy(cstream);
     opj_destroy_codec(ccodec);
     opj_image_destroy(image);
-    fprintf(stderr, "failed to encode image: opj_end_compress\n");
+    dt_print(DT_DEBUG_ALWAYS, "failed to encode image: opj_end_compress\n");
     rc = 0;
     goto exit;
   }
