@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2010-2021 darktable developers.
+    Copyright (C) 2010-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,9 @@
 #include "common/colorspaces.h"
 #include "common/darktable.h"
 #include "common/exif.h"
-#include "common/imageio.h"
-#include "common/imageio_module.h"
 #include "control/conf.h"
+#include "imageio/imageio_common.h"
+#include "imageio/imageio_module.h"
 #include "imageio/format/imageio_format_api.h"
 #include <inttypes.h>
 #include <setjmp.h>
@@ -56,7 +56,6 @@ typedef struct dt_imageio_jpeg_gui_data_t
   GtkWidget *quality;
 } dt_imageio_jpeg_gui_data_t;
 
-
 // error functions
 struct dt_imageio_jpeg_error_mgr
 {
@@ -151,149 +150,6 @@ static void write_icc_profile(j_compress_ptr cinfo, const JOCTET *icc_data_ptr,
 }
 
 
-#if 0
-/*
- * Prepare for reading an ICC profile
- */
-
-void
-setup_read_icc_profile (j_decompress_ptr cinfo)
-{
-  /* Tell the library to keep any APP2 data it may find */
-  jpeg_save_markers(cinfo, ICC_MARKER, 0xFFFF);
-}
-
-
-/*
- * Handy subroutine to test whether a saved marker is an ICC profile marker.
- */
-
-static boolean
-marker_is_icc (jpeg_saved_marker_ptr marker)
-{
-  return
-    marker->marker == ICC_MARKER &&
-    marker->data_length >= ICC_OVERHEAD_LEN &&
-    /* verify the identifying string */
-    GETJOCTET(marker->data[0]) == 0x49 &&
-    GETJOCTET(marker->data[1]) == 0x43 &&
-    GETJOCTET(marker->data[2]) == 0x43 &&
-    GETJOCTET(marker->data[3]) == 0x5F &&
-    GETJOCTET(marker->data[4]) == 0x50 &&
-    GETJOCTET(marker->data[5]) == 0x52 &&
-    GETJOCTET(marker->data[6]) == 0x4F &&
-    GETJOCTET(marker->data[7]) == 0x46 &&
-    GETJOCTET(marker->data[8]) == 0x49 &&
-    GETJOCTET(marker->data[9]) == 0x4C &&
-    GETJOCTET(marker->data[10]) == 0x45 &&
-    GETJOCTET(marker->data[11]) == 0x0;
-}
-
-
-/*
- * See if there was an ICC profile in the JPEG file being read;
- * if so, reassemble and return the profile data.
- *
- * TRUE is returned if an ICC profile was found, FALSE if not.
- * If TRUE is returned, *icc_data_ptr is set to point to the
- * returned data, and *icc_data_len is set to its length.
- *
- * IMPORTANT: the data at **icc_data_ptr has been allocated with malloc()
- * and must be freed by the caller with free() when the caller no longer
- * needs it.  (Alternatively, we could write this routine to use the
- * IJG library's memory allocator, so that the data would be freed implicitly
- * at jpeg_finish_decompress() time.  But it seems likely that many apps
- * will prefer to have the data stick around after decompression finishes.)
- *
- * NOTE: if the file contains invalid ICC APP2 markers, we just silently
- * return FALSE.  You might want to issue an error message instead.
- */
-
-boolean
-read_icc_profile (j_decompress_ptr cinfo,
-                  JOCTET **icc_data_ptr,
-                  unsigned int *icc_data_len)
-{
-  int num_markers = 0;
-#define MAX_SEQ_NO 255 /* sufficient since marker numbers are bytes */
-  char marker_present[MAX_SEQ_NO+1];    /* 1 if marker found */
-  unsigned int data_length[MAX_SEQ_NO+1]; /* size of profile data in marker */
-  unsigned int data_offset[MAX_SEQ_NO+1]; /* offset for data in marker */
-
-  *icc_data_ptr = NULL;   /* avoid confusion if FALSE return */
-  *icc_data_len = 0;
-
-  /* This first pass over the saved markers discovers whether there are
-   * any ICC markers and verifies the consistency of the marker numbering.
-   */
-
-  for(int seq_no = 1; seq_no <= MAX_SEQ_NO; seq_no++)
-    marker_present[seq_no] = 0;
-
-  for(jpeg_saved_marker_ptr marker = cinfo->marker_list; marker != NULL; marker = marker->next)
-  {
-    if(marker_is_icc(marker))
-    {
-      if(num_markers == 0)
-        num_markers = GETJOCTET(marker->data[13]);
-      else if(num_markers != GETJOCTET(marker->data[13]))
-        return FALSE;   /* inconsistent num_markers fields */
-      const int seq_no = GETJOCTET(marker->data[12]);
-      if(seq_no <= 0 || seq_no > num_markers)
-        return FALSE;   /* bogus sequence number */
-      if(marker_present[seq_no])
-        return FALSE;   /* duplicate sequence numbers */
-      marker_present[seq_no] = 1;
-      data_length[seq_no] = marker->data_length - ICC_OVERHEAD_LEN;
-    }
-  }
-
-  if(num_markers == 0)
-    return FALSE;
-
-  /* Check for missing markers, count total space needed,
-   * compute offset of each marker's part of the data.
-   */
-
-  unsigned int total_length = 0;
-  for(int seq_no = 1; seq_no <= num_markers; seq_no++)
-  {
-    if(marker_present[seq_no] == 0)
-      return FALSE;   /* missing sequence number */
-    data_offset[seq_no] = total_length;
-    total_length += data_length[seq_no];
-  }
-
-  if(total_length <= 0)
-    return FALSE;   /* found only empty markers? */
-
-  /* Allocate space for assembled data */
-  JOCTET *icc_data = (JOCTET *)calloc(total_length, sizeof(JOCTET));
-  if(icc_data == NULL)
-    return FALSE;   /* oops, out of memory */
-
-  /* and fill it in */
-  for(jpeg_saved_marker_ptr marker = cinfo->marker_list; marker != NULL; marker = marker->next)
-  {
-    if(marker_is_icc(marker))
-    {
-      const int seq_no = GETJOCTET(marker->data[12]);
-      JOCTET *dst_ptr = icc_data + data_offset[seq_no];
-      JOCTET FAR *src_ptr = marker->data + ICC_OVERHEAD_LEN;
-      unsigned int length = data_length[seq_no];
-      while(length--)
-      {
-        *dst_ptr++ = *src_ptr++;
-      }
-    }
-  }
-
-  *icc_data_ptr = icc_data;
-  *icc_data_len = total_length;
-
-  return TRUE;
-}
-#endif
 #undef ICC_MARKER
 #undef ICC_OVERHEAD_LEN
 #undef MAX_BYTES_IN_MARKER
@@ -303,7 +159,7 @@ read_icc_profile (j_decompress_ptr cinfo,
 
 int write_image(dt_imageio_module_data_t *jpg_tmp, const char *filename, const void *in_tmp,
                 dt_colorspaces_color_profile_type_t over_type, const char *over_filename,
-                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
+                void *exif, int exif_len, dt_imgid_t imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
                 const gboolean export_masks)
 {
   dt_imageio_jpeg_t *jpg = (dt_imageio_jpeg_t *)jpg_tmp;
