@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2010-2021 darktable developers.
+    Copyright (C) 2010-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,11 +29,13 @@
 #include "bauhaus/bauhaus.h"
 #include "common/colorspaces.h"
 #include "common/darktable.h"
-#include "common/imageio.h"
-#include "common/imageio_module.h"
 #include "control/conf.h"
+#include "imageio/imageio_common.h"
+#include "imageio/imageio_module.h"
 #include "imageio/format/imageio_format_api.h"
 
+#include <exiv2/exv_conf.h>
+
 DT_MODULE(3)
 
 typedef struct dt_imageio_png_t
@@ -62,6 +64,7 @@ typedef struct dt_imageio_png_gui_t
  * for making useful code much more readable and discoverable ;)
  */
 
+#if !defined(PNG_eXIf_SUPPORTED) || (EXIV2_MAJOR_VERSION < 1 && EXIV2_MINOR_VERSION <= 27)
 static void PNGwriteRawProfile(png_struct *ping, png_info *ping_info, char *profile_type, guint8 *profile_data,
                                png_uint_32 length)
 {
@@ -116,10 +119,11 @@ static void PNGwriteRawProfile(png_struct *ping, png_info *ping_info, char *prof
   png_free(ping, text[0].key);
   png_free(ping, text);
 }
+#endif
 
 int write_image(dt_imageio_module_data_t *p_tmp, const char *filename, const void *ivoid,
                 dt_colorspaces_color_profile_type_t over_type, const char *over_filename,
-                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
+                void *exif, int exif_len, dt_imgid_t imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
                 const gboolean export_masks)
 {
   dt_imageio_png_t *p = (dt_imageio_png_t *)p_tmp;
@@ -193,6 +197,9 @@ int write_image(dt_imageio_module_data_t *p_tmp, const char *filename, const voi
   // write exif data
   if(exif && exif_len > 0)
   {
+#if defined(PNG_eXIf_SUPPORTED) && (EXIV2_MAJOR_VERSION >= 1 || EXIV2_MINOR_VERSION > 27)
+    png_set_eXIf_1(png_ptr, info_ptr, (uint32_t)exif_len, (png_bytep)exif);
+#else
     /* The legacy tEXt chunk storage scheme implies the "Exif\0\0" APP1 prefix */
     uint8_t *buf = malloc(exif_len + 6);
     if(buf)
@@ -202,6 +209,7 @@ int write_image(dt_imageio_module_data_t *p_tmp, const char *filename, const voi
       PNGwriteRawProfile(png_ptr, info_ptr, "exif", buf, exif_len + 6);
       free(buf);
     }
+#endif
   }
 
   png_write_info(png_ptr, info_ptr);
@@ -313,24 +321,6 @@ static int __attribute__((__unused__)) read_header(const char *filename, dt_imag
 #undef NUM_BYTES_CHECK
 }
 
-#if 0
-int dt_imageio_png_read_assure_8(dt_imageio_png_t *png)
-{
-  if(setjmp(png_jmpbuf(png->png_ptr)))
-  {
-    fclose(png->f);
-    png_destroy_read_struct(&png->png_ptr, NULL, NULL);
-    return 1;
-  }
-  uint32_t bit_depth = png_get_bit_depth(png->png_ptr, png->info_ptr);
-  // strip down to 8 bit channels
-  if(bit_depth == 16)
-    png_set_strip_16(png->png_ptr);
-
-  return 0;
-}
-#endif
-
 int read_image(dt_imageio_module_data_t *p_tmp, uint8_t *out)
 {
   dt_imageio_png_t *png = (dt_imageio_png_t *)p_tmp;
