@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2013-2022 darktable developers.
+    Copyright (C) 2013-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@
 #include "bauhaus/bauhaus.h"
 #include "common/darktable.h"
 #include "common/exif.h"
-#include "common/imageio.h"
-#include "common/imageio_module.h"
 #include "control/conf.h"
+#include "imageio/imageio_common.h"
+#include "imageio/imageio_module.h"
 #include "imageio/format/imageio_format_api.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -117,7 +117,7 @@ void cleanup(dt_imageio_module_format_t *self)
 
 int write_image(dt_imageio_module_data_t *webp, const char *filename, const void *in_tmp,
                 dt_colorspaces_color_profile_type_t over_type, const char *over_filename,
-                void *exif, int exif_len, int imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
+                void *exif, int exif_len, dt_imgid_t imgid, int num, int total, struct dt_dev_pixelpipe_t *pipe,
                 const gboolean export_masks)
 {
   int res = 1;
@@ -152,7 +152,7 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
   config.partition_limit = 70;
   if(!WebPValidateConfig(&config))
   {
-    fprintf(stderr, "[webp export] error validating encoder configuration\n");
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error validating encoder configuration\n");
     goto error;
   }
 
@@ -171,13 +171,13 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
       err = WebPMuxSetChunk(mux, "ICCP", &icc_profile, 0);
       if(err != WEBP_MUX_OK)
       {
-        fprintf(stderr, "[webp export] error adding ICC profile to WebP stream\n");
+        dt_print(DT_DEBUG_ALWAYS, "[webp export] error adding ICC profile to WebP stream\n");
         goto error;
       }
     }
     else
     {
-      fprintf(stderr, "[webp export] error allocating ICC profile buffer\n");
+      dt_print(DT_DEBUG_ALWAYS, "[webp export] error allocating ICC profile buffer\n");
       goto error;
     }
   }
@@ -203,7 +203,7 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
 
   if(!WebPEncode(&config, &pic))
   {
-    fprintf(stderr, "[webp export] error (%d) during encoding: %s\n", pic.error_code,
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error (%d) during encoding: %s\n", pic.error_code,
             get_error_str(pic.error_code));
     goto error;
   }
@@ -213,7 +213,7 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
   err = WebPMuxSetImage(mux, &bitstream, 0);
   if(err != WEBP_MUX_OK)
   {
-    fprintf(stderr, "[webp export] error adding image to WebP stream\n");
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error adding image to WebP stream\n");
     goto error;
   }
 
@@ -221,19 +221,19 @@ int write_image(dt_imageio_module_data_t *webp, const char *filename, const void
   err = WebPMuxAssemble(mux, &assembled_data);
   if(err != WEBP_MUX_OK)
   {
-    fprintf(stderr, "[webp export] error assembling the WebP file\n");
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error assembling the WebP file\n");
     goto error;
   }
 
   out = g_fopen(filename, "w+b");
   if(!out)
   {
-    fprintf(stderr, "[webp export] error creating file %s\n", filename);
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error creating file %s\n", filename);
     goto error;
   }
   if(fwrite(assembled_data.bytes, assembled_data.size, 1, out) != 1)
   {
-    fprintf(stderr, "[webp export] error writing %zu bytes to file %s\n", assembled_data.size, filename);
+    dt_print(DT_DEBUG_ALWAYS, "[webp export] error writing %zu bytes to file %s\n", assembled_data.size, filename);
     goto error;
   }
 
