@@ -1,6 +1,6 @@
 /*
   This file is part of darktable,
-  Copyright (C) 2020 darktable developers.
+  Copyright (C) 2020-2023 darktable developers.
 
   darktable is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -138,6 +138,14 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
                                          ivoid, ovoid, roi_in, roi_out))
     return; // image has been copied through to output and module's trouble flag has been updated
 
+  float *restrict temp;
+  if(!dt_iop_alloc_image_buffers(self,roi_in,roi_out,
+                                 4 | DT_IMGSZ_INPUT, &temp,
+                                 0, NULL))
+  {
+    dt_iop_copy_image_roi(ovoid, ivoid, piece->colors, roi_in, roi_out, 0);
+    return;
+  }
   dt_iop_censorize_data_t *data = (dt_iop_censorize_data_t *)piece->data;
   const float *const restrict in = DT_IS_ALIGNED((const float *const restrict)ivoid);
   float *const restrict out = DT_IS_ALIGNED((float *const restrict)ovoid);
@@ -146,8 +154,6 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   const int height = roi_in->height;
   const int ch = 4;
 
-  float *const restrict temp = dt_alloc_align_float((size_t)width * height * ch);
-
   const float sigma_1 = data->radius_1 * roi_in->scale / piece->iscale;
   const float sigma_2 = data->radius_2 * roi_in->scale / piece->iscale;
   const size_t pixel_radius = data->pixelate * roi_in->scale / piece->iscale;
@@ -159,7 +165,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   dt_aligned_pixel_t RGBmax, RGBmin;
   for(int k = 0; k < 4; k++)
   {
-    RGBmax[k] = INFINITY;
+    RGBmax[k] = FLT_MAX;
     RGBmin[k] = 0.f;
   }
 
@@ -250,9 +256,6 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   if(noise != 0.f)
     make_noise(output, noise, width, height);
 
-  if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK)
-    dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
-
   dt_free_align(temp);
 }
 
@@ -292,8 +295,8 @@ int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_m
 
   if(unbound)
   {
-    for(int k = 0; k < 4; k++) RGBmax[k] = INFINITY;
-    for(int k = 0; k < 4; k++) RGBmin[k] = -INFINITY;
+    for(int k = 0; k < 4; k++) RGBmax[k] = FLT_MAX;
+    for(int k = 0; k < 4; k++) RGBmin[k] = -FLT_MAX;
   }
 
   if(d->lowpass_algo == LOWPASS_ALGO_GAUSSIAN)
