@@ -770,7 +770,7 @@ void process(struct dt_iop_module_t *self,
   // figure out the number of pixels each thread needs to process
   // round up to a multiple of 4 pixels so that each chunk starts aligned(64)
   const size_t nthreads = dt_get_num_threads();
-  const size_t chunksize = 4 * (((npixels / nthreads) + 3) / 4);
+  const size_t chunksize = 4 * ((((npixels + nthreads -1) / nthreads) + 3) / 4);
 #pragma omp parallel for simd default(none)                             \
   dt_omp_firstprivate(in, out, mode, npixels, nthreads, chunksize, \
                       grey, saturation, saturation_out, lift, lift_sop, \
