@@ -503,7 +503,7 @@ static void _transform_lcms(const dt_iop_colorout_data_t *const d,
   // figure out the number of pixels each thread needs to process
   // round up to a multiple of 4 pixels so that each chunk starts aligned(64)
   const size_t nthreads = dt_get_num_threads();
-  const size_t chunksize = 4 * (((npixels / nthreads) + 3) / 4);
+  const size_t chunksize = 4 * ((((npixels + nthreads - 1) / nthreads) + 3) / 4);
 #pragma omp parallel for default(none)                                  \
     dt_omp_firstprivate(in, out, npixels, chunksize, nthreads, d, gamutcheck) \
     schedule(static)
