@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2009-2021 darktable developers.
+    Copyright (C) 2009-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -24,9 +25,6 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(__SSE__)
-#include <xmmintrin.h>
-#endif
 
 #include "bauhaus/bauhaus.h"
 #include "common/histogram.h"
@@ -117,6 +115,7 @@ typedef struct dt_iop_exposure_global_data_t
   int kernel_exposure;
 } dt_iop_exposure_global_data_t;
 
+#define EXPOSURE_CORRECTION_UNDEFINED (-FLT_MAX)
 
 const char *name()
 {
@@ -125,13 +124,14 @@ const char *name()
 
 const char** description(struct dt_iop_module_t *self)
 {
-  return dt_iop_set_description(self,
-                                _("redo the exposure of the shot as if you were still in-camera\n"
-                                  "using a color-safe brightening similar to increasing ISO setting"),
-                                _("corrective and creative"),
-                                _("linear, RGB, scene-referred"),
-                                _("linear, RGB"),
-                                _("linear, RGB, scene-referred"));
+  return dt_iop_set_description
+    (self,
+     _("redo the exposure of the shot as if you were still in-camera\n"
+       "using a color-safe brightening similar to increasing ISO setting"),
+     _("corrective and creative"),
+     _("linear, RGB, scene-referred"),
+     _("linear, RGB"),
+     _("linear, RGB, scene-referred"));
 }
 
 int default_group()
@@ -144,20 +144,32 @@ int flags()
   return IOP_FLAGS_ALLOW_TILING | IOP_FLAGS_SUPPORTS_BLENDING;
 }
 
-int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+int default_colorspace(dt_iop_module_t *self,
+                       dt_dev_pixelpipe_t *pipe,
+                       dt_dev_pixelpipe_iop_t *piece)
 {
   return IOP_CS_RGB;
 }
 
-static void _exposure_proxy_set_exposure(struct dt_iop_module_t *self, const float exposure);
+static void _exposure_proxy_set_exposure(struct dt_iop_module_t *self,
+                                         const float exposure);
+
 static float _exposure_proxy_get_exposure(struct dt_iop_module_t *self);
-static void _exposure_proxy_set_black(struct dt_iop_module_t *self, const float black);
+
+static void _exposure_proxy_set_black(struct dt_iop_module_t *self,
+                                      const float black);
+
 static float _exposure_proxy_get_black(struct dt_iop_module_t *self);
 static void _paint_hue(dt_iop_module_t *self);
-static void _exposure_set_black(struct dt_iop_module_t *self, const float black);
 
-int legacy_params(dt_iop_module_t *self, const void *const old_params, const int old_version,
-                  void *new_params, const int new_version)
+static void _exposure_set_black(struct dt_iop_module_t *self,
+                                const float black);
+
+int legacy_params(dt_iop_module_t *self,
+                  const void *const old_params,
+                  const int old_version,
+                  void *new_params,
+                  const int new_version)
 {
   if(old_version == 2 && new_version == 6)
   {
@@ -259,46 +271,80 @@ int legacy_params(dt_iop_module_t *self, const void *const old_params, const int
   return 1;
 }
 
-void init_presets (dt_iop_module_so_t *self)
+void init_presets(dt_iop_module_so_t *self)
 {
-  dt_gui_presets_add_generic(_("magic lantern defaults"), self->op,
-                             self->version(),
-                             &(dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_DEFLICKER,
-                                                         .black = 0.0f,
-                                                         .exposure = 0.0f,
-                                                         .deflicker_percentile = 50.0f,
-                                                         .deflicker_target_level = -4.0f,
-                                                         .compensate_exposure_bias = FALSE},
-                             sizeof(dt_iop_exposure_params_t), 1, DEVELOP_BLEND_CS_RGB_DISPLAY);
+  self->pref_based_presets = TRUE;
 
+  dt_gui_presets_add_generic
+    (_("magic lantern defaults"), self->op,
+     self->version(),
+     &(dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_DEFLICKER,
+                                 .black = 0.0f,
+                                 .exposure = 0.0f,
+                                 .deflicker_percentile = 50.0f,
+                                 .deflicker_target_level = -4.0f,
+                                 .compensate_exposure_bias = FALSE},
+     sizeof(dt_iop_exposure_params_t), 1, DEVELOP_BLEND_CS_RGB_DISPLAY);
 
-  // For scene-referred workflow, since filmic doesn't brighten as base curve does,
-  // we need an initial exposure boost. This might be too much in some cases but…
-  // (the preset name is used in develop.c)
-  dt_gui_presets_add_generic(_("scene-referred default"), self->op, self->version(),
-                             &(dt_iop_exposure_params_t){.mode = EXPOSURE_MODE_MANUAL,
-                                                         .black = -0.000244140625f,
-                                                         .exposure = 0.7f,
-                                                         .deflicker_percentile = 50.0f,
-                                                         .deflicker_target_level = -4.0f,
-                                                         .compensate_exposure_bias = TRUE},
-                             sizeof(dt_iop_exposure_params_t), 1, DEVELOP_BLEND_CS_RGB_SCENE);
+  const gboolean is_scene_referred = dt_is_scene_referred();
 
-  dt_gui_presets_update_ldr(_("scene-referred default"), self->op,
-                            self->version(), FOR_RAW);
+  if(is_scene_referred)
+  {
+    // For scene-referred workflow, since filmic doesn't brighten as base curve does,
+    // we need an initial exposure boost. This preset has the same value as what is
+    // auto-applied (see reload_default below) for scene-referred workflow.
+    dt_gui_presets_add_generic
+      (_("scene-referred default"), self->op, self->version(),
+       NULL, 0,
+       1, DEVELOP_BLEND_CS_RGB_SCENE);
+
+    dt_gui_presets_update_ldr(_("scene-referred default"), self->op,
+                              self->version(), FOR_RAW);
+
+    dt_gui_presets_update_autoapply(_("scene-referred default"),
+                                    self->op, self->version(), TRUE);
+  }
 }
 
-static void _deflicker_prepare_histogram(dt_iop_module_t *self, uint32_t **histogram,
+void reload_defaults(dt_iop_module_t *module)
+{
+  dt_iop_exposure_params_t *d = module->default_params;
+
+  const gboolean scene_raw =
+     dt_image_is_rawprepare_supported(&module->dev->image_storage)
+     && dt_is_scene_referred();
+
+  d->mode = EXPOSURE_MODE_MANUAL;
+
+  if(scene_raw && module->multi_priority == 0)
+  {
+    const gboolean mono = dt_image_is_monochrome(&module->dev->image_storage);
+    d->exposure = mono ? 0.0f : 0.7f;
+    d->black =    mono ? 0.0f : -0.000244140625f;
+    d->compensate_exposure_bias = TRUE;
+  }
+  else
+  {
+    d->exposure = 0.0f;
+    d->black = 0.0f;
+    d->compensate_exposure_bias = FALSE;
+  }
+}
+
+static void _deflicker_prepare_histogram(dt_iop_module_t *self,
+                                         uint32_t **histogram,
                                          dt_dev_histogram_stats_t *histogram_stats)
 {
-  const dt_image_t *img = dt_image_cache_get(darktable.image_cache, self->dev->image_storage.id, 'r');
+  const dt_image_t *img =
+    dt_image_cache_get(darktable.image_cache, self->dev->image_storage.id, 'r');
   dt_image_t image = *img;
   dt_image_cache_read_release(darktable.image_cache, img);
 
   if(image.buf_dsc.channels != 1 || image.buf_dsc.datatype != TYPE_UINT16) return;
 
   dt_mipmap_buffer_t buf;
-  dt_mipmap_cache_get(darktable.mipmap_cache, &buf, self->dev->image_storage.id, DT_MIPMAP_FULL,
+  dt_mipmap_cache_get(darktable.mipmap_cache, &buf,
+                      self->dev->image_storage.id, DT_MIPMAP_FULL,
                       DT_MIPMAP_BLOCKING, 'r');
   if(!buf.buf)
   {
@@ -315,15 +361,14 @@ static void _deflicker_prepare_histogram(dt_iop_module_t *self, uint32_t **histo
                                       // FIXME: get those from rawprepare IOP somehow !!!
                                       .crop_x = image.crop_x,
                                       .crop_y = image.crop_y,
-                                      .crop_width = image.crop_width,
-                                      .crop_height = image.crop_height };
+                                      .crop_right = image.crop_right,
+                                      .crop_bottom = image.crop_bottom };
 
   histogram_params.roi = &histogram_roi;
   histogram_params.bins_count = DEFLICKER_BINS_COUNT;
 
-  dt_histogram_worker(&histogram_params, histogram_stats, buf.buf, histogram,
-                      dt_histogram_helper_cs_RAW_uint16, NULL);
-  histogram_stats->ch = 1u;
+  dt_histogram_helper(&histogram_params, histogram_stats, IOP_CS_RAW, IOP_CS_NONE,
+                      buf.buf, histogram, NULL, FALSE, NULL);
 
   dt_mipmap_cache_release(darktable.mipmap_cache, &buf);
 }
@@ -343,28 +388,29 @@ static double _raw_to_ev(uint32_t raw, uint32_t black_level, uint32_t white_leve
   return raw_ev;
 }
 
-static void _compute_correction(dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
+static void _compute_correction(dt_iop_module_t *self,
+                                dt_iop_params_t *p1,
+                                dt_dev_pixelpipe_t *pipe,
                                 const uint32_t *const histogram,
-                                const dt_dev_histogram_stats_t *const histogram_stats, float *correction)
+                                const dt_dev_histogram_stats_t *const histogram_stats,
+                                float *correction)
 {
   const dt_iop_exposure_params_t *const p = (const dt_iop_exposure_params_t *const)p1;
 
-  *correction = NAN;
+  *correction = EXPOSURE_CORRECTION_UNDEFINED;
 
   if(histogram == NULL) return;
 
-  const size_t total = (size_t)histogram_stats->ch * histogram_stats->pixels;
-
   const double thr
-      = CLAMP(((double)total * (double)p->deflicker_percentile / (double)100.0), 0.0, (double)total);
+      = CLAMP(((double)histogram_stats->pixels * (double)p->deflicker_percentile
+               / (double)100.0), 0.0, (double)histogram_stats->pixels);
 
   size_t n = 0;
   uint32_t raw = 0;
 
-  for(uint32_t i = 0; i < histogram_stats->bins_count; i++)
+  for(size_t i = 0; i < histogram_stats->bins_count; i++)
   {
-    for(uint32_t k = 0; k < histogram_stats->ch; k++)
-      n += histogram[4 * i + k];
+    n += histogram[i];
 
     if((double)n >= thr)
     {
@@ -374,12 +420,16 @@ static void _compute_correction(dt_iop_module_t *self, dt_iop_params_t *p1, dt_d
   }
 
   const double ev
-      = _raw_to_ev(raw, (uint32_t)pipe->dsc.rawprepare.raw_black_level, pipe->dsc.rawprepare.raw_white_point);
+      = _raw_to_ev(raw, (uint32_t)pipe->dsc.rawprepare.raw_black_level,
+                   pipe->dsc.rawprepare.raw_white_point);
 
   *correction = p->deflicker_target_level - ev;
 }
 
-static void _process_common_setup(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece)
+static gboolean _show_computed(gpointer user_data);
+
+static void _process_common_setup(dt_iop_module_t *self,
+                                  dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t*)self->gui_data;
   dt_iop_exposure_data_t *d = piece->data;
@@ -392,16 +442,18 @@ static void _process_common_setup(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t
     if(g)
     {
       // histogram is precomputed and cached
-      _compute_correction(self, &d->params, piece->pipe, g->deflicker_histogram, &g->deflicker_histogram_stats,
-                         &exposure);
+      _compute_correction(self, &d->params, piece->pipe,
+                          g->deflicker_histogram, &g->deflicker_histogram_stats,
+                          &exposure);
     }
     else
     {
       uint32_t *histogram = NULL;
       dt_dev_histogram_stats_t histogram_stats;
       _deflicker_prepare_histogram(self, &histogram, &histogram_stats);
-      _compute_correction(self, &d->params, piece->pipe, histogram, &histogram_stats, &exposure);
-      free(histogram);
+      _compute_correction(self, &d->params, piece->pipe, histogram,
+                          &histogram_stats, &exposure);
+      dt_free_align(histogram);
     }
 
     // second, show computed correction in UI.
@@ -410,6 +462,8 @@ static void _process_common_setup(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t
       dt_iop_gui_enter_critical_section(self);
       g->deflicker_computed_exposure = exposure;
       dt_iop_gui_leave_critical_section(self);
+
+      g_idle_add(_show_computed, self);
     }
   }
 
@@ -418,8 +472,12 @@ static void _process_common_setup(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t
 }
 
 #ifdef HAVE_OPENCL
-int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out,
-               const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+int process_cl(struct dt_iop_module_t *self,
+               dt_dev_pixelpipe_iop_t *piece,
+               cl_mem dev_in,
+               cl_mem dev_out,
+               const dt_iop_roi_t *const roi_in,
+               const dt_iop_roi_t *const roi_out)
 {
   dt_iop_exposure_data_t *d = (dt_iop_exposure_data_t *)piece->data;
   dt_iop_exposure_global_data_t *gd = (dt_iop_exposure_global_data_t *)self->global_data;
@@ -432,22 +490,30 @@ int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_m
   const int height = roi_in->height;
 
   err = dt_opencl_enqueue_kernel_2d_args(devid, gd->kernel_exposure, width, height,
-    CLARG(dev_in), CLARG(dev_out), CLARG(width), CLARG(height), CLARG((d->black)), CLARG((d->scale)));
+                                         CLARG(dev_in), CLARG(dev_out),
+                                         CLARG(width), CLARG(height),
+                                         CLARG((d->black)), CLARG((d->scale)));
   if(err != CL_SUCCESS) goto error;
   for(int k = 0; k < 3; k++) piece->pipe->dsc.processed_maximum[k] *= d->scale;
 
   return TRUE;
 
 error:
-  dt_print(DT_DEBUG_OPENCL, "[opencl_exposure] couldn't enqueue kernel! %s\n", cl_errstr(err));
+  dt_print(DT_DEBUG_OPENCL,
+           "[opencl_exposure] couldn't enqueue kernel! %s\n", cl_errstr(err));
   return FALSE;
 }
 #endif
 
-void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const i, void *const o,
-             const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void process(struct dt_iop_module_t *self,
+             dt_dev_pixelpipe_iop_t *piece,
+             const void *const i,
+             void *const o,
+             const dt_iop_roi_t *const roi_in,
+             const dt_iop_roi_t *const roi_out)
 {
-  const dt_iop_exposure_data_t *const d = (const dt_iop_exposure_data_t *const)piece->data;
+  const dt_iop_exposure_data_t *const d =
+    (const dt_iop_exposure_data_t *const)piece->data;
 
   _process_common_setup(self, piece);
 
@@ -467,11 +533,8 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   {
     out[k] = (in[k] - black) * scale;
   }
-
-  if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK)
-    dt_iop_alpha_copy(i, o, roi_out->width, roi_out->height);
-
-  for(int k = 0; k < 3; k++) piece->pipe->dsc.processed_maximum[k] *= d->scale;
+  for(int k = 0; k < 3; k++)
+    piece->pipe->dsc.processed_maximum[k] *= d->scale;
 }
 
 
@@ -484,14 +547,16 @@ static float _get_exposure_bias(const struct dt_iop_module_t *self)
     bias = self->dev->image_storage.exif_exposure_bias;
 
   // sanity checks, don't trust exif tags too much
-  if(!isnan(bias))
+  if(bias != DT_EXIF_TAG_UNINITIALIZED)
     return CLAMP(bias, -5.0f, 5.0f);
   else
     return 0.0f;
 }
 
 
-void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
+void commit_params(struct dt_iop_module_t *self,
+                   dt_iop_params_t *p1,
+                   dt_dev_pixelpipe_t *pipe,
                    dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)p1;
@@ -502,7 +567,8 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pix
   d->params.deflicker_percentile = p->deflicker_percentile;
   d->params.deflicker_target_level = p->deflicker_target_level;
 
-  // If exposure bias compensation has been required, add it on top of user exposure correction
+  // If exposure bias compensation has been required, add it on top of
+  // user exposure correction
   if(p->compensate_exposure_bias)
     d->params.exposure -= _get_exposure_bias(self);
 
@@ -517,12 +583,16 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pix
   }
 }
 
-void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+void init_pipe(struct dt_iop_module_t *self,
+               dt_dev_pixelpipe_t *pipe,
+               dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = malloc(sizeof(dt_iop_exposure_data_t));
 }
 
-void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+void cleanup_pipe(struct dt_iop_module_t *self,
+                  dt_dev_pixelpipe_t *pipe,
+                  dt_dev_pixelpipe_iop_t *piece)
 {
   free(piece->data);
   piece->data = NULL;
@@ -553,11 +623,15 @@ void gui_update(struct dt_iop_module_t *self)
 
   dt_iop_color_picker_reset(self, TRUE);
 
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->compensate_exposure_bias), p->compensate_exposure_bias);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->compensate_exposure_bias),
+                               p->compensate_exposure_bias);
   /* xgettext:no-c-format */
-  gchar *label = g_strdup_printf(_("compensate camera exposure (%+.1f EV)"), _get_exposure_bias(self));
+  gchar *label = g_strdup_printf(_("compensate camera exposure (%+.1f EV)"),
+                                 _get_exposure_bias(self));
   gtk_button_set_label(GTK_BUTTON(g->compensate_exposure_bias), label);
-  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->compensate_exposure_bias))), PANGO_ELLIPSIZE_MIDDLE);
+  gtk_label_set_ellipsize
+    (GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->compensate_exposure_bias))),
+     PANGO_ELLIPSIZE_MIDDLE);
   g_free(label);
 
   g->spot_RGB[0] = 0.f;
@@ -573,12 +647,12 @@ void gui_update(struct dt_iop_module_t *self)
 
   dt_iop_gui_leave_critical_section(self);
 
-  free(g->deflicker_histogram);
+  dt_free_align(g->deflicker_histogram);
   g->deflicker_histogram = NULL;
 
   gtk_label_set_text(g->deflicker_used_EC, "");
   dt_iop_gui_enter_critical_section(self);
-  g->deflicker_computed_exposure = NAN;
+  g->deflicker_computed_exposure = EXPOSURE_CORRECTION_UNDEFINED;
   dt_iop_gui_leave_critical_section(self);
 
   switch(p->mode)
@@ -586,7 +660,8 @@ void gui_update(struct dt_iop_module_t *self)
     case EXPOSURE_MODE_DEFLICKER:
       _autoexp_disable(self);
       gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "deflicker");
-      _deflicker_prepare_histogram(self, &g->deflicker_histogram, &g->deflicker_histogram_stats);
+      _deflicker_prepare_histogram(self, &g->deflicker_histogram,
+                                   &g->deflicker_histogram_stats);
       break;
     case EXPOSURE_MODE_MANUAL:
     default:
@@ -594,6 +669,8 @@ void gui_update(struct dt_iop_module_t *self)
       break;
   }
 
+  dt_bauhaus_combobox_set(g->spot_mode, 0);
+
   dt_gui_update_collapsible_section(&g->cs);
 }
 
@@ -614,7 +691,8 @@ void cleanup_global(dt_iop_module_so_t *module)
   module->data = NULL;
 }
 
-static void _exposure_set_white(struct dt_iop_module_t *self, const float white)
+static void _exposure_set_white(struct dt_iop_module_t *self,
+                                const float white)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
@@ -632,7 +710,8 @@ static void _exposure_set_white(struct dt_iop_module_t *self, const float white)
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void _exposure_proxy_set_exposure(struct dt_iop_module_t *self, const float exposure)
+static void _exposure_proxy_set_exposure(struct dt_iop_module_t *self,
+                                         const float exposure)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
@@ -670,7 +749,8 @@ static float _exposure_proxy_get_exposure(struct dt_iop_module_t *self)
   }
 }
 
-static void _exposure_set_black(struct dt_iop_module_t *self, const float black)
+static void _exposure_set_black(struct dt_iop_module_t *self,
+                                const float black)
 {
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
@@ -689,7 +769,8 @@ static void _exposure_set_black(struct dt_iop_module_t *self, const float black)
   dt_dev_add_history_item(darktable.develop, self, TRUE);
 }
 
-static void _exposure_proxy_set_black(struct dt_iop_module_t *self, const float black)
+static void _exposure_proxy_set_black(struct dt_iop_module_t *self,
+                                      const float black)
 {
   _autoexp_disable(self);
   _exposure_set_black(self, black);
@@ -711,7 +792,8 @@ static void _auto_set_exposure(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe)
   const float *RGB = self->picked_color;
 
   // Get input profile, assuming we are before colorin
-  const dt_iop_order_iccprofile_info_t *const input_profile = dt_ioppr_get_pipe_input_profile_info(pipe);
+  const dt_iop_order_iccprofile_info_t *const input_profile =
+    dt_ioppr_get_pipe_input_profile_info(pipe);
   if(input_profile == NULL) return;
 
   // Convert to XYZ
@@ -719,7 +801,7 @@ static void _auto_set_exposure(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe)
   dt_aligned_pixel_t Lab;
   dot_product(RGB, input_profile->matrix_in, XYZ);
   dt_XYZ_to_Lab(XYZ, Lab);
-  Lab[1] = Lab[2] = 0.f; // make color grey to get only the equivalent lighness
+  Lab[1] = Lab[2] = Lab[3] = 0.f; // make color grey to get only the equivalent lighness
   dt_Lab_to_XYZ(Lab, XYZ);
   dt_XYZ_to_sRGB(XYZ, g->spot_RGB);
 
@@ -728,10 +810,11 @@ static void _auto_set_exposure(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe)
   dt_Lab_2_LCH(Lab, Lch);
 
   // Write report in GUI
+  gchar *str = g_strdup_printf(_("L : \t%.1f %%"), Lch[0]);
   ++darktable.gui->reset;
-  gtk_label_set_text(GTK_LABEL(g->Lch_origin),
-                     g_strdup_printf(_("L : \t%.1f %%"), Lch[0]));
+  gtk_label_set_text(GTK_LABEL(g->Lch_origin), str);
   --darktable.gui->reset;
+  g_free(str);
 
   const dt_spot_mode_t mode = dt_bauhaus_combobox_get(g->spot_mode);
 
@@ -792,21 +875,25 @@ static void _auto_set_exposure(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe)
 }
 
 
-void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpipe_iop_t *piece)
+void color_picker_apply(dt_iop_module_t *self,
+                        GtkWidget *picker,
+                        dt_dev_pixelpipe_iop_t *piece)
 {
   if(darktable.gui->reset) return;
   _auto_set_exposure(self, piece->pipe);
 }
 
 
-void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
+void gui_changed(dt_iop_module_t *self,
+                 GtkWidget *w,
+                 void *previous)
 {
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
   dt_iop_exposure_params_t *p = (dt_iop_exposure_params_t *)self->params;
 
   if(w == g->mode)
   {
-    free(g->deflicker_histogram);
+    dt_free_align(g->deflicker_histogram);
     g->deflicker_histogram = NULL;
 
     switch(p->mode)
@@ -823,7 +910,8 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
           break;
         }
         gtk_stack_set_visible_child_name(GTK_STACK(g->mode_stack), "deflicker");
-        _deflicker_prepare_histogram(self, &g->deflicker_histogram, &g->deflicker_histogram_stats);
+        _deflicker_prepare_histogram(self, &g->deflicker_histogram,
+                                     &g->deflicker_histogram_stats);
         break;
       case EXPOSURE_MODE_MANUAL:
       default:
@@ -846,29 +934,28 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 }
 
 
-static gboolean _draw(GtkWidget *widget, cairo_t *cr, dt_iop_module_t *self)
+static gboolean _show_computed(gpointer user_data)
 {
-  if(darktable.gui->reset) return FALSE;
-
+  dt_iop_module_t *self = user_data;
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
 
   dt_iop_gui_enter_critical_section(self);
-  if(!isnan(g->deflicker_computed_exposure))
+  if(g->deflicker_computed_exposure != EXPOSURE_CORRECTION_UNDEFINED)
   {
     gchar *str = g_strdup_printf(_("%.2f EV"), g->deflicker_computed_exposure);
 
-    ++darktable.gui->reset;
     gtk_label_set_text(g->deflicker_used_EC, str);
-    --darktable.gui->reset;
 
     g_free(str);
   }
   dt_iop_gui_leave_critical_section(self);
-  return FALSE;
+
+  return G_SOURCE_REMOVE;
 }
 
-
-static gboolean _target_color_draw(GtkWidget *widget, cairo_t *crf, gpointer user_data)
+static gboolean _target_color_draw(GtkWidget *widget,
+                                   cairo_t *crf,
+                                   gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
@@ -912,7 +999,9 @@ static gboolean _target_color_draw(GtkWidget *widget, cairo_t *crf, gpointer use
 }
 
 
-static gboolean _origin_color_draw(GtkWidget *widget, cairo_t *crf, gpointer user_data)
+static gboolean _origin_color_draw(GtkWidget *widget,
+                                   cairo_t *crf,
+                                   gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   dt_iop_exposure_gui_data_t *g = (dt_iop_exposure_gui_data_t *)self->gui_data;
@@ -975,7 +1064,8 @@ static void _paint_hue(dt_iop_module_t *self)
 }
 
 
-static void _spot_settings_changed_callback(GtkWidget *slider, dt_iop_module_t *self)
+static void _spot_settings_changed_callback(GtkWidget *slider,
+                                            dt_iop_module_t *self)
 {
   if(darktable.gui->reset) return;
 
@@ -996,7 +1086,7 @@ static void _spot_settings_changed_callback(GtkWidget *slider, dt_iop_module_t *
   const dt_spot_mode_t mode = dt_bauhaus_combobox_get(g->spot_mode);
   if(mode == DT_SPOT_MODE_CORRECT)
     _auto_set_exposure(self, darktable.develop->pipe);
-  // else : just record new values and do nothing
+  // else : just record new values and do nothing
 }
 
 void gui_reset(struct dt_iop_module_t *self)
@@ -1013,12 +1103,15 @@ void gui_init(struct dt_iop_module_t *self)
   g->mode_stack = GTK_STACK(gtk_stack_new());
   gtk_stack_set_homogeneous(GTK_STACK(g->mode_stack),FALSE);
 
-  GtkWidget *vbox_manual = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
+  GtkWidget *vbox_manual = self->widget =
+    gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
   gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_manual, "manual");
 
-  g->compensate_exposure_bias = dt_bauhaus_toggle_from_params(self, "compensate_exposure_bias");
-  gtk_widget_set_tooltip_text(g->compensate_exposure_bias, _("automatically remove the camera exposure bias\n"
-                                                             "this is useful if you exposed the image to the right."));
+  g->compensate_exposure_bias = dt_bauhaus_toggle_from_params
+    (self, "compensate_exposure_bias");
+  gtk_widget_set_tooltip_text(g->compensate_exposure_bias,
+                              _("automatically remove the camera exposure bias\n"
+                                "this is useful if you exposed the image to the right."));
 
   g->exposure = dt_color_picker_new(self, DT_COLOR_PICKER_AREA,
                                     dt_bauhaus_slider_from_params(self, N_("exposure")));
@@ -1027,28 +1120,33 @@ void gui_init(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set_format(g->exposure, _(" EV"));
   dt_bauhaus_slider_set_soft_range(g->exposure, -3.0, 4.0);
 
-  GtkWidget *vbox_deflicker = self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
+  GtkWidget *vbox_deflicker = self->widget =
+    gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
   gtk_stack_add_named(GTK_STACK(g->mode_stack), vbox_deflicker, "deflicker");
 
   g->deflicker_percentile = dt_bauhaus_slider_from_params(self, "deflicker_percentile");
   dt_bauhaus_slider_set_format(g->deflicker_percentile, "%");
-  gtk_widget_set_tooltip_text(g->deflicker_percentile,
-                              // xgettext:no-c-format
-                              _("where in the histogram to meter for deflicking. E.g. 50% is median"));
+  gtk_widget_set_tooltip_text
+    (g->deflicker_percentile,
+     // xgettext:no-c-format
+     _("where in the histogram to meter for deflicking. E.g. 50% is median"));
 
   g->deflicker_target_level = dt_bauhaus_slider_from_params(self, "deflicker_target_level");
   dt_bauhaus_slider_set_format(g->deflicker_target_level, _(" EV"));
-  gtk_widget_set_tooltip_text(g->deflicker_target_level,
-                              _("where to place the exposure level for processed pics, EV below overexposure."));
+  gtk_widget_set_tooltip_text
+    (g->deflicker_target_level,
+     _("where to place the exposure level for processed pics, EV below overexposure."));
 
   GtkBox *hbox1 = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
-  gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(dt_ui_label_new(_("computed EC: "))), FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox1),
+                     GTK_WIDGET(dt_ui_label_new(_("computed EC: "))), FALSE, FALSE, 0);
   g->deflicker_used_EC = GTK_LABEL(dt_ui_label_new("")); // This gets filled in by process
-  gtk_widget_set_tooltip_text(GTK_WIDGET(g->deflicker_used_EC), _("what exposure correction has actually been used"));
+  gtk_widget_set_tooltip_text(GTK_WIDGET(g->deflicker_used_EC),
+                              _("what exposure correction has actually been used"));
   gtk_box_pack_start(GTK_BOX(hbox1), GTK_WIDGET(g->deflicker_used_EC), FALSE, FALSE, 0);
 
   dt_iop_gui_enter_critical_section(self);
-  g->deflicker_computed_exposure = NAN;
+  g->deflicker_computed_exposure = EXPOSURE_CORRECTION_UNDEFINED;
   dt_iop_gui_leave_critical_section(self);
 
   gtk_box_pack_start(GTK_BOX(vbox_deflicker), GTK_WIDGET(hbox1), FALSE, FALSE, 0);
@@ -1061,10 +1159,12 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->mode_stack), TRUE, TRUE, 0);
 
   g->black = dt_bauhaus_slider_from_params(self, "black");
-  gtk_widget_set_tooltip_text(g->black, _("adjust the black level to unclip negative RGB values.\n"
-                                          "you should never use it to add more density in blacks!\n"
-                                          "if poorly set, it will clip near-black colors out of gamut\n"
-                                          "by pushing RGB values into negatives."));
+  gtk_widget_set_tooltip_text
+    (g->black,
+     _("adjust the black level to unclip negative RGB values.\n"
+       "you should never use it to add more density in blacks!\n"
+       "if poorly set, it will clip near-black colors out of gamut\n"
+       "by pushing RGB values into negatives."));
   dt_bauhaus_slider_set_digits(g->black, 4);
   dt_bauhaus_slider_set_soft_range(g->black, -0.1, 0.1);
 
@@ -1072,28 +1172,40 @@ void gui_init(struct dt_iop_module_t *self)
     (&g->cs,
      "plugins/darkroom/exposure/mapping",
      _("spot exposure mapping"),
-     GTK_BOX(self->widget));
+     GTK_BOX(self->widget),
+     DT_ACTION(self));
 
-  gtk_widget_set_tooltip_text(g->cs.expander, _("define a target brightness, in terms of exposure, for a selected region of the image (the control sample), which you then match against the same target brightness in other images. the control sample can either be a critical part of your subject or a non-moving and consistently-lit surface over your series of images."));
+  gtk_widget_set_tooltip_text
+    (g->cs.expander,
+     _("define a target brightness, in terms of exposure,\n"
+       "for a selected region of the image (the control sample),\n"
+       "which you then match against the same target brightness\n"
+       "in other images. the control sample can either\n"
+       "be a critical part of your subject or a non-moving and\n"
+       "consistently-lit surface over your series of images."));
 
-  DT_BAUHAUS_COMBOBOX_NEW_FULL(g->spot_mode, self, NULL, N_("spot mode"),
-                                _("\"correction\" automatically adjust exposure\n"
-                                  "such that the input lightness is mapped to the target.\n"
-                                  "\"measure\" simply shows how an input color is mapped by the exposure compensation\n"
-                                  "and can be used to define a target."),
-                                0, _spot_settings_changed_callback, self,
-                                N_("correction"),
-                                N_("measure"));
+  DT_BAUHAUS_COMBOBOX_NEW_FULL
+    (g->spot_mode, self, NULL, N_("spot mode"),
+     _("\"correction\" automatically adjust exposure\n"
+       "such that the input lightness is mapped to the target.\n"
+       "\"measure\" simply shows how an input color is mapped by\n"
+       "the exposure compensation and can be used to define a target."),
+     0, _spot_settings_changed_callback, self,
+     N_("correction"),
+     N_("measure"));
   gtk_box_pack_start(GTK_BOX(g->cs.container), GTK_WIDGET(g->spot_mode), TRUE, TRUE, 0);
 
-  GtkWidget *hhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
+  GtkWidget *hhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL,
+                                 DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
   GtkWidget *vvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
 
-  gtk_box_pack_start(GTK_BOX(vvbox), dt_ui_section_label_new(_("input")), FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vvbox), dt_ui_section_label_new(C_("section", "input")),
+                     FALSE, FALSE, 0);
 
   g->origin_spot = GTK_WIDGET(gtk_drawing_area_new());
-  gtk_widget_set_size_request(g->origin_spot, 2 * DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width),
-                                              DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
+  gtk_widget_set_size_request(g->origin_spot,
+                              2 * DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width),
+                              DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
   gtk_widget_set_tooltip_text(GTK_WIDGET(g->origin_spot),
                               _("the input color that should be mapped to the target"));
 
@@ -1101,19 +1213,22 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(vvbox), g->origin_spot, TRUE, TRUE, 0);
 
   g->Lch_origin = gtk_label_new(_("L : \tN/A"));
-  gtk_widget_set_tooltip_text(GTK_WIDGET(g->Lch_origin),
-                              _("these LCh coordinates are computed from CIE Lab 1976 coordinates"));
+  gtk_widget_set_tooltip_text
+    (GTK_WIDGET(g->Lch_origin),
+     _("these LCh coordinates are computed from CIE Lab 1976 coordinates"));
   gtk_box_pack_start(GTK_BOX(vvbox), GTK_WIDGET(g->Lch_origin), FALSE, FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX(hhbox), GTK_WIDGET(vvbox), FALSE, FALSE, DT_BAUHAUS_SPACE);
 
   vvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
 
-  gtk_box_pack_start(GTK_BOX(vvbox), dt_ui_section_label_new(_("target")), FALSE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vvbox), dt_ui_section_label_new(C_("section", "target")),
+                     FALSE, TRUE, 0);
 
   g->target_spot = GTK_WIDGET(gtk_drawing_area_new());
-  gtk_widget_set_size_request(g->target_spot, 2 * DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width),
-                                              DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
+  gtk_widget_set_size_request(g->target_spot,
+                              2 * DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width),
+                              DT_PIXEL_APPLY_DPI(darktable.bauhaus->quad_width));
   gtk_widget_set_tooltip_text(GTK_WIDGET(g->target_spot),
                               _("the desired target exposure after mapping"));
 
@@ -1125,14 +1240,13 @@ void gui_init(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set_format(g->lightness_spot, "%");
   dt_bauhaus_slider_set_default(g->lightness_spot, 50.f);
   gtk_box_pack_start(GTK_BOX(vvbox), GTK_WIDGET(g->lightness_spot), TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->lightness_spot), "value-changed", G_CALLBACK(_spot_settings_changed_callback), self);
+  g_signal_connect(G_OBJECT(g->lightness_spot), "value-changed",
+                   G_CALLBACK(_spot_settings_changed_callback), self);
 
   gtk_box_pack_start(GTK_BOX(hhbox), GTK_WIDGET(vvbox), TRUE, TRUE, DT_BAUHAUS_SPACE);
 
   gtk_box_pack_start(GTK_BOX(g->cs.container), GTK_WIDGET(hhbox), FALSE, FALSE, 0);
 
-  g_signal_connect(G_OBJECT(self->widget), "draw", G_CALLBACK(_draw), self);
-
   /* register hooks with current dev so that  histogram
      can interact with this module.
   */
@@ -1151,9 +1265,11 @@ void gui_cleanup(struct dt_iop_module_t *self)
   if(darktable.develop->proxy.exposure.module == self)
     darktable.develop->proxy.exposure.module = NULL;
 
-  free(g->deflicker_histogram);
+  dt_free_align(g->deflicker_histogram);
   g->deflicker_histogram = NULL;
 
+  g_idle_remove_by_data(self);
+
   IOP_GUI_FREE;
 }
 
