@@ -1,6 +1,6 @@
 /*
    This file is part of darktable,
-   Copyright (C) 2018-2021 darktable developers.
+   Copyright (C) 2018-2023 darktable developers.
 
    darktable is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
    You should have received a copy of the GNU General Public License
    along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -26,6 +27,7 @@
 #include "control/control.h"
 #include "develop/develop.h"
 #include "develop/imageop_math.h"
+#include "develop/openmp_maths.h"
 #include "dtgtk/button.h"
 #include "dtgtk/drawingarea.h"
 #include "dtgtk/expander.h"
@@ -45,10 +47,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef __SSE2__
-#include "common/sse.h"
-#endif
-
 #define DT_GUI_CURVE_EDITOR_INSET DT_PIXEL_APPLY_DPI(1)
 
 
@@ -371,17 +369,125 @@ void init_presets(dt_iop_module_so_t *self)
                              self->version(), &p, sizeof(p), 1, DEVELOP_BLEND_CS_RGB_DISPLAY);
 }
 
-static inline float gaussian(float x, float std)
+// we need to move the actual processing of each pixel into a separate
+// function to help the optimizer.  This will get inlined, but actually
+// having the code inside the loop costs us 30% extra time with GCC10.
+static void _process_pixel(const dt_aligned_pixel_t in,
+                           dt_aligned_pixel_t out,
+                           const float grey_source,
+                           const float black_source,
+                           const float inv_dynamic_range,
+                           const dt_aligned_pixel_t output_power,
+                           const float saturation,
+                           const float EPS,
+                           const int desaturate,
+                           const int preserve_color,
+                           const dt_iop_filmic_data_t *const data)
 {
-  return expf(- (x * x) / (2.0f * std * std)) / (std * powf(2.0f * M_PI, 0.5f));
+    dt_aligned_pixel_t XYZ;
+    dt_Lab_to_XYZ(in, XYZ);
+
+    dt_aligned_pixel_t input_rgb;
+    dt_XYZ_to_prophotorgb(XYZ, input_rgb);
+
+    float concavity, luma;
+
+    // Global desaturation
+    if(desaturate)
+    {
+      luma = XYZ[1];
+
+      for_each_channel(c)
+      {
+        input_rgb[c] = luma + saturation * (input_rgb[c] - luma);
+      }
+    }
+
+    dt_aligned_pixel_t rgb;
+    if(preserve_color)
+    {
+      dt_aligned_pixel_t ratios;
+      float max = dt_vector_channel_max(input_rgb);
+
+      // Save the ratios
+      for_each_channel(c)
+        ratios[c] = input_rgb[c] / max;
+
+      // Log tone-mapping
+      max = max / grey_source;
+      max = (max > EPS) ? (fastlog2(max) - black_source) * inv_dynamic_range : EPS;
+      max = CLAMP(max, 0.0f, 1.0f);
+
+      // Filmic S curve on the max RGB
+      size_t index = CLAMP(max * 0x10000ul, 0, 0xffff);
+      max = data->table[index];
+      concavity = data->grad_2[index];
+
+      // Re-apply ratios
+      for_each_channel(c)
+        rgb[c] = ratios[c] * max;
+
+      luma = max;
+    }
+    else
+    {
+      size_t DT_ALIGNED_ARRAY index[4];
+
+      for_each_channel(c)
+        input_rgb[c] /= grey_source;
+      dt_aligned_pixel_t log_rgb;
+      dt_vector_log2(input_rgb, log_rgb);
+      for_each_channel(c)
+      {
+        // Log tone-mapping on RGB
+        rgb[c] = (input_rgb[c] > EPS) ? (log_rgb[c] - black_source) * inv_dynamic_range : EPS;
+      }
+      for_each_channel(c)
+      {
+        rgb[c] = CLAMP(rgb[c], 0.0f, 1.0f);
+        // Store the index of the LUT
+        index[c] = CLAMP(rgb[c] * 0x10000ul, 0, 0xffff);
+      }
+
+      // Concavity
+      const float XYZ_luma = dt_prophotorgb_to_XYZ_luma(rgb);
+      concavity = data->grad_2[(int)CLAMP(XYZ_luma * 0x10000ul, 0, 0xffff)];
+
+      // Filmic S curve
+      for_each_channel(c)
+        rgb[c] = data->table[index[c]];
+
+      luma = dt_prophotorgb_to_XYZ_luma(rgb);
+    }
+
+    // Desaturate on the non-linear parts of the curve
+    for_each_channel(c)
+    {
+      // Desaturate on the non-linear parts of the curve
+      rgb[c] = luma + concavity * (rgb[c] - luma);
+      rgb[c] = CLAMP(rgb[c], 0.0f, 1.0f);
+    }
+    dt_aligned_pixel_t output_rgb;
+    dt_vector_powf(rgb, output_power, output_rgb);
+    // transform the result back to Lab
+    // sRGB -> XYZ -> Lab
+    dt_aligned_pixel_t res;
+    dt_prophotorgb_to_Lab(output_rgb, res);
+    copy_pixel_nontemporal(out, res);
 }
 
-void process(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid, void *const ovoid,
-             const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void process(dt_iop_module_t *self,
+             dt_dev_pixelpipe_iop_t *piece,
+             const void *const ivoid,
+             void *const ovoid,
+             const dt_iop_roi_t *const roi_in,
+             const dt_iop_roi_t *const roi_out)
 {
-  dt_iop_filmic_data_t *const data = (dt_iop_filmic_data_t *)piece->data;
+  if(!dt_iop_have_required_input_format(4 /*we need full-color pixels*/, self, piece->colors,
+                                        ivoid, ovoid, roi_in, roi_out))
+    return;
 
-  const int ch = piece->colors;
+  dt_iop_filmic_data_t *const data = (dt_iop_filmic_data_t *)piece->data;
 
   /** The log2(x) -> -INF when x -> 0
   * thus very low values (noise) will get even lower, resulting in noise negative amplification,
@@ -397,231 +503,32 @@ void process(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *c
   const int desaturate = (data->global_saturation == 100.0f) ? FALSE : TRUE;
   const float saturation = data->global_saturation / 100.0f;
 
+  const float *const restrict in = (float*)ivoid;
+  float *const restrict out = (float*)ovoid;
+  const size_t npixels = (size_t)roi_out->width * roi_out->height;
+
+  const float grey_source = data->grey_source;
+  const float black_source = data->black_source;
+  const float inv_dynamic_range = 1.0f / data->dynamic_range;
+  const dt_aligned_pixel_t output_power = {
+    data->output_power, data->output_power,
+    data->output_power, data->output_power
+  };
+
 #ifdef _OPENMP
 #pragma omp parallel for SIMD() default(none) \
-  dt_omp_firstprivate(ch, data, desaturate, ivoid, ovoid, preserve_color, roi_out, saturation, EPS) \
+  dt_omp_firstprivate(data, desaturate, in, out, npixels, grey_source, black_source, \
+                      inv_dynamic_range, output_power, preserve_color, saturation, EPS) \
   schedule(static)
 #endif
-  for(size_t k = 0; k < (size_t)roi_out->height * roi_out->width * ch; k += ch)
+  for(size_t k = 0; k < (size_t)4 * npixels; k += 4)
   {
-    float *in = ((float *)ivoid) + k;
-    float *out = ((float *)ovoid) + k;
-
-    dt_aligned_pixel_t XYZ;
-    dt_Lab_to_XYZ(in, XYZ);
-
-    dt_aligned_pixel_t rgb = { 0.0f };
-    dt_XYZ_to_prophotorgb(XYZ, rgb);
-
-    float concavity, luma;
-
-    // Global desaturation
-    if(desaturate)
-    {
-      luma = XYZ[1];
-
-      for(int c = 0; c < 3; c++)
-      {
-        rgb[c] = luma + saturation * (rgb[c] - luma);
-      }
-    }
-
-    if(preserve_color)
-    {
-      int index;
-      dt_aligned_pixel_t ratios;
-      float max = fmaxf(fmaxf(rgb[0], rgb[1]), rgb[2]);
-
-      // Save the ratios
-      for(int c = 0; c < 3; ++c) ratios[c] = rgb[c] / max;
-
-      // Log tone-mapping
-      max = max / data->grey_source;
-      max = (max > EPS) ? (fastlog2(max) - data->black_source) / data->dynamic_range : EPS;
-      max = CLAMP(max, 0.0f, 1.0f);
-
-      // Filmic S curve on the max RGB
-      index = CLAMP(max * 0x10000ul, 0, 0xffff);
-      max = data->table[index];
-      concavity = data->grad_2[index];
-
-      // Re-apply ratios
-      for(int c = 0; c < 3; ++c) rgb[c] = ratios[c] * max;
-
-      luma = max;
-    }
-    else
-    {
-      int DT_ALIGNED_ARRAY index[4];
-
-      for(int c = 0; c < 3; c++)
-      {
-        // Log tone-mapping on RGB
-        rgb[c] = rgb[c] / data->grey_source;
-        rgb[c] = (rgb[c] > EPS) ? (fastlog2(rgb[c]) - data->black_source) / data->dynamic_range : EPS;
-        rgb[c] = CLAMP(rgb[c], 0.0f, 1.0f);
-
-        // Store the index of the LUT
-        index[c] = CLAMP(rgb[c] * 0x10000ul, 0, 0xffff);
-      }
-
-      // Concavity
-      dt_prophotorgb_to_XYZ(rgb, XYZ);
-      concavity = data->grad_2[(int)CLAMP(XYZ[1] * 0x10000ul, 0, 0xffff)];
-
-      // Filmic S curve
-      for(int c = 0; c < 3; c++) rgb[c] = data->table[index[c]];
-
-      dt_prophotorgb_to_XYZ(rgb, XYZ);
-      luma = XYZ[1];
-    }
-
-    // Desaturate on the non-linear parts of the curve
-    for(int c = 0; c < 3; c++)
-    {
-      // Desaturate on the non-linear parts of the curve
-      rgb[c] = luma + concavity * (rgb[c] - luma);
-
-      // Apply the transfer function of the display
-      rgb[c] = powf(CLAMP(rgb[c], 0.0f, 1.0f), data->output_power);
-    }
-
-    // transform the result back to Lab
-    // sRGB -> XYZ
-    dt_prophotorgb_to_Lab(rgb, out);
+    _process_pixel(in + k, out +k, grey_source, black_source, inv_dynamic_range, output_power,
+                   saturation, EPS, desaturate, preserve_color, data);
   }
-
-  if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK)
-    dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
+  dt_omploop_sfence();
 }
 
-
-#if defined(__SSE__)
-void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
-             void *const ovoid, const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
-{
-  dt_iop_filmic_data_t *const data = (dt_iop_filmic_data_t *)piece->data;
-
-  const int ch = piece->colors;
-  const int preserve_color = data->preserve_color;
-
-  const float grey = data->grey_source;
-  const float black = data->black_source;
-  const float dynamic_range = data->dynamic_range;
-  const float saturation = (data->global_saturation / 100.0f);
-
-  const __m128 grey_sse = _mm_set1_ps(grey);
-  const __m128 black_sse = _mm_set1_ps(black);
-  const __m128 dynamic_range_sse = _mm_set1_ps(dynamic_range);
-  const __m128 power = _mm_set1_ps(data->output_power);
-  const __m128 saturation_sse = _mm_set1_ps(saturation);
-
-  // If saturation == 100, we have a no-op. Disable the op then.
-  const int desaturate = (data->global_saturation == 100.0f) ? FALSE : TRUE;
-
-  const float eps = powf(2.0f, -16);
-  const __m128 EPS = _mm_setr_ps(eps, eps, eps, 0.0f);
-  const __m128 zero = _mm_setzero_ps();
-  const __m128 one = _mm_set1_ps(1.0f);
-
-#ifdef _OPENMP
-#pragma omp parallel for default(none) \
-  dt_omp_firstprivate(black, black_sse, ch, data, desaturate, dynamic_range, \
-                      dynamic_range_sse, EPS, grey, grey_sse, ivoid, one, \
-                      ovoid, power, preserve_color, roi_out, saturation_sse, \
-                      zero, eps) \
-  schedule(static)
-#endif
-  for(size_t k = 0; k < (size_t)roi_out->height * roi_out->width * ch; k += ch)
-  {
-    float *in = ((float *)ivoid) + k;
-    float *out = ((float *)ovoid) + k;
-
-    __m128 XYZ = dt_Lab_to_XYZ_sse2(_mm_load_ps(in));
-    __m128 rgb = dt_XYZ_to_prophotoRGB_sse2(XYZ);
-
-    __m128 concavity;
-    __m128 luma;
-
-    // Global saturation adjustment
-    if(desaturate)
-    {
-      luma = _mm_set1_ps(XYZ[1]);
-      rgb = luma + saturation_sse * (rgb - luma);
-    }
-
-    if(preserve_color)
-    {
-      // Get the max of the RGB values
-      float max = fmax(fmaxf(rgb[0], rgb[1]), rgb[2]);
-      __m128 max_sse = _mm_set1_ps(max);
-
-      // Save the ratios
-      const __m128 ratios = rgb / max_sse;
-
-      // Log tone-mapping
-      max = max / grey;
-      max = (max > eps) ? (fastlog2(max) - black) / dynamic_range : eps;
-      max = CLAMP(max, 0.0f, 1.0f);
-
-      // Filmic S curve on the max RGB
-      const int index = CLAMP(max * 0x10000ul, 0, 0xffff);
-      max = data->table[index];
-      concavity = _mm_set1_ps(data->grad_2[index]);
-
-      // Re-apply ratios
-      max_sse = _mm_set1_ps(max);
-      rgb = ratios * max_sse;
-      luma = max_sse;
-    }
-    else
-    {
-      // Log tone-mapping
-      rgb = rgb / grey_sse;
-      rgb = _mm_max_ps(rgb, EPS);
-      rgb = _mm_log2_ps(rgb);
-      rgb -= black_sse;
-      rgb /=  dynamic_range_sse;
-      rgb = _mm_max_ps(rgb, zero);
-      rgb = _mm_min_ps(rgb, one);
-
-      // Store the derivative at the pixel luminance
-      XYZ = dt_prophotoRGB_to_XYZ_sse2(rgb);
-      concavity = _mm_set1_ps(data->grad_2[(int)CLAMP(XYZ[1] * 0x10000ul, 0, 0xffff)]);
-
-      // Unpack SSE vector to regular array
-      dt_aligned_pixel_t rgb_unpack;
-
-      // Filmic S curve
-      for(int c = 0; c < 4; ++c)
-      {
-        rgb_unpack[c] = data->table[(int)CLAMP(rgb[c] * 0x10000ul, 0, 0xffff)];
-      }
-
-      rgb = _mm_load_ps(rgb_unpack);
-      XYZ = dt_prophotoRGB_to_XYZ_sse2(rgb);
-      luma = _mm_set1_ps(XYZ[1]);
-    }
-
-    rgb = luma + concavity * (rgb - luma);
-    rgb = _mm_max_ps(rgb, zero);
-    rgb = _mm_min_ps(rgb, one);
-
-    // Apply the transfer function of the display
-    rgb = _mm_pow_ps(rgb, power);
-
-    // transform the result back to Lab
-    // sRGB -> XYZ
-    XYZ = dt_prophotoRGB_to_XYZ_sse2(rgb);
-    // XYZ -> Lab
-    _mm_stream_ps(out, dt_XYZ_to_Lab_sse2(XYZ));
-  }
-
-  if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK) dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
-}
-#endif
-
-
 #ifdef HAVE_OPENCL
 int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out,
                const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
@@ -848,7 +755,7 @@ void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpi
   else if(picker == g->auto_button)
     apply_autotune(self);
   else
-    fprintf(stderr, "[filmic] unknown color picker\n");
+    dt_print(DT_DEBUG_ALWAYS, "[filmic] unknown color picker\n");
 }
 
 static void grey_point_source_callback(GtkWidget *slider, gpointer user_data)
@@ -1389,7 +1296,7 @@ void init(dt_iop_module_t *module)
 {
   module->params = calloc(1, sizeof(dt_iop_filmic_params_t));
   module->default_params = calloc(1, sizeof(dt_iop_filmic_params_t));
-  module->default_enabled = 0;
+  module->default_enabled = FALSE;
   module->params_size = sizeof(dt_iop_filmic_params_t);
   module->gui_data = NULL;
 
@@ -1574,7 +1481,7 @@ void gui_init(dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->area), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(g->area), "draw", G_CALLBACK(dt_iop_tonecurve_draw), self);
 
-  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_section_label_new(_("logarithmic shaper")), FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_section_label_new(C_("section", "logarithmic shaper")), FALSE, FALSE, 0);
 
   // grey_point_source slider
   g->grey_point_source = dt_bauhaus_slider_new_with_range(self, 0.0, 100., 0, p->grey_point_source, 2);
@@ -1585,7 +1492,8 @@ void gui_init(dt_iop_module_t *self)
   gtk_widget_set_tooltip_text(g->grey_point_source, _("adjust to match the average luminance of the subject.\n"
                                                       "except in back-lighting situations, this should be around 18%."));
   g_signal_connect(G_OBJECT(g->grey_point_source), "value-changed", G_CALLBACK(grey_point_source_callback), self);
-  dt_color_picker_new(self, DT_COLOR_PICKER_AREA, g->grey_point_source);
+  dt_color_picker_new(self, DT_COLOR_PICKER_AREA | DT_COLOR_PICKER_DENOISE,
+                      g->grey_point_source);
 
   // White slider
   g->white_point_source = dt_bauhaus_slider_new_with_range(self, 0.0, 16.0, 0, p->white_point_source, 2);
@@ -1597,7 +1505,8 @@ void gui_init(dt_iop_module_t *self)
                                                        "this is a reading a lightmeter would give you on the scene.\n"
                                                        "adjust so highlights clipping is avoided"));
   g_signal_connect(G_OBJECT(g->white_point_source), "value-changed", G_CALLBACK(white_point_source_callback), self);
-  dt_color_picker_new(self, DT_COLOR_PICKER_AREA, g->white_point_source);
+  dt_color_picker_new(self, DT_COLOR_PICKER_AREA | DT_COLOR_PICKER_DENOISE,
+                      g->white_point_source);
 
   // Black slider
   g->black_point_source = dt_bauhaus_slider_new_with_range(self, -16.0, -0.1, 0, p->black_point_source, 2);
@@ -1609,7 +1518,8 @@ void gui_init(dt_iop_module_t *self)
                                                        "this is a reading a lightmeter would give you on the scene.\n"
                                                        "increase to get more contrast.\ndecrease to recover more details in low-lights."));
   g_signal_connect(G_OBJECT(g->black_point_source), "value-changed", G_CALLBACK(black_point_source_callback), self);
-  dt_color_picker_new(self, DT_COLOR_PICKER_AREA, g->black_point_source);
+  dt_color_picker_new(self, DT_COLOR_PICKER_AREA | DT_COLOR_PICKER_DENOISE,
+                      g->black_point_source);
 
   // Security factor
   g->security_factor = dt_bauhaus_slider_new_with_range(self, -50., 50., 0, p->security_factor, 2);
@@ -1623,13 +1533,14 @@ void gui_init(dt_iop_module_t *self)
   // Auto tune slider
   g->auto_button = dt_bauhaus_combobox_new(self);
   dt_bauhaus_widget_set_label(g->auto_button, NULL, N_("auto tune levels"));
-  dt_color_picker_new(self, DT_COLOR_PICKER_AREA, g->auto_button);
+  dt_color_picker_new(self, DT_COLOR_PICKER_AREA | DT_COLOR_PICKER_DENOISE,
+                      g->auto_button);
   gtk_widget_set_tooltip_text(g->auto_button, _("try to optimize the settings with some guessing.\n"
                                                 "this will fit the luminance range inside the histogram bounds.\n"
                                                 "works better for landscapes and evenly-lit pictures\nbut fails for high-keys and low-keys." ));
   gtk_box_pack_start(GTK_BOX(self->widget), g->auto_button, TRUE, TRUE, 0);
 
-  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_section_label_new(_("filmic S curve")), FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(self->widget), dt_ui_section_label_new(C_("section", "filmic S curve")), FALSE, FALSE, 0);
 
   // contrast slider
   g->contrast = dt_bauhaus_slider_new_with_range(self, 0., 5., 0, p->contrast, 3);
@@ -1708,7 +1619,7 @@ void gui_init(dt_iop_module_t *self)
   // add collapsible section for those extra options that are generally not to be used
 
   GtkWidget *destdisp_head = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, DT_BAUHAUS_SPACE);
-  GtkWidget *destdisp = dt_ui_section_label_new(_("destination/display"));
+  GtkWidget *destdisp = dt_ui_section_label_new(C_("section", "destination/display"));
   g->extra_toggle = dtgtk_togglebutton_new(dtgtk_cairo_paint_solid_arrow, CPF_DIRECTION_LEFT, NULL);
   GtkWidget *extra_options = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
   gtk_box_pack_start(GTK_BOX(destdisp_head), destdisp, TRUE, TRUE, 0);
