@@ -118,14 +118,20 @@ static float lookup(const float *lut, const float i)
   return lut[bin1] * f + lut[bin0] * (1. - f);
 }
 
-void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const i, void *const o,
-             const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void process(struct dt_iop_module_t *self,
+             dt_dev_pixelpipe_iop_t *piece,
+             const void *const restrict i,
+             void *const restrict o,
+             const dt_iop_roi_t *const roi_in,
+             const dt_iop_roi_t *const roi_out)
 {
+  if(!dt_iop_have_required_input_format(4 /*we need full-color pixels*/, self, piece->colors,
+                                         i, o, roi_in, roi_out))
+    return;
   dt_iop_lowlight_data_t *d = (dt_iop_lowlight_data_t *)(piece->data);
-  const int ch = piece->colors;
 
   // empiric coefficient
-  const float c = 0.5f;
+  const float coeff = 0.5f;
   const float threshold = 0.01f;
 
   // scotopic white, blue saturated
@@ -134,19 +140,20 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
 
   dt_Lab_to_XYZ(Lab_sw, XYZ_sw);
 
+  const float *lut = d->lut;
+  const size_t npixels = (size_t)roi_out->height * roi_out->width;
+
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(ch, i, o, roi_out, threshold, c) \
-  shared(d, XYZ_sw) \
+  dt_omp_firstprivate(i, o, npixels, threshold, coeff, lut, XYZ_sw)        \
   schedule(static)
 #endif
-  for(size_t k = 0; k < (size_t)roi_out->width * roi_out->height; k++)
+  for(size_t k = 0; k < (size_t)npixels; k++)
   {
-    float *in = (float *)i + ch * k;
-    float *out = (float *)o + ch * k;
+    const float *const in = (float *)i + 4 * k;
+    float *const out = (float *)o + 4 * k;
     dt_aligned_pixel_t XYZ, XYZ_s;
     float V;
-    float w;
 
     dt_Lab_to_XYZ(in, XYZ);
 
@@ -163,22 +170,21 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
     }
 
     // scale using empiric coefficient and fit inside limits
-    V = fminf(1.0f, fmaxf(0.0f, c * V));
+//    V = fminf(1.0f, fmaxf(0.0f, coeff * V));
+    V = CLIP(coeff * V);
 
     // blending coefficient from curve
-    w = lookup(d->lut, in[0] / 100.f);
+    const float w = lookup(lut, in[0] / 100.f);
 
-    XYZ_s[0] = V * XYZ_sw[0];
-    XYZ_s[1] = V * XYZ_sw[1];
-    XYZ_s[2] = V * XYZ_sw[2];
+    for_each_channel(c)
+      XYZ_s[c] = V * XYZ_sw[c];
 
-    XYZ[0] = w * XYZ[0] + (1.0f - w) * XYZ_s[0];
-    XYZ[1] = w * XYZ[1] + (1.0f - w) * XYZ_s[1];
-    XYZ[2] = w * XYZ[2] + (1.0f - w) * XYZ_s[2];
+    for_each_channel(c)
+      XYZ[c] = w * XYZ[c] + (1.0f - w) * XYZ_s[c];
 
-    dt_XYZ_to_Lab(XYZ, out);
-
-    out[3] = in[3];
+    dt_aligned_pixel_t res;
+    dt_XYZ_to_Lab(XYZ, res);
+    copy_pixel_nontemporal(out, res);
   }
 }
 
@@ -281,7 +287,6 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_lowlight_gui_data_t *g = (dt_iop_lowlight_gui_data_t *)self->gui_data;
   dt_iop_lowlight_params_t *p = (dt_iop_lowlight_params_t *)self->params;
   dt_bauhaus_slider_set(g->scale_blueness, p->blueness);
-  dt_iop_cancel_history_update(self);
   gtk_widget_queue_draw(self->widget);
 }
 
@@ -698,7 +703,7 @@ static gboolean lowlight_motion_notify(GtkWidget *widget, GdkEventMotion *event,
       dt_iop_lowlight_get_params(p, c->mouse_x, c->mouse_y + c->mouse_pick, c->mouse_radius);
     }
     gtk_widget_queue_draw(widget);
-    dt_iop_queue_history_update(self, FALSE);
+    dt_dev_add_history_item_target(darktable.develop, self, TRUE, widget);
   }
   else if(event->y > height)
   {
@@ -736,7 +741,7 @@ static gboolean lowlight_button_press(GtkWidget *widget, GdkEventButton *event,
       p->transition_x[k] = d->transition_x[k];
       p->transition_y[k] = d->transition_y[k];
     }
-    dt_dev_add_history_item(darktable.develop, self, TRUE);
+    dt_dev_add_history_item_target(darktable.develop, self, TRUE, widget);
     gtk_widget_queue_draw(self->widget);
   }
   else if(event->button == 1)
@@ -787,18 +792,8 @@ static gboolean lowlight_scrolled(GtkWidget *widget, GdkEventScroll *event, gpoi
   int delta_y;
   if(dt_gui_get_scroll_unit_deltas(event, NULL, &delta_y))
   {
-    if(dt_modifier_is(event->state, GDK_CONTROL_MASK))
-    {
-      //adjust aspect
-      const int aspect = dt_conf_get_int("plugins/darkroom/lowlight/aspect_percent");
-      dt_conf_set_int("plugins/darkroom/lowlight/aspect_percent", aspect + delta_y);
-      dtgtk_drawing_area_set_aspect_ratio(widget, aspect / 100.0);
-    }
-    else
-    {
-      c->mouse_radius = CLAMP(c->mouse_radius * (1.0 + 0.1 * delta_y), 0.2 / DT_IOP_LOWLIGHT_BANDS, 1.0);
-      gtk_widget_queue_draw(widget);
-    }
+    c->mouse_radius = CLAMP(c->mouse_radius * (1.0 + 0.1 * delta_y), 0.2 / DT_IOP_LOWLIGHT_BANDS, 1.0);
+    gtk_widget_queue_draw(widget);
   }
 
   return TRUE;
@@ -819,20 +814,15 @@ void gui_init(struct dt_iop_module_t *self)
   c->mouse_x = c->mouse_y = c->mouse_pick = -1.0;
   c->dragging = 0;
   c->x_move = -1;
-  self->timeout_handle = 0;
   c->mouse_radius = 1.0 / DT_IOP_LOWLIGHT_BANDS;
 
   self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, DT_BAUHAUS_SPACE);
 
-  const float aspect = dt_conf_get_int("plugins/darkroom/lowlight/aspect_percent") / 100.0;
-  c->area = GTK_DRAWING_AREA(dtgtk_drawing_area_new_with_aspect_ratio(aspect));
+  c->area = GTK_DRAWING_AREA(dt_ui_resize_wrap(NULL, 0, "plugins/darkroom/lowlight/aspect_percent"));
   g_object_set_data(G_OBJECT(c->area), "iop-instance", self);
   dt_action_define_iop(self, NULL, N_("graph"), GTK_WIDGET(c->area), NULL);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(c->area), FALSE, FALSE, 0);
 
-  gtk_widget_add_events(GTK_WIDGET(c->area), GDK_POINTER_MOTION_MASK | darktable.gui->scroll_mask
-                                           | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
-                                           | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
   g_signal_connect(G_OBJECT(c->area), "draw", G_CALLBACK(lowlight_draw), self);
   g_signal_connect(G_OBJECT(c->area), "button-press-event", G_CALLBACK(lowlight_button_press), self);
   g_signal_connect(G_OBJECT(c->area), "button-release-event", G_CALLBACK(lowlight_button_release), self);
@@ -849,7 +839,6 @@ void gui_cleanup(struct dt_iop_module_t *self)
 {
   dt_iop_lowlight_gui_data_t *c = (dt_iop_lowlight_gui_data_t *)self->gui_data;
   dt_draw_curve_destroy(c->transition_curve);
-  dt_iop_cancel_history_update(self);
 
   IOP_GUI_FREE;
 }
