@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2015-2021 darktable developers.
+    Copyright (C) 2015-2023 darktable developers.
 
     (based on code by johannes hanika)
 
@@ -21,18 +21,18 @@
 #include "config.h"
 #endif
 #include "bauhaus/bauhaus.h"
-#include "common/imageio_rawspeed.h" // for dt_rawspeed_crop_dcraw_filters
 #include "common/opencl.h"
 #include "common/imagebuf.h"
+#include "common/image_cache.h"
+#include "common/dng_opcode.h"
 #include "develop/imageop.h"
 #include "develop/imageop_gui.h"
 #include "develop/tiling.h"
-#include "common/image_cache.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "gui/presets.h"
+#include "imageio/imageio_rawspeed.h" // for dt_rawspeed_crop_dcraw_filters
 #include "iop/iop_api.h"
-#include "common/dng_opcode.h"
 
 #include <gtk/gtk.h>
 #include <stdint.h>
@@ -48,10 +48,10 @@ typedef enum dt_iop_rawprepare_flat_field_t
 
 typedef struct dt_iop_rawprepare_params_t
 {
-  int32_t x; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop left"
-  int32_t y; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop top"
-  int32_t width; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop right"
-  int32_t height; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop bottom"
+  int32_t left; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop left"
+  int32_t top; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop top"
+  int32_t right; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop right"
+  int32_t bottom; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "crop bottom"
   uint16_t raw_black_level_separate[4]; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "black level"
   uint16_t raw_white_point; // $MIN: 0 $MAX: UINT16_MAX $DESCRIPTION: "white point"
   dt_iop_rawprepare_flat_field_t flat_field; // $DEFAULT: FLAT_FIELD_OFF $DESCRIPTION: "flat field correction"
@@ -61,13 +61,13 @@ typedef struct dt_iop_rawprepare_gui_data_t
 {
   GtkWidget *black_level_separate[4];
   GtkWidget *white_point;
-  GtkWidget *x, *y, *width, *height;
+  GtkWidget *left, *top, *right, *bottom;
   GtkWidget *flat_field;
 } dt_iop_rawprepare_gui_data_t;
 
 typedef struct dt_iop_rawprepare_data_t
 {
-  int32_t x, y, width, height; // crop, now unused, for future expansion
+  int32_t left, top, right, bottom;
   float sub[4];
   float div[4];
 
@@ -115,21 +115,26 @@ int default_group()
   return IOP_GROUP_BASIC | IOP_GROUP_TECHNICAL;
 }
 
-int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+int default_colorspace(dt_iop_module_t *self,
+                       dt_dev_pixelpipe_t *pipe,
+                       dt_dev_pixelpipe_iop_t *piece)
 {
   return IOP_CS_RAW;
 }
 
-int legacy_params(dt_iop_module_t *self, const void *const old_params, const int old_version,
-                  void *new_params, const int new_version)
+int legacy_params(dt_iop_module_t *self,
+                  const void *const old_params,
+                  const int old_version,
+                  void *new_params,
+                  const int new_version)
 {
   typedef struct dt_iop_rawprepare_params_t dt_iop_rawprepare_params_v2_t;
   typedef struct dt_iop_rawprepare_params_v1_t
   {
-    int32_t x;
-    int32_t y;
-    int32_t width;
-    int32_t height;
+    int32_t left;
+    int32_t top;
+    int32_t right;
+    int32_t bottom;
     uint16_t raw_black_level_separate[4];
     uint16_t raw_white_point;
   } dt_iop_rawprepare_params_v1_t;
@@ -161,10 +166,10 @@ void init_presets(dt_iop_module_so_t *self)
   dt_database_start_transaction(darktable.db);
 
   dt_gui_presets_add_generic(_("passthrough"), self->op, self->version(),
-                             &(dt_iop_rawprepare_params_t){.x = 0,
-                                                           .y = 0,
-                                                           .width = 0,
-                                                           .height = 0,
+                             &(dt_iop_rawprepare_params_t){.left = 0,
+                                                           .top = 0,
+                                                           .right = 0,
+                                                           .bottom = 0,
                                                            .raw_black_level_separate[0] = 0,
                                                            .raw_black_level_separate[1] = 0,
                                                            .raw_black_level_separate[2] = 0,
@@ -175,22 +180,29 @@ void init_presets(dt_iop_module_so_t *self)
   dt_database_release_transaction(darktable.db);
 }
 
-static int compute_proper_crop(dt_dev_pixelpipe_iop_t *piece, const dt_iop_roi_t *const roi_in, int value)
+static int _compute_proper_crop(dt_dev_pixelpipe_iop_t *piece,
+                                const dt_iop_roi_t *const roi_in,
+                                const int value)
 {
   const float scale = roi_in->scale / piece->iscale;
   return (int)roundf((float)value * scale);
 }
 
-int distort_transform(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, float *const restrict points, size_t points_count)
+int distort_transform(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        float *const restrict points,
+        size_t points_count)
 {
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
 
   // nothing to be done if parameters are set to neutral values (no top/left crop)
-  if(d->x == 0 && d->y == 0) return 1;
+  if(d->left == 0 && d->top == 0) return 1;
 
   const float scale = piece->buf_in.scale / piece->iscale;
 
-  const float x = (float)d->x * scale, y = (float)d->y * scale;
+  const float x = (float)d->left * scale;
+  const float y = (float)d->top * scale;
 
 #ifdef _OPENMP
 #pragma omp parallel for simd default(none) \
@@ -207,17 +219,21 @@ int distort_transform(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, floa
   return 1;
 }
 
-int distort_backtransform(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, float *const restrict points,
-                          size_t points_count)
+int distort_backtransform(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        float *const restrict points,
+        size_t points_count)
 {
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
 
   // nothing to be done if parameters are set to neutral values (no top/left crop)
-  if(d->x == 0 && d->y == 0) return 1;
+  if(d->left == 0 && d->top == 0) return 1;
 
   const float scale = piece->buf_in.scale / piece->iscale;
 
-  const float x = (float)d->x * scale, y = (float)d->y * scale;
+  const float x = (float)d->left * scale;
+  const float y = (float)d->top * scale;
 
 #ifdef _OPENMP
 #pragma omp parallel for simd default(none) \
@@ -234,43 +250,59 @@ int distort_backtransform(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece,
   return 1;
 }
 
-void distort_mask(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, const float *const in,
-                  float *const out, const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void distort_mask(
+        struct dt_iop_module_t *self,
+        struct dt_dev_pixelpipe_iop_t *piece,
+        const float *const in,
+        float *const out,
+        const dt_iop_roi_t *const roi_in,
+        const dt_iop_roi_t *const roi_out)
 {
   dt_iop_copy_image_roi(out, in, 1, roi_in, roi_out, TRUE);
 }
 
 // we're not scaling here (bayer input), so just crop borders
-void modify_roi_out(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, dt_iop_roi_t *roi_out,
-                    const dt_iop_roi_t *const roi_in)
+void modify_roi_out(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        dt_iop_roi_t *roi_out,
+        const dt_iop_roi_t *const roi_in)
 {
   *roi_out = *roi_in;
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
 
   roi_out->x = roi_out->y = 0;
 
-  const int32_t x = d->x + d->width, y = d->y + d->height;
+  const int32_t x = d->left + d->right;
+  const int32_t y = d->top + d->bottom;
 
   const float scale = roi_in->scale / piece->iscale;
   roi_out->width -= (int)roundf((float)x * scale);
   roi_out->height -= (int)roundf((float)y * scale);
 }
 
-void modify_roi_in(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const dt_iop_roi_t *const roi_out,
-                   dt_iop_roi_t *roi_in)
+void modify_roi_in(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        const dt_iop_roi_t *const roi_out,
+        dt_iop_roi_t *roi_in)
 {
   *roi_in = *roi_out;
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
 
-  const int32_t x = d->x + d->width, y = d->y + d->height;
+  const int32_t x = d->left + d->right;
+  const int32_t y = d->top + d->bottom;
 
   const float scale = roi_in->scale / piece->iscale;
   roi_in->width += (int)roundf((float)x * scale);
   roi_in->height += (int)roundf((float)y * scale);
 }
 
-void output_format(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece,
-                   dt_iop_buffer_dsc_t *dsc)
+void output_format(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_t *pipe,
+        dt_dev_pixelpipe_iop_t *piece,
+        dt_iop_buffer_dsc_t *dsc)
 {
   default_output_format(self, pipe, piece, dsc);
 
@@ -280,8 +312,10 @@ void output_format(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixel
   dsc->rawprepare.raw_white_point = d->rawprepare.raw_white_point;
 }
 
-static void adjust_xtrans_filters(dt_dev_pixelpipe_t *pipe,
-                                  uint32_t crop_x, uint32_t crop_y)
+static void _adjust_xtrans_filters(
+        dt_dev_pixelpipe_t *pipe,
+        const uint32_t crop_x,
+        const uint32_t crop_y)
 {
   for(int i = 0; i < 6; ++i)
   {
@@ -292,25 +326,26 @@ static void adjust_xtrans_filters(dt_dev_pixelpipe_t *pipe,
   }
 }
 
-static int BL(const dt_iop_roi_t *const roi_out, const dt_iop_rawprepare_data_t *const d, const int row,
-              const int col)
+static int _BL(const dt_iop_roi_t *const roi_out,
+               const dt_iop_rawprepare_data_t *const d,
+               const int row,
+               const int col)
 {
-  return ((((row + roi_out->y + d->y) & 1) << 1) + ((col + roi_out->x + d->x) & 1));
+  return ((((row + roi_out->y + d->top) & 1) << 1) + ((col + roi_out->x + d->left) & 1));
 }
 
-/* Some comments about the cpu code path; tests with gcc 10.x show a clear performance gain for the
-   compile generated code vs SSE specific code. This depends slightly on the cpu but it's 1.2 to 3-fold
-   better for all tested cases.
-*/
-void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
-             void *const ovoid, const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void process(
+        struct dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        const void *const ivoid,
+        void *const ovoid,
+        const dt_iop_roi_t *const roi_in,
+        const dt_iop_roi_t *const roi_out)
 {
   const dt_iop_rawprepare_data_t *const d = (dt_iop_rawprepare_data_t *)piece->data;
 
-  // fprintf(stderr, "roi in %d %d %d %d\n", roi_in->x, roi_in->y, roi_in->width, roi_in->height);
-  // fprintf(stderr, "roi out %d %d %d %d\n", roi_out->x, roi_out->y, roi_out->width, roi_out->height);
-
-  const int csx = compute_proper_crop(piece, roi_in, d->x), csy = compute_proper_crop(piece, roi_in, d->y);
+  const int csx = _compute_proper_crop(piece, roi_in, d->left);
+  const int csy = _compute_proper_crop(piece, roi_in, d->top);
 
   if(piece->pipe->dsc.filters && piece->dsc_in.channels == 1
      && piece->dsc_in.datatype == TYPE_UINT16)
@@ -332,13 +367,14 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
         const size_t pin = (size_t)(roi_in->width * (j + csy) + csx) + i;
         const size_t pout = (size_t)j * roi_out->width + i;
 
-        const int id = BL(roi_out, d, j, i);
+        const int id = _BL(roi_out, d, j, i);
         out[pout] = (in[pin] - d->sub[id]) / d->div[id];
       }
     }
 
-    piece->pipe->dsc.filters = dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
-    adjust_xtrans_filters(piece->pipe, csx, csy);
+    piece->pipe->dsc.filters =
+      dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
+    _adjust_xtrans_filters(piece->pipe, csx, csy);
   }
   else if(piece->pipe->dsc.filters && piece->dsc_in.channels == 1
           && piece->dsc_in.datatype == TYPE_FLOAT)
@@ -360,13 +396,14 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
         const size_t pin = (size_t)(roi_in->width * (j + csy) + csx) + i;
         const size_t pout = (size_t)j * roi_out->width + i;
 
-        const int id = BL(roi_out, d, j, i);
+        const int id = _BL(roi_out, d, j, i);
         out[pout] = (in[pin] - d->sub[id]) / d->div[id];
       }
     }
 
-    piece->pipe->dsc.filters = dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
-    adjust_xtrans_filters(piece->pipe, csx, csy);
+    piece->pipe->dsc.filters =
+      dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
+    _adjust_xtrans_filters(piece->pipe, csx, csy);
   }
   else
   { // pre-downsampled buffer that needs black/white scaling
@@ -374,13 +411,11 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
     const float *const in = (const float *const)ivoid;
     float *const out = (float *const)ovoid;
 
-    const float sub = d->sub[0], div = d->div[0];
-
     const int ch = piece->colors;
 
 #ifdef _OPENMP
 #pragma omp parallel for SIMD() default(none) \
-    dt_omp_firstprivate(ch, csx, csy, div, in, out, roi_in, roi_out, sub) \
+    dt_omp_firstprivate(ch, csx, csy, d, in, out, roi_in, roi_out) \
     schedule(static) collapse(3)
 #endif
     for(int j = 0; j < roi_out->height; j++)
@@ -392,7 +427,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
           const size_t pin = (size_t)ch * (roi_in->width * (j + csy) + csx + i) + c;
           const size_t pout = (size_t)ch * (j * roi_out->width + i) + c;
 
-          out[pout] = (in[pin] - sub) / div;
+          out[pout] = (in[pin] - d->sub[c]) / d->div[c];
         }
       }
     }
@@ -402,10 +437,10 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   {
     const uint32_t map_w = d->gainmaps[0]->map_points_h;
     const uint32_t map_h = d->gainmaps[0]->map_points_v;
-    const float im_to_rel_x = 1.0 / piece->buf_in.width;
-    const float im_to_rel_y = 1.0 / piece->buf_in.height;
-    const float rel_to_map_x = 1.0 / d->gainmaps[0]->map_spacing_h;
-    const float rel_to_map_y = 1.0 / d->gainmaps[0]->map_spacing_v;
+    const float im_to_rel_x = 1.0f / piece->buf_in.width;
+    const float im_to_rel_y = 1.0f / piece->buf_in.height;
+    const float rel_to_map_x = 1.0f / d->gainmaps[0]->map_spacing_h;
+    const float rel_to_map_y = 1.0f / d->gainmaps[0]->map_spacing_v;
     const float map_origin_h = d->gainmaps[0]->map_origin_h;
     const float map_origin_v = d->gainmaps[0]->map_origin_v;
     float *const out = (float *const)ovoid;
@@ -431,7 +466,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
       }
       for(int i = 0; i < roi_out->width; i++)
       {
-        const int id = BL(roi_out, d, j, i);
+        const int id = _BL(roi_out, d, j, i);
         const float x_map = CLAMP(((roi_out->x + csx + i) * im_to_rel_x - map_origin_h) * rel_to_map_x, 0, map_w);
         const uint32_t x_i0 = MIN(x_map, map_w - 1);
         const uint32_t x_i1 = MIN(x_i0 + 1, map_w - 1);
@@ -443,14 +478,20 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
     }
   }
 
-  dt_dev_write_rawdetail_mask(piece, (float *const)ovoid, roi_in, DT_DEV_DETAIL_MASK_RAWPREPARE);
+  if(!dt_image_is_raw(&piece->pipe->image) && piece->pipe->want_detail_mask)
+    dt_dev_write_rawdetail_mask(piece, (float *const)ovoid, roi_in, FALSE);
 
   for(int k = 0; k < 4; k++) piece->pipe->dsc.processed_maximum[k] = 1.0f;
 }
 
 #ifdef HAVE_OPENCL
-int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out,
-               const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+int process_cl(
+        dt_iop_module_t *self,
+        dt_dev_pixelpipe_iop_t *piece,
+        cl_mem dev_in,
+        cl_mem dev_out,
+        const dt_iop_roi_t *const roi_in,
+        const dt_iop_roi_t *const roi_out)
 {
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
   dt_iop_rawprepare_global_data_t *gd = (dt_iop_rawprepare_global_data_t *)self->global_data;
@@ -464,7 +505,9 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
   int kernel = -1;
   gboolean gainmap_args = FALSE;
 
-  if(piece->pipe->dsc.filters && piece->dsc_in.channels == 1 && piece->dsc_in.datatype == TYPE_UINT16)
+  if(piece->pipe->dsc.filters
+     && piece->dsc_in.channels == 1
+     && piece->dsc_in.datatype == TYPE_UINT16)
   {
     if(d->apply_gainmaps)
     {
@@ -476,7 +519,9 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
       kernel = gd->kernel_rawprepare_1f;
     }
   }
-  else if(piece->pipe->dsc.filters && piece->dsc_in.channels == 1 && piece->dsc_in.datatype == TYPE_FLOAT)
+  else if(piece->pipe->dsc.filters
+          && piece->dsc_in.channels == 1
+          && piece->dsc_in.datatype == TYPE_FLOAT)
   {
     if(d->apply_gainmaps)
     {
@@ -493,7 +538,8 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
     kernel = gd->kernel_rawprepare_4f;
   }
 
-  const int csx = compute_proper_crop(piece, roi_in, d->x), csy = compute_proper_crop(piece, roi_in, d->y);
+  const int csx = _compute_proper_crop(piece, roi_in, d->left);
+  const int csy = _compute_proper_crop(piece, roi_in, d->top);
 
   dev_sub = dt_opencl_copy_host_to_device_constant(devid, sizeof(float) * 4, d->sub);
   if(dev_sub == NULL) goto error;
@@ -510,8 +556,8 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
   if(gainmap_args)
   {
     const int map_size[2] = { d->gainmaps[0]->map_points_h, d->gainmaps[0]->map_points_v };
-    const float im_to_rel[2] = { 1.0 / piece->buf_in.width, 1.0 / piece->buf_in.height };
-    const float rel_to_map[2] = { 1.0 / d->gainmaps[0]->map_spacing_h, 1.0 / d->gainmaps[0]->map_spacing_v };
+    const float im_to_rel[2] = { 1.0f / piece->buf_in.width, 1.0f / piece->buf_in.height };
+    const float rel_to_map[2] = { 1.0f / d->gainmaps[0]->map_spacing_h, 1.0f / d->gainmaps[0]->map_spacing_v };
     const float map_origin[2] = { d->gainmaps[0]->map_origin_h, d->gainmaps[0]->map_origin_v };
 
     for(int i = 0; i < 4; i++)
@@ -523,8 +569,10 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
       if(err != CL_SUCCESS) goto error;
     }
 
-    dt_opencl_set_kernel_args(devid, kernel, 10, CLARG(dev_gainmap[0]), CLARG(dev_gainmap[1]), CLARG(dev_gainmap[2]),
-      CLARG(dev_gainmap[3]), CLARG(map_size), CLARG(im_to_rel), CLARG(rel_to_map), CLARG(map_origin));
+    dt_opencl_set_kernel_args
+      (devid, kernel, 10,
+       CLARG(dev_gainmap[0]), CLARG(dev_gainmap[1]), CLARG(dev_gainmap[2]), CLARG(dev_gainmap[3]),
+       CLARG(map_size), CLARG(im_to_rel), CLARG(rel_to_map), CLARG(map_origin));
   }
   err = dt_opencl_enqueue_kernel_2d(devid, kernel, sizes);
   if(err != CL_SUCCESS) goto error;
@@ -535,15 +583,18 @@ int process_cl(dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_
 
   if(piece->pipe->dsc.filters)
   {
-    piece->pipe->dsc.filters = dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
-    adjust_xtrans_filters(piece->pipe, csx, csy);
+    piece->pipe->dsc.filters =
+      dt_rawspeed_crop_dcraw_filters(self->dev->image_storage.buf_dsc.filters, csx, csy);
+    _adjust_xtrans_filters(piece->pipe, csx, csy);
   }
 
   for(int k = 0; k < 4; k++) piece->pipe->dsc.processed_maximum[k] = 1.0f;
 
-  err = dt_dev_write_rawdetail_mask_cl(piece, dev_out, roi_in, DT_DEV_DETAIL_MASK_RAWPREPARE);
-  if(err != CL_SUCCESS) goto error;
-
+  if(!dt_image_is_raw(&piece->pipe->image) && piece->pipe->want_detail_mask)
+  {
+    err = dt_dev_write_rawdetail_mask_cl(piece, dev_out, roi_in, FALSE);
+    if(err != CL_SUCCESS) goto error;
+  }
   return TRUE;
 
 error:
@@ -555,7 +606,7 @@ error:
 }
 #endif
 
-static int image_is_normalized(const dt_image_t *const image)
+static int _image_is_normalized(const dt_image_t *const image)
 {
   // if raw with floating-point data, if not special magic whitelevel, then it needs normalization
   if((image->flags & DT_IMAGE_HDR) == DT_IMAGE_HDR)
@@ -574,26 +625,51 @@ static int image_is_normalized(const dt_image_t *const image)
   return image->buf_dsc.channels == 1 && image->buf_dsc.datatype == TYPE_FLOAT;
 }
 
-static gboolean image_set_rawcrops(const uint32_t imgid, int dx, int dy)
+static gboolean _image_set_rawcrops(
+        dt_iop_module_t *self,
+        const dt_imgid_t imgid,
+        const int left,
+        const int right,
+        const int top,
+        const int bottom)
 {
-  dt_image_t *img = NULL;
-  img = dt_image_cache_get(darktable.image_cache, imgid, 'r');
-  const gboolean test = (img->p_width == img->width - dx)
-                     && (img->p_height == img->height - dy);
+  dt_image_t *img = dt_image_cache_get(darktable.image_cache, imgid, 'r');
+
+  const gboolean cropvalid = (left >= 0) && (right >= 0) && (top >= 0) && (bottom >= 0)
+    && (left+right < img->width / 2) && (top + bottom < img->height /2);
+
+  const gboolean testdim =
+      (img->p_width == img->width - left - right)
+   && (img->p_height == img->height - top - bottom);
 
   dt_image_cache_read_release(darktable.image_cache, img);
-  if(test) return FALSE;
+  if(testdim && cropvalid) return FALSE;
+
+  if(!cropvalid)
+  {
+    dt_print
+      (DT_DEBUG_ALWAYS,
+       "[rawprepare] got wrong crop parameters left=%i, right=%i, top=%i, bottom=%i for size=%ix%i\n",
+       left, right, top, bottom, img->width, img->height);
+    dt_iop_set_module_trouble_message(self,
+     _("invalid crop parameters"),
+     _("please reset to defaults, update your preset or set to something correct"),
+       "invalid crop parameters");
+  }
+  else
+    dt_iop_set_module_trouble_message(self, NULL, NULL, NULL);
 
   img = dt_image_cache_get(darktable.image_cache, imgid, 'w');
-  img->p_width = img->width - dx;
-  img->p_height = img->height - dy;
+  img->p_width = img->width - (cropvalid ? left + right : 0);
+  img->p_height = img->height - (cropvalid ? top + bottom : 0);
   dt_image_cache_write_release(darktable.image_cache, img, DT_IMAGE_CACHE_RELAXED);
+
   return TRUE;
 }
 
 // check if image contains GainMaps of the exact type that we can apply here
 // we may reject some GainMaps that are valid according to Adobe DNG spec but we do not support
-gboolean check_gain_maps(dt_iop_module_t *self, dt_dng_gain_map_t **gainmaps_out)
+static gboolean _check_gain_maps(dt_iop_module_t *self, dt_dng_gain_map_t **gainmaps_out)
 {
   const dt_image_t *const image = &(self->dev->image_storage);
   dt_dng_gain_map_t *gainmaps[4] = {0};
@@ -601,19 +677,27 @@ gboolean check_gain_maps(dt_iop_module_t *self, dt_dng_gain_map_t **gainmaps_out
   if(g_list_length(image->dng_gain_maps) != 4)
     return FALSE;
 
+  // FIXME checks for witdh / height might be wrong
   for(int i = 0; i < 4; i++)
   {
     // check that each GainMap applies to one filter of a Bayer image,
     // covers the entire image, and is not a 1x1 no-op
     dt_dng_gain_map_t *g = (dt_dng_gain_map_t *)g_list_nth_data(image->dng_gain_maps, i);
-    if(g == NULL ||
-      g->plane != 0 || g->planes != 1 || g->map_planes != 1 ||
-      g->row_pitch != 2 || g->col_pitch != 2 ||
-      g->map_points_v < 2 || g->map_points_h < 2 ||
-      g->top > 1 || g->left > 1 ||
-      g->bottom != image->height || g->right != image->width)
+    if(g == NULL
+       || g->plane != 0
+       || g->planes != 1
+       || g->map_planes != 1
+       || g->row_pitch != 2
+       || g->col_pitch != 2
+       || g->map_points_v < 2
+       || g->map_points_h < 2
+       || g->top > 1
+       || g->left > 1
+       || g->bottom != image->height
+       || g->right != image->width)
       return FALSE;
-    uint32_t filter = ((g->top & 1) << 1) + (g->left & 1);
+
+    const uint32_t filter = ((g->top & 1) << 1) + (g->left & 1);
     gainmaps[filter] = g;
   }
 
@@ -624,12 +708,12 @@ gboolean check_gain_maps(dt_iop_module_t *self, dt_dng_gain_map_t **gainmaps_out
   // check that each GainMap has the same shape
   for(int i = 1; i < 4; i++)
   {
-    if(gainmaps[i]->map_points_h != gainmaps[0]->map_points_h ||
-      gainmaps[i]->map_points_v != gainmaps[0]->map_points_v ||
-      gainmaps[i]->map_spacing_h != gainmaps[0]->map_spacing_h ||
-      gainmaps[i]->map_spacing_v != gainmaps[0]->map_spacing_v ||
-      gainmaps[i]->map_origin_h != gainmaps[0]->map_origin_h ||
-      gainmaps[i]->map_origin_v != gainmaps[0]->map_origin_v)
+    if(gainmaps[i]->map_points_h != gainmaps[0]->map_points_h
+       || gainmaps[i]->map_points_v != gainmaps[0]->map_points_v
+       || gainmaps[i]->map_spacing_h != gainmaps[0]->map_spacing_h
+       || gainmaps[i]->map_spacing_v != gainmaps[0]->map_spacing_v
+       || gainmaps[i]->map_origin_h != gainmaps[0]->map_origin_h
+       || gainmaps[i]->map_origin_v != gainmaps[0]->map_origin_v)
       return FALSE;
   }
 
@@ -639,16 +723,19 @@ gboolean check_gain_maps(dt_iop_module_t *self, dt_dng_gain_map_t **gainmaps_out
   return TRUE;
 }
 
-void commit_params(dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelpipe_t *pipe,
-                   dt_dev_pixelpipe_iop_t *piece)
+void commit_params(
+        dt_iop_module_t *self,
+        dt_iop_params_t *params,
+        dt_dev_pixelpipe_t *pipe,
+        dt_dev_pixelpipe_iop_t *piece)
 {
   const dt_iop_rawprepare_params_t *const p = (dt_iop_rawprepare_params_t *)params;
   dt_iop_rawprepare_data_t *d = (dt_iop_rawprepare_data_t *)piece->data;
 
-  d->x = p->x;
-  d->y = p->y;
-  d->width = p->width;
-  d->height = p->height;
+  d->left = p->left;
+  d->top = p->top;
+  d->right = p->right;
+  d->bottom = p->bottom;
 
   if(piece->pipe->dsc.filters)
   {
@@ -662,20 +749,16 @@ void commit_params(dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelp
   }
   else
   {
-    const float normalizer
-        = ((piece->pipe->image.flags & DT_IMAGE_HDR) == DT_IMAGE_HDR) ? 1.0f : (float)UINT16_MAX;
-    const float white = (float)p->raw_white_point / normalizer;
-    float black = 0;
-    for(int i = 0; i < 4; i++)
-    {
-      black += p->raw_black_level_separate[i] / normalizer;
-    }
-    black /= 4.0f;
+    const float normalizer =
+      ((piece->pipe->image.flags & DT_IMAGE_HDR) == DT_IMAGE_HDR)
+      ? 1.0f
+      : (float)UINT16_MAX;
 
+    const float white = (float)p->raw_white_point / normalizer;
     for(int i = 0; i < 4; i++)
     {
-      d->sub[i] = black;
-      d->div[i] = (white - black);
+      d->sub[i] = (float)p->raw_black_level_separate[i] / normalizer;
+      d->div[i] = (white - d->sub[i]);
     }
   }
 
@@ -684,23 +767,23 @@ void commit_params(dt_iop_module_t *self, dt_iop_params_t *params, dt_dev_pixelp
   {
     black += (float)p->raw_black_level_separate[i];
   }
-  d->rawprepare.raw_black_level = (uint16_t)(black / 4.0f);
+  d->rawprepare.raw_black_level = (uint16_t)roundf(black / 4.0f);
   d->rawprepare.raw_white_point = p->raw_white_point;
 
   if(p->flat_field == FLAT_FIELD_EMBEDDED)
-    d->apply_gainmaps = check_gain_maps(self, d->gainmaps);
+    d->apply_gainmaps = _check_gain_maps(self, d->gainmaps);
   else
     d->apply_gainmaps = FALSE;
 
-  if(image_set_rawcrops(pipe->image.id, d->x + d->width, d->y + d->height))
+   if(_image_set_rawcrops(self, pipe->image.id, d->left, d->right, d->top, d->bottom))
     DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_METADATA_UPDATE);
 
   if(!(dt_image_is_rawprepare_supported(&piece->pipe->image))
-     || image_is_normalized(&piece->pipe->image))
-    piece->enabled = 0;
+     || _image_is_normalized(&piece->pipe->image))
+    piece->enabled = FALSE;
 
-  if(piece->pipe->want_detail_mask == (DT_DEV_DETAIL_MASK_REQUIRED | DT_DEV_DETAIL_MASK_RAWPREPARE))
-    piece->process_tiling_ready = 0;
+  if(piece->pipe->want_detail_mask)
+    piece->process_tiling_ready = FALSE;
 }
 
 void init_pipe(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -720,12 +803,12 @@ void reload_defaults(dt_iop_module_t *self)
   const dt_image_t *const image = &(self->dev->image_storage);
 
   // if there are embedded GainMaps, they should be applied by default to avoid uneven color cast
-  gboolean has_gainmaps = check_gain_maps(self, NULL);
+  const gboolean has_gainmaps = _check_gain_maps(self, NULL);
 
-  *d = (dt_iop_rawprepare_params_t){.x = image->crop_x,
-                                    .y = image->crop_y,
-                                    .width = image->crop_width,
-                                    .height = image->crop_height,
+  *d = (dt_iop_rawprepare_params_t){.left = image->crop_x,
+                                    .top = image->crop_y,
+                                    .right = image->crop_right,
+                                    .bottom = image->crop_bottom,
                                     .raw_black_level_separate[0] = image->raw_black_level_separate[0],
                                     .raw_black_level_separate[1] = image->raw_black_level_separate[1],
                                     .raw_black_level_separate[2] = image->raw_black_level_separate[2],
@@ -733,8 +816,8 @@ void reload_defaults(dt_iop_module_t *self)
                                     .raw_white_point = image->raw_white_point,
                                     .flat_field = has_gainmaps ? FLAT_FIELD_EMBEDDED : FLAT_FIELD_OFF };
 
-  self->hide_enable_button = 1;
-  self->default_enabled = dt_image_is_rawprepare_supported(image) && !image_is_normalized(image);
+  self->hide_enable_button = TRUE;
+  self->default_enabled = dt_image_is_rawprepare_supported(image) && !_image_is_normalized(image);
 
   if(self->widget)
     gtk_stack_set_visible_child_name(GTK_STACK(self->widget), self->default_enabled ? "raw" : "non_raw");
@@ -768,7 +851,9 @@ void gui_update(dt_iop_module_t *self)
   dt_iop_rawprepare_gui_data_t *g = (dt_iop_rawprepare_gui_data_t *)self->gui_data;
   dt_iop_rawprepare_params_t *p = (dt_iop_rawprepare_params_t *)self->params;
 
-  const gboolean is_monochrome = (self->dev->image_storage.flags & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)) != 0;
+  const gboolean is_monochrome =
+    (self->dev->image_storage.flags & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)) != 0;
+
   if(is_monochrome)
   {
     // we might have to deal with old edits, so get average first
@@ -780,11 +865,24 @@ void gui_update(dt_iop_module_t *self)
       dt_bauhaus_slider_set(g->black_level_separate[i], av / 4);
   }
 
-  // don't show upper three black levels for monochromes
-  for(int i = 1; i < 4; i++)
+  const gboolean is_sraw = (self->dev->image_storage.flags & DT_IMAGE_S_RAW) != 0;
+
+  // monochromes also have the sRaw flag set
+  if(is_sraw && !is_monochrome)
+  {
+    // we might have to deal with old edits, so copy
+    for(int i = 0; i < 4; i++)
+      if(p->raw_black_level_separate[i] == 0)
+        dt_bauhaus_slider_set(g->black_level_separate[i], p->raw_black_level_separate[0]);
+  }
+
+  // don't show upper three black levels for monochromes, nor last one for sRaws
+  for(int i = 1; i < 3; i++)
     gtk_widget_set_visible(g->black_level_separate[i], !is_monochrome);
 
-  gtk_widget_set_visible(g->flat_field, check_gain_maps(self, NULL));
+  gtk_widget_set_visible(g->black_level_separate[3], !(is_monochrome || is_sraw));
+
+  gtk_widget_set_visible(g->flat_field, _check_gain_maps(self, NULL));
   dt_bauhaus_combobox_set(g->flat_field, p->flat_field);
 }
 
@@ -793,7 +891,9 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
   dt_iop_rawprepare_gui_data_t *g = (dt_iop_rawprepare_gui_data_t *)self->gui_data;
   dt_iop_rawprepare_params_t *p = (dt_iop_rawprepare_params_t *)self->params;
 
-  const gboolean is_monochrome = (self->dev->image_storage.flags & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)) != 0;
+  const gboolean is_monochrome =
+    (self->dev->image_storage.flags & (DT_IMAGE_MONOCHROME | DT_IMAGE_MONOCHROME_BAYER)) != 0;
+
   if(is_monochrome)
   {
     if(w == g->black_level_separate[0])
@@ -803,6 +903,19 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
         dt_bauhaus_slider_set(g->black_level_separate[i], val);
     }
   }
+
+  const gboolean is_sraw = (self->dev->image_storage.flags & DT_IMAGE_S_RAW) != 0;
+
+  // monochromes also have the sRaw flag set
+  if(is_sraw && !is_monochrome)
+  {
+    if(w == g->black_level_separate[0] || w == g->black_level_separate[1] || w == g->black_level_separate[2])
+    {
+      const float sum = (float)p->raw_black_level_separate[0] + (float)p->raw_black_level_separate[1]
+                        + (float)p->raw_black_level_separate[2];
+      dt_bauhaus_slider_set(g->black_level_separate[3], (uint16_t)roundf(sum / 3.0f));
+    }
+  }
 }
 
 const gchar *black_label[]
@@ -834,35 +947,38 @@ void gui_init(dt_iop_module_t *self)
   dt_bauhaus_slider_set_soft_max(g->white_point, 16384);
 
   g->flat_field = dt_bauhaus_combobox_from_params(self, "flat_field");
-  gtk_widget_set_tooltip_text(g->flat_field, _("flat field correction to compensate for lens shading"));
+  gtk_widget_set_tooltip_text
+    (g->flat_field,
+     _("raw flat field correction to compensate for lens shading"));
 
   if(dt_conf_get_bool("plugins/darkroom/rawprepare/allow_editing_crop"))
   {
     gtk_box_pack_start(GTK_BOX(self->widget),
-                       dt_ui_section_label_new(_("crop")), FALSE, FALSE, 0);
+                       dt_ui_section_label_new(C_("section", "crop")), FALSE, FALSE, 0);
 
-    g->x = dt_bauhaus_slider_from_params(self, "x");
-    gtk_widget_set_tooltip_text(g->x, _("crop from left border"));
-    dt_bauhaus_slider_set_soft_max(g->x, 256);
+    g->left = dt_bauhaus_slider_from_params(self, "left");
+    gtk_widget_set_tooltip_text(g->left, _("crop left border"));
+    dt_bauhaus_slider_set_soft_max(g->left, 256);
 
-    g->y = dt_bauhaus_slider_from_params(self, "y");
-    gtk_widget_set_tooltip_text(g->y, _("crop from top"));
-    dt_bauhaus_slider_set_soft_max(g->y, 256);
+    g->top = dt_bauhaus_slider_from_params(self, "top");
+    gtk_widget_set_tooltip_text(g->top, _("crop top border"));
+    dt_bauhaus_slider_set_soft_max(g->top, 256);
 
-    g->width = dt_bauhaus_slider_from_params(self, "width");
-    gtk_widget_set_tooltip_text(g->width, _("crop from right border"));
-    dt_bauhaus_slider_set_soft_max(g->width, 256);
+    g->right = dt_bauhaus_slider_from_params(self, "right");
+    gtk_widget_set_tooltip_text(g->right, _("crop right border"));
+    dt_bauhaus_slider_set_soft_max(g->right, 256);
 
-    g->height = dt_bauhaus_slider_from_params(self, "height");
-    gtk_widget_set_tooltip_text(g->height, _("crop from bottom"));
-    dt_bauhaus_slider_set_soft_max(g->height, 256);
+    g->bottom = dt_bauhaus_slider_from_params(self, "bottom");
+    gtk_widget_set_tooltip_text(g->bottom, _("crop bottom border"));
+    dt_bauhaus_slider_set_soft_max(g->bottom, 256);
   }
 
   // start building top level widget
   self->widget = gtk_stack_new();
   gtk_stack_set_homogeneous(GTK_STACK(self->widget), FALSE);
 
-  GtkWidget *label_non_raw = dt_ui_label_new(_("raw black/white point correction\nonly works for the sensors that need it."));
+  GtkWidget *label_non_raw =
+    dt_ui_label_new(_("raw black/white point correction\nonly works for the sensors that need it."));
 
   gtk_stack_add_named(GTK_STACK(self->widget), label_non_raw, "non_raw");
   gtk_stack_add_named(GTK_STACK(self->widget), box_raw, "raw");
