@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2019-2022 darktable developers.
+    Copyright (C) 2019-2023 darktable developers.
 
 
     darktable is free software: you can redistribute it and/or modify
@@ -19,7 +19,6 @@
 #include "bauhaus/bauhaus.h"
 #include "common/darktable.h"
 #include "common/debug.h"
-#include "common/imageio_module.h"
 #include "control/conf.h"
 #include "control/control.h"
 #include "control/signal.h"
@@ -27,6 +26,7 @@
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "gui/gtkentry.h"
+#include "imageio/imageio_module.h"
 #include "libs/lib.h"
 #include "libs/lib_api.h"
 #ifdef GDK_WINDOWING_QUARTZ
@@ -286,6 +286,8 @@ char *dt_lib_export_metadata_configuration_dialog(char *metadata_presets, const
   GtkWidget *win = dt_ui_main_window(darktable.gui->ui);
   GtkWidget *dialog = gtk_dialog_new_with_buttons(_("edit metadata exportation"), GTK_WINDOW(win), GTK_DIALOG_DESTROY_WITH_PARENT,
                                        _("cancel"), GTK_RESPONSE_NONE, _("save"), GTK_RESPONSE_ACCEPT, NULL);
+  dt_gui_dialog_add_help(GTK_DIALOG(dialog), "export_dialog");
+
   d->dialog = dialog;
   g_signal_connect(dialog, "key-press-event", G_CALLBACK(dt_handle_dialog_enter), NULL);
 
