@@ -164,7 +164,7 @@ static void _grouping_widget_init(dt_lib_filtering_rule_t *rule, const dt_collec
   _widgets_grouping_t *grouping = (_widgets_grouping_t *)g_malloc0(sizeof(_widgets_grouping_t));
   grouping->rule = rule;
 
-  DT_BAUHAUS_COMBOBOX_NEW_FULL(grouping->combo, self, NULL, N_("grouping filter"),
+  DT_BAUHAUS_COMBOBOX_NEW_FULL(grouping->combo, self, N_("rules"), N_("grouping"),
                                _("select the type of grouped image to filter"), 0, _grouping_changed,
                                grouping, N_("all images"), N_("ungrouped images"), N_("grouped images"),
                                N_("group leaders"), N_("group followers"));
