@@ -54,7 +54,7 @@ typedef struct dt_lib_image_t
       *copy_metadata_button, *paste_metadata_button, *clear_metadata_button,
       *ratings_flag, *colors_flag, *metadata_flag, *geotags_flag, *tags_flag;
   GtkWidget *page1; // saved here for lua extensions
-  int imageid;
+  dt_imgid_t imageid;
 } dt_lib_image_t;
 
 typedef enum dt_lib_metadata_id
@@ -72,10 +72,9 @@ const char *name(dt_lib_module_t *self)
   return _("selected image[s]");
 }
 
-const char **views(dt_lib_module_t *self)
+dt_view_type_flags_t views(dt_lib_module_t *self)
 {
-  static const char *v[] = {"lighttable", NULL};
-  return v;
+  return DT_VIEW_LIGHTTABLE;
 }
 
 uint32_t container(dt_lib_module_t *self)
@@ -87,15 +86,16 @@ uint32_t container(dt_lib_module_t *self)
  * if there is an expanded group, then they will be joined there, otherwise a new one will be created. */
 static void _group_helper_function(void)
 {
-  int new_group_id = darktable.gui->expanded_group_id;
+  dt_imgid_t new_group_id = darktable.gui->expanded_group_id;
   GList *imgs = NULL;
   sqlite3_stmt *stmt;
   DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "SELECT imgid FROM main.selected_images", -1,
                               &stmt, NULL);
   while(sqlite3_step(stmt) == SQLITE_ROW)
   {
-    int id = sqlite3_column_int(stmt, 0);
-    if(new_group_id == -1) new_group_id = id;
+    dt_imgid_t id = sqlite3_column_int(stmt, 0);
+    if(!dt_is_valid_imgid(new_group_id))
+      new_group_id = id;
     dt_grouping_add_to_group(new_group_id, id);
     imgs = g_list_prepend(imgs, GINT_TO_POINTER(id));
   }
@@ -104,7 +104,7 @@ static void _group_helper_function(void)
   if(darktable.gui->grouping)
     darktable.gui->expanded_group_id = new_group_id;
   else
-    darktable.gui->expanded_group_id = -1;
+    darktable.gui->expanded_group_id = NO_IMGID;
   dt_collection_update_query(darktable.collection, DT_COLLECTION_CHANGE_RELOAD, DT_COLLECTION_PROP_GROUPING, imgs);
   dt_control_queue_redraw_center();
 }
@@ -120,16 +120,16 @@ static void _ungroup_helper_function(void)
   {
     const int id = sqlite3_column_int(stmt, 0);
     const int new_group_id = dt_grouping_remove_from_group(id);
-    if(new_group_id != -1)
+    if(dt_is_valid_imgid(new_group_id))
     {
-      // new_group_id == -1 if image to be ungrouped was a single image and no change to any group was made
+      // new_!dt_is_valid_imgid(group_id) if image to be ungrouped was a single image and no change to any group was made
       imgs = g_list_prepend(imgs, GINT_TO_POINTER(id));
     }
   }
   sqlite3_finalize(stmt);
   if(imgs != NULL)
   {
-    darktable.gui->expanded_group_id = -1;
+    darktable.gui->expanded_group_id = NO_IMGID;
     dt_collection_update_query(darktable.collection, DT_COLLECTION_CHANGE_RELOAD, DT_COLLECTION_PROP_GROUPING,
                                g_list_reverse(imgs));
     dt_control_queue_redraw_center();
@@ -175,18 +175,17 @@ static void button_clicked(GtkWidget *widget, gpointer user_data)
     dt_control_refresh_exif();
 }
 
-static void _update(dt_lib_module_t *self)
+void gui_update(dt_lib_module_t *self)
 {
-  dt_lib_cancel_postponed_update(self);
   dt_lib_image_t *d = (dt_lib_image_t *)self->data;
   const int nbimgs = dt_act_on_get_images_nb(FALSE, FALSE);
 
   const gboolean act_on_any = (nbimgs > 0);
   const gboolean act_on_one = (nbimgs == 1);
   const gboolean act_on_mult = (nbimgs > 1);
-  const uint32_t selected_cnt = dt_collection_get_selected_count(darktable.collection);
+  const uint32_t selected_cnt = dt_collection_get_selected_count();
   const gboolean can_paste
-      = d->imageid > 0 && (act_on_mult || (act_on_one && (d->imageid != dt_act_on_get_main_image())));
+      = dt_is_valid_imgid(d->imageid) && (act_on_mult || (act_on_one && (d->imageid != dt_act_on_get_main_image())));
 
   gtk_widget_set_sensitive(GTK_WIDGET(d->remove_button), act_on_any);
   gtk_widget_set_sensitive(GTK_WIDGET(d->delete_button), act_on_any);
@@ -227,8 +226,8 @@ static void _update(dt_lib_module_t *self)
   else
   {
     // exact one image to act on
-    const int imgid = dt_act_on_get_main_image();
-    if(imgid >= 0)
+    const dt_imgid_t imgid = dt_act_on_get_main_image();
+    if(dt_is_valid_imgid(imgid))
     {
       dt_image_t *img = dt_image_cache_get(darktable.image_cache, imgid, 'r');
       const gboolean is_bw = (dt_image_monochrome_flags(img) != 0);
@@ -261,19 +260,19 @@ static void _update(dt_lib_module_t *self)
 
 static void _image_selection_changed_callback(gpointer instance, dt_lib_module_t *self)
 {
-  _update(self);
+  dt_lib_gui_queue_update(self);
 }
 
 static void _collection_updated_callback(gpointer instance, dt_collection_change_t query_change,
                                          dt_collection_properties_t changed_property, gpointer imgs, int next,
                                          dt_lib_module_t *self)
 {
-  _update(self);
+  dt_lib_gui_queue_update(self);
 }
 
 static void _mouse_over_image_callback(gpointer instance, dt_lib_module_t *self)
 {
-  dt_lib_queue_postponed_update(self, _update);
+  dt_lib_gui_queue_update(self);
 }
 
 static void _image_preference_changed(gpointer instance, gpointer user_data)
@@ -310,7 +309,7 @@ static void _execute_metadata(dt_lib_module_t *self, const int action)
   const gboolean dtmetadata_flag = dt_conf_get_bool("plugins/lighttable/copy_metadata/metadata");
   const gboolean geotag_flag = dt_conf_get_bool("plugins/lighttable/copy_metadata/geotags");
   const gboolean dttag_flag = dt_conf_get_bool("plugins/lighttable/copy_metadata/tags");
-  const int imageid = d->imageid;
+  const dt_imgid_t imageid = d->imageid;
   GList *imgs = dt_act_on_get_images(FALSE, TRUE, FALSE);
   if(imgs)
   {
@@ -384,7 +383,7 @@ static void copy_metadata_callback(GtkWidget *widget, dt_lib_module_t *self)
 
   d->imageid = dt_act_on_get_main_image();
 
-  _update(self);
+  dt_lib_gui_queue_update(self);
 }
 
 static void paste_metadata_callback(GtkWidget *widget, dt_lib_module_t *self)
@@ -455,12 +454,11 @@ void gui_init(dt_lib_module_t *self)
 {
   dt_lib_image_t *d = (dt_lib_image_t *)malloc(sizeof(dt_lib_image_t));
   self->data = (void *)d;
-  self->timeout_handle = 0;
 
   static struct dt_action_def_t notebook_def = { };
   self->widget = GTK_WIDGET(dt_ui_notebook_new(&notebook_def));
   dt_action_define(DT_ACTION(self), NULL, N_("page"), GTK_WIDGET(self->widget), &notebook_def);
-  dt_gui_add_help_link(self->widget, dt_get_help_url("image"));
+  dt_gui_add_help_link(self->widget, "image");
 
   GtkWidget *page1 = dt_ui_notebook_page(GTK_NOTEBOOK(self->widget), N_("images"), NULL);
   GtkWidget *page2 = dt_ui_notebook_page(GTK_NOTEBOOK(self->widget), N_("metadata"), NULL);
@@ -627,7 +625,6 @@ void gui_init(dt_lib_module_t *self)
   dt_action_register(DT_ACTION(self), N_("duplicate virgin"), _duplicate_virgin, GDK_KEY_d, GDK_CONTROL_MASK | GDK_SHIFT_MASK);
 
   d->imageid = 0;
-  _update(self);
   _image_preference_changed(NULL, self); // update delete button label/tooltip
 }
 #undef ellipsize_button
@@ -636,12 +633,11 @@ void gui_reset(dt_lib_module_t *self)
 {
   dt_lib_image_t *d = (dt_lib_image_t *)self->data;
   d->imageid = 0;
-  _update(self);
+  dt_lib_gui_queue_update(self);
 }
 
 void gui_cleanup(dt_lib_module_t *self)
 {
-  dt_lib_cancel_postponed_update(self);
   DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_image_preference_changed), self);
   DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_image_selection_changed_callback), self);
   DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_mouse_over_image_callback), self);
