@@ -39,10 +39,9 @@ const char *name(dt_lib_module_t *self)
   return _("map settings");
 }
 
-const char **views(dt_lib_module_t *self)
+dt_view_type_flags_t views(dt_lib_module_t *self)
 {
-  static const char *v[] = {"map", NULL};
-  return v;
+  return DT_VIEW_MAP;
 }
 
 uint32_t container(dt_lib_module_t *self)
