@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2011-2021 darktable developers.
+    Copyright (C) 2011-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "control/conf.h"
 #include "control/control.h"
 #include "develop/develop.h"
+#include "dtgtk/drawingarea.h"
 #include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "libs/lib.h"
@@ -36,36 +37,46 @@ typedef struct dt_lib_navigation_t
 {
   int dragging;
   int zoom_w, zoom_h;
+  GtkWidget *zoom;
 } dt_lib_navigation_t;
 
 
 /* expose function for navigation module */
-static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, gpointer user_data);
+static gboolean _lib_navigation_draw_callback(GtkWidget *widget,
+                                              cairo_t *crf,
+                                              gpointer user_data);
 /* motion notify callback handler*/
-static gboolean _lib_navigation_motion_notify_callback(GtkWidget *widget, GdkEventMotion *event,
+static gboolean _lib_navigation_motion_notify_callback(GtkWidget *widget,
+                                                       GdkEventMotion *event,
                                                        gpointer user_data);
 /* button press callback */
-static gboolean _lib_navigation_button_press_callback(GtkWidget *widget, GdkEventButton *event,
+static gboolean _lib_navigation_button_press_callback(GtkWidget *widget,
+                                                      GdkEvent *event,
                                                       gpointer user_data);
 /* button release callback */
-static gboolean _lib_navigation_button_release_callback(GtkWidget *widget, GdkEventButton *event,
+static gboolean _lib_navigation_button_release_callback(GtkWidget *widget,
+                                                        GdkEventButton *event,
                                                         gpointer user_data);
 /* leave notify callback */
-static gboolean _lib_navigation_leave_notify_callback(GtkWidget *widget, GdkEventCrossing *event,
+static gboolean _lib_navigation_leave_notify_callback(GtkWidget *widget,
+                                                      GdkEventCrossing *event,
                                                       gpointer user_data);
 
 /* helper function for position set */
-static void _lib_navigation_set_position(struct dt_lib_module_t *self, double x, double y, int wd, int ht);
+static void _lib_navigation_set_position(struct dt_lib_module_t *self,
+                                         const double x,
+                                         const double y,
+                                         const int wd,
+                                         const int ht);
 
 const char *name(dt_lib_module_t *self)
 {
   return _("navigation");
 }
 
-const char **views(dt_lib_module_t *self)
+dt_view_type_flags_t views(dt_lib_module_t *self)
 {
-  static const char *v[] = {"darkroom", NULL};
-  return v;
+  return DT_VIEW_DARKROOM;
 }
 
 uint32_t container(dt_lib_module_t *self)
@@ -84,10 +95,32 @@ int position(const dt_lib_module_t *self)
 }
 
 
-static void _lib_navigation_control_redraw_callback(gpointer instance, gpointer user_data)
+static void _lib_navigation_control_redraw_callback(gpointer instance,
+                                                    gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
-  dt_control_queue_redraw_widget(self->widget);
+  dt_lib_navigation_t *d = (dt_lib_navigation_t *)self->data;
+
+  dt_develop_t *dev = darktable.develop;
+
+  dt_dev_zoom_t zoom = dt_control_get_dev_zoom();
+  int closeup = dt_control_get_dev_closeup();
+  const float cur_scale = dt_dev_get_zoom_scale(dev, zoom, 1<<closeup, 0);
+
+  gchar *zoomline = zoom == DT_ZOOM_FIT ? g_strdup(_("fit"))
+                  : zoom == DT_ZOOM_FILL ? g_strdup(C_("navigationbox", "fill"))
+                  : 0.5 * dt_dev_get_zoom_scale(dev, DT_ZOOM_FIT, 1.0, 0)
+                    == dt_dev_get_zoom_scale(dev, DT_ZOOM_FREE, 1.0, 0)
+                         ? g_strdup(_("small"))
+                         : g_strdup_printf("%.0f%%", cur_scale * 100 * darktable.gui->ppd);
+  ++darktable.gui->reset;
+  dt_bauhaus_combobox_set(d->zoom, -1);
+  if(!dt_bauhaus_combobox_set_from_text(d->zoom, zoomline))
+    dt_bauhaus_combobox_set_text(d->zoom, zoomline);
+  --darktable.gui->reset;
+  g_free(zoomline);
+
+  gtk_widget_queue_draw(gtk_bin_get_child(GTK_BIN(self->widget)));
 }
 
 
@@ -102,6 +135,7 @@ static void _lib_navigation_collapse_callback(dt_action_t *action)
   dt_lib_set_visible(self, !visible);
 }
 
+static void _zoom_changed(GtkWidget *widget, gpointer user_data);
 
 void gui_init(dt_lib_module_t *self)
 {
@@ -110,35 +144,75 @@ void gui_init(dt_lib_module_t *self)
   self->data = (void *)d;
 
   /* create drawingarea */
-  self->widget = gtk_drawing_area_new();
-  gtk_widget_set_events(self->widget, GDK_EXPOSURE_MASK | GDK_ENTER_NOTIFY_MASK
-                                      | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK
-                                      | GDK_BUTTON_RELEASE_MASK | GDK_STRUCTURE_MASK);
+  GtkWidget *thumbnail = dt_ui_resize_wrap(NULL, 0,
+                                           "plugins/darkroom/navigation/aspect_percent");
+  gtk_widget_set_tooltip_text
+    (thumbnail,
+     _("navigation\nclick or drag to position zoomed area in center view"));
 
   /* connect callbacks */
-  gtk_widget_set_app_paintable(self->widget, TRUE);
-  g_signal_connect(G_OBJECT(self->widget), "draw", G_CALLBACK(_lib_navigation_draw_callback), self);
-  g_signal_connect(G_OBJECT(self->widget), "button-press-event",
+  gtk_widget_set_app_paintable(thumbnail, TRUE);
+  g_signal_connect(G_OBJECT(thumbnail), "draw",
+                   G_CALLBACK(_lib_navigation_draw_callback), self);
+  g_signal_connect(G_OBJECT(thumbnail), "button-press-event",
                    G_CALLBACK(_lib_navigation_button_press_callback), self);
-  g_signal_connect(G_OBJECT(self->widget), "button-release-event",
+  g_signal_connect(G_OBJECT(thumbnail), "scroll-event",
+                   G_CALLBACK(_lib_navigation_button_press_callback), self);
+  g_signal_connect(G_OBJECT(thumbnail), "button-release-event",
                    G_CALLBACK(_lib_navigation_button_release_callback), self);
-  g_signal_connect(G_OBJECT(self->widget), "motion-notify-event",
+  g_signal_connect(G_OBJECT(thumbnail), "motion-notify-event",
                    G_CALLBACK(_lib_navigation_motion_notify_callback), self);
-  g_signal_connect(G_OBJECT(self->widget), "leave-notify-event",
+  g_signal_connect(G_OBJECT(thumbnail), "leave-notify-event",
                    G_CALLBACK(_lib_navigation_leave_notify_callback), self);
 
   /* set size of navigation draw area */
-  gtk_widget_set_size_request(self->widget, -1, DT_PIXEL_APPLY_DPI(175));
-  gtk_widget_set_name(GTK_WIDGET(self->widget), "navigation-module");
-  dt_action_t *ac = dt_action_define(&darktable.view_manager->proxy.darkroom.view->actions, NULL,
-                                     N_("hide navigation thumbnail"), self->widget, NULL);
-  dt_action_register(ac, NULL, _lib_navigation_collapse_callback, GDK_KEY_N, GDK_CONTROL_MASK | GDK_SHIFT_MASK);
+  // gtk_widget_set_size_request(thumbnail, -1, DT_PIXEL_APPLY_DPI(175));
+  gtk_widget_set_name(GTK_WIDGET(thumbnail), "navigation-module");
+  dt_action_t *ac = dt_action_define(&darktable.view_manager->proxy.darkroom.view->actions,
+                                     NULL,
+                                     N_("hide navigation thumbnail"), thumbnail, NULL);
+  dt_action_register(ac, NULL, _lib_navigation_collapse_callback,
+                     GDK_KEY_N, GDK_CONTROL_MASK | GDK_SHIFT_MASK);
 
   /* connect a redraw callback to control draw all and preview pipe finish signals */
-  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED,
-                            G_CALLBACK(_lib_navigation_control_redraw_callback), self);
+  DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals,
+                                  DT_SIGNAL_DEVELOP_PREVIEW_PIPE_FINISHED,
+                                  G_CALLBACK(_lib_navigation_control_redraw_callback),
+                                  self);
   DT_DEBUG_CONTROL_SIGNAL_CONNECT(darktable.signals, DT_SIGNAL_CONTROL_NAVIGATION_REDRAW,
-                            G_CALLBACK(_lib_navigation_control_redraw_callback), self);
+                                  G_CALLBACK(_lib_navigation_control_redraw_callback),
+                                  self);
+
+  DT_BAUHAUS_COMBOBOX_NEW_FULL(d->zoom, darktable.view_manager->proxy.darkroom.view,
+                               NULL, N_("zoom"), _("image zoom level"),
+                               -1, _zoom_changed, NULL,
+                               N_("small"),
+                               N_("fit"),
+                               NC_("navigationbox", "fill"),
+                               N_("50%"),
+                               N_("100%"),
+                               N_("200%"),
+                               N_("400%"),
+                               N_("800%"),
+                               N_("1600%"));
+
+  ac = dt_action_section(&darktable.view_manager->proxy.darkroom.view->actions, N_("zoom"));
+  dt_shortcut_register(ac, 0, DT_ACTION_EFFECT_COMBO_SEPARATOR + 2,
+                       GDK_KEY_3, GDK_MOD1_MASK);
+  dt_shortcut_register(ac, 0, DT_ACTION_EFFECT_COMBO_SEPARATOR + 3,
+                       GDK_KEY_2, GDK_MOD1_MASK);
+
+  dt_bauhaus_combobox_set_editable(d->zoom, TRUE);
+  DT_BAUHAUS_WIDGET(d->zoom)->show_label = FALSE;
+  gtk_widget_set_halign(d->zoom, GTK_ALIGN_END);
+  gtk_widget_set_valign(d->zoom, GTK_ALIGN_END);
+  gtk_widget_set_name(d->zoom, "nav-zoom");
+
+  self->widget = gtk_overlay_new();
+  gtk_container_add(GTK_CONTAINER(self->widget), thumbnail);
+  gtk_overlay_add_overlay(GTK_OVERLAY(self->widget), d->zoom);
+  dt_gui_add_class(self->widget, "dt_plugin_ui_main");
+  gtk_widget_show_all(self->widget);
 
   darktable.lib->proxy.navigation.module = self;
 }
@@ -146,7 +220,9 @@ void gui_init(dt_lib_module_t *self)
 void gui_cleanup(dt_lib_module_t *self)
 {
   /* disconnect from signal */
-  DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_lib_navigation_control_redraw_callback), self);
+  DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals,
+                                     G_CALLBACK(_lib_navigation_control_redraw_callback),
+                                     self);
 
   g_free(self->data);
   self->data = NULL;
@@ -154,11 +230,10 @@ void gui_cleanup(dt_lib_module_t *self)
 
 
 
-static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, gpointer user_data)
+static gboolean _lib_navigation_draw_callback(GtkWidget *widget,
+                                              cairo_t *crf,
+                                              gpointer user_data)
 {
-  dt_lib_module_t *self = (dt_lib_module_t *)user_data;
-  dt_lib_navigation_t *d = (dt_lib_navigation_t *)self->data;
-
   GtkAllocation allocation;
   gtk_widget_get_allocation(widget, &allocation);
   int width = allocation.width, height = allocation.height;
@@ -173,7 +248,8 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
   gtk_render_background(context, cr, 0, 0, allocation.width, allocation.height);
 
   /* draw navigation image if available */
-  if(dev->preview_pipe->output_backbuf && dev->image_storage.id == dev->preview_pipe->output_imgid)
+  if(dev->preview_pipe->output_backbuf
+     && dev->image_storage.id == dev->preview_pipe->output_imgid)
   {
     dt_pthread_mutex_t *mutex = &dev->preview_pipe->backbuf_mutex;
     dt_pthread_mutex_lock(mutex);
@@ -185,7 +261,8 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
 
     const int stride = cairo_format_stride_for_width(CAIRO_FORMAT_RGB24, wd);
     cairo_surface_t *surface
-        = cairo_image_surface_create_for_data(dev->preview_pipe->output_backbuf, CAIRO_FORMAT_RGB24, wd, ht, stride);
+        = cairo_image_surface_create_for_data(dev->preview_pipe->output_backbuf,
+                                              CAIRO_FORMAT_RGB24, wd, ht, stride);
     cairo_translate(cr, width / 2.0, height / 2.0f);
     cairo_scale(cr, scale, scale);
     cairo_translate(cr, -.5f * wd, -.5f * ht);
@@ -202,8 +279,6 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
     float zoom_y = dt_control_get_dev_zoom_y();
     const float min_scale = dt_dev_get_zoom_scale(dev, DT_ZOOM_FIT, 1<<closeup, 0);
     const float cur_scale = dt_dev_get_zoom_scale(dev, zoom, 1<<closeup, 0);
-    // avoid numerical instability for small resolutions:
-    double h, w;
     if(cur_scale > min_scale)
     {
       // Add a dark overlay on the picture to make it fade
@@ -212,7 +287,8 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
       cairo_fill(cr);
 
       float boxw = 1, boxh = 1;
-      dt_dev_check_zoom_bounds(darktable.develop, &zoom_x, &zoom_y, zoom, closeup, &boxw, &boxh);
+      dt_dev_check_zoom_bounds(darktable.develop, &zoom_x, &zoom_y, zoom,
+                               closeup, &boxw, &boxh);
 
       // Repaint the original image in the area of interest
       cairo_set_source_surface(cr, surface, 0, 0);
@@ -235,94 +311,6 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
     }
     cairo_restore(cr);
 
-    PangoLayout *layout;
-    PangoRectangle ink;
-    PangoFontDescription *desc = pango_font_description_copy_static(darktable.bauhaus->pango_font_desc);
-    layout = pango_cairo_create_layout(cr);
-    const float fontsize = DT_PIXEL_APPLY_DPI(14);
-    pango_font_description_set_absolute_size(desc, fontsize * PANGO_SCALE);
-    pango_layout_set_font_description(layout, desc);
-
-    if(fabsf(cur_scale - min_scale) > 0.001f)
-    {
-      /* Zoom % */
-      cairo_translate(cr, 0, height);
-      cairo_set_source_rgba(cr, 1., 1., 1., 0.5);
-      cairo_set_line_join(cr, CAIRO_LINE_JOIN_ROUND);
-
-      char zoomline[6];
-      snprintf(zoomline, sizeof(zoomline), "%.0f%%", cur_scale * 100 * darktable.gui->ppd);
-
-      pango_layout_set_text(layout, zoomline, -1);
-      pango_layout_get_pixel_extents(layout, &ink, NULL);
-      h = d->zoom_h = ink.height;
-      w = d->zoom_w = ink.width;
-
-      cairo_move_to(cr, width - w - h * 1.1 - ink.x, - h - ink.y);
-
-      cairo_save(cr);
-      cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(1));
-
-      GdkRGBA *color;
-      gtk_style_context_get(context, gtk_widget_get_state_flags(widget), "background-color", &color, NULL);
-
-      gdk_cairo_set_source_rgba(cr, color);
-      pango_cairo_layout_path(cr, layout);
-      cairo_stroke_preserve(cr);
-      cairo_set_source_rgb(cr, 0.8, 0.8, 0.8);
-      cairo_fill(cr);
-      cairo_restore(cr);
-
-      gdk_rgba_free(color);
-    }
-    else
-    {
-      // draw the zoom-to-fit icon
-      cairo_translate(cr, 0, height);
-      cairo_set_source_rgb(cr, 0.8, 0.8, 0.8);
-      pango_layout_set_text(layout, "100%", -1); // dummy text, just to get the height
-      pango_layout_get_pixel_extents(layout, &ink, NULL);
-
-      h = d->zoom_h = ink.height;
-      w = h * 1.5;
-      float sp = h * 0.6;
-      d->zoom_w = w + sp;
-
-      cairo_move_to(cr, width - w - h - sp, -1.0 * h);
-      cairo_rectangle(cr, width - w - h - sp, -1.0 * h, w, h);
-      cairo_set_source_rgb(cr, 0.2, 0.2, 0.2);
-      cairo_fill(cr);
-
-      cairo_set_line_width(cr, DT_PIXEL_APPLY_DPI(2));
-
-      cairo_set_source_rgb(cr, 0.6, 0.6, 0.6);
-      cairo_move_to(cr, width - w * 0.8 - h - sp, -1.0 * h);
-      cairo_line_to(cr, width - w - h - sp, -1.0 * h);
-      cairo_line_to(cr, width - w - h - sp, -0.7 * h);
-      cairo_stroke(cr);
-      cairo_move_to(cr, width - w - h - sp, -0.3 * h);
-      cairo_line_to(cr, width - w - h - sp, 0);
-      cairo_line_to(cr, width - w * 0.8 - h - sp, 0);
-      cairo_stroke(cr);
-      cairo_move_to(cr, width - w * 0.2 - h - sp, 0);
-      cairo_line_to(cr, width - h - sp, 0);
-      cairo_line_to(cr, width - h - sp, -0.3 * h);
-      cairo_stroke(cr);
-      cairo_move_to(cr, width - h - sp, -0.7 * h);
-      cairo_line_to(cr, width - h - sp, -1.0 * h);
-      cairo_line_to(cr, width - w * 0.2 - h - sp, -1.0 * h);
-      cairo_stroke(cr);
-    }
-
-    pango_font_description_free(desc);
-    g_object_unref(layout);
-
-    cairo_move_to(cr, width - 0.95 * h, -0.9 * h);
-    cairo_line_to(cr, width - 0.05 * h, -0.9 * h);
-    cairo_line_to(cr, width - 0.5 * h, -0.1 * h);
-    cairo_fill(cr);
-    cairo_surface_destroy(surface);
-
     dt_pthread_mutex_unlock(mutex);
   }
 
@@ -335,34 +323,43 @@ static gboolean _lib_navigation_draw_callback(GtkWidget *widget, cairo_t *crf, g
   return TRUE;
 }
 
-void _lib_navigation_set_position(dt_lib_module_t *self, double x, double y, int wd, int ht)
+void _lib_navigation_set_position(dt_lib_module_t *self,
+                                  const double x,
+                                  const double y,
+                                  const int wd,
+                                  const int ht)
 {
   dt_lib_navigation_t *d = (dt_lib_navigation_t *)self->data;
 
   dt_dev_zoom_t zoom = dt_control_get_dev_zoom();
-  int closeup = dt_control_get_dev_closeup();
-  float zoom_x = dt_control_get_dev_zoom_x();
-  float zoom_y = dt_control_get_dev_zoom_y();
+  const int closeup = dt_control_get_dev_closeup();
 
   if(d->dragging && zoom != DT_ZOOM_FIT)
   {
     const int inset = DT_NAVIGATION_INSET;
-    const float width = wd - 2 * inset, height = ht - 2 * inset;
+    const float width = wd - 2 * inset;
+    const float height = ht - 2 * inset;
     const dt_develop_t *dev = darktable.develop;
     int iwd, iht;
     dt_dev_get_processed_size(dev, &iwd, &iht);
-    zoom_x = fmaxf(
-        -.5,
-        fminf(((x - inset) / width - .5f) / (iwd * fminf(wd / (float)iwd, ht / (float)iht) / (float)wd), .5));
-    zoom_y = fmaxf(
-        -.5, fminf(((y - inset) / height - .5f) / (iht * fminf(wd / (float)iwd, ht / (float)iht) / (float)ht),
-                   .5));
-    dt_dev_check_zoom_bounds(darktable.develop, &zoom_x, &zoom_y, zoom, closeup, NULL, NULL);
+
+    float zoom_x = fmaxf(
+      -.5,
+      fminf(((x - inset) / width - .5f) / (iwd * fminf(wd / (float)iwd,
+                                                       ht / (float)iht) / (float)wd),
+            .5));
+    float zoom_y = fmaxf(
+      -.5,
+      fminf(((y - inset) / height - .5f) / (iht * fminf(wd / (float)iwd,
+                                                        ht / (float)iht) / (float)ht),
+            .5));
+    dt_dev_check_zoom_bounds(darktable.develop, &zoom_x, &zoom_y,
+                             zoom, closeup, NULL, NULL);
     dt_control_set_dev_zoom_x(zoom_x);
     dt_control_set_dev_zoom_y(zoom_y);
 
     /* redraw myself */
-    gtk_widget_queue_draw(self->widget);
+    _lib_navigation_control_redraw_callback(NULL, self);
 
     /* redraw pipe */
     dt_dev_invalidate(darktable.develop);
@@ -370,18 +367,24 @@ void _lib_navigation_set_position(dt_lib_module_t *self, double x, double y, int
   }
 }
 
-static gboolean _lib_navigation_motion_notify_callback(GtkWidget *widget, GdkEventMotion *event,
+static gboolean _lib_navigation_motion_notify_callback(GtkWidget *widget,
+                                                       GdkEventMotion *event,
                                                        gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   GtkAllocation allocation;
   gtk_widget_get_allocation(widget, &allocation);
-  _lib_navigation_set_position(self, event->x, event->y, allocation.width, allocation.height);
+  _lib_navigation_set_position(self, event->x, event->y,
+                               allocation.width, allocation.height);
   return TRUE;
 }
 
-static void _zoom_preset_change(uint64_t val)
+static void _zoom_changed(GtkWidget *widget, gpointer user_data)
 {
+  int val = dt_bauhaus_combobox_get(widget);
+  if(val == -1 && 1 != sscanf(dt_bauhaus_combobox_get_text(widget), "%d", &val))
+    return;
+
   // dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_develop_t *dev = darktable.develop;
   if(!dev) return;
@@ -403,13 +406,19 @@ static void _zoom_preset_change(uint64_t val)
     scale = 0.5 * dt_dev_get_zoom_scale(dev, DT_ZOOM_FIT, 1.0, 0);
     zoom = DT_ZOOM_FREE;
   }
-  else if(val == 1u)
+  else if(val == 1u || val == -1u)
   {
     // fit to screen
     zoom = DT_ZOOM_FIT;
     scale = dt_dev_get_zoom_scale(dev, DT_ZOOM_FIT, 1.0, 0);
   }
   else if(val == 2u)
+  {
+    // fit to screen
+    zoom = DT_ZOOM_FILL;
+    closeup = 0;
+  }
+  else if(val == 4u)
   {
     // 100%
     if(low_ppd == 1)
@@ -423,20 +432,20 @@ static void _zoom_preset_change(uint64_t val)
       zoom = DT_ZOOM_FREE;
     }
   }
-  else if(val == 3u)
+  else if(val == 5u)
   {
     // 200%
     scale = dt_dev_get_zoom_scale(dev, DT_ZOOM_1, 1.0, 0);
     zoom = DT_ZOOM_1;
     if(low_ppd) closeup = 1;
   }
-  else if(val == 4u)
+  else if(val == 3u)
   {
     // 50%
     scale = 0.5f / ppd;
     zoom = DT_ZOOM_FREE;
   }
-  else if(val == 5u)
+  else if(val == 8u)
   {
     // 1600%
     scale = dt_dev_get_zoom_scale(dev, DT_ZOOM_1, 1.0, 0);
@@ -457,6 +466,11 @@ static void _zoom_preset_change(uint64_t val)
     zoom = DT_ZOOM_1;
     closeup = (low_ppd) ? 3 : 2;
   }
+  else
+  {
+    scale = val / 100.0f * ppd;
+    zoom = DT_ZOOM_FREE;
+  }
 
   // zoom_x = (1.0/(scale*(1<<closeup)))*(zoom_x - .5f*dev->width )/procw;
   // zoom_y = (1.0/(scale*(1<<closeup)))*(zoom_y - .5f*dev->height)/proch;
@@ -468,74 +482,40 @@ static void _zoom_preset_change(uint64_t val)
   dt_control_set_dev_zoom_x(zoom_x);
   dt_control_set_dev_zoom_y(zoom_y);
   dt_dev_invalidate(dev);
-  dt_control_queue_redraw();
+  dt_control_queue_redraw_center();
+  dt_control_navigation_redraw();
 }
 
-static void _zoom_preset_callback(GtkButton *button, gpointer user_data)
-{
-  _zoom_preset_change((uint64_t)user_data);
-}
-
-static gboolean _lib_navigation_button_press_callback(GtkWidget *widget, GdkEventButton *event,
+static gboolean _lib_navigation_button_press_callback(GtkWidget *widget,
+                                                      GdkEvent *event,
                                                       gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_navigation_t *d = (dt_lib_navigation_t *)self->data;
-
   GtkAllocation allocation;
   gtk_widget_get_allocation(widget, &allocation);
-  const int w = allocation.width;
-  const int h = allocation.height;
-
-  if(event->x >= w - DT_NAVIGATION_INSET - d->zoom_h - d->zoom_w
-     && event->y >= h - DT_NAVIGATION_INSET - d->zoom_h)
+  if(event->type == GDK_BUTTON_PRESS && event->button.button != 2)
   {
-    // we show the zoom menu
-    GtkMenuShell *menu = GTK_MENU_SHELL(gtk_menu_new());
-    GtkWidget *item;
-
-    item = gtk_menu_item_new_with_label(_("small"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)0);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("fit to screen"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)1);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("50%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)4);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("100%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)2);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("200%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)3);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("400%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)6);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("800%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)7);
-    gtk_menu_shell_append(menu, item);
-
-    item = gtk_menu_item_new_with_label(_("1600%"));
-    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(_zoom_preset_callback), (gpointer)5);
-    gtk_menu_shell_append(menu, item);
-
-    dt_gui_menu_popup(GTK_MENU(menu), NULL, 0, 0);
+    d->dragging = 1;
+    _lib_navigation_set_position(self, event->button.x, event->button.y,
+                                 allocation.width, allocation.height);
 
     return TRUE;
   }
-  d->dragging = 1;
-  _lib_navigation_set_position(self, event->x, event->y, w, h);
-  return TRUE;
+  else
+  {
+    GtkWidget *center = dt_ui_center(darktable.gui->ui);
+    GtkAllocation center_alloc;
+    gtk_widget_get_allocation(center, &center_alloc);
+    event->button.x *= (gdouble)center_alloc.width / allocation.width;
+    event->button.y *= (gdouble)center_alloc.height / allocation.height;
+
+    return gtk_widget_event(center, event);
+  }
 }
 
-static gboolean _lib_navigation_button_release_callback(GtkWidget *widget, GdkEventButton *event,
+static gboolean _lib_navigation_button_release_callback(GtkWidget *widget,
+                                                        GdkEventButton *event,
                                                         gpointer user_data)
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
@@ -545,7 +525,8 @@ static gboolean _lib_navigation_button_release_callback(GtkWidget *widget, GdkEv
   return TRUE;
 }
 
-static gboolean _lib_navigation_leave_notify_callback(GtkWidget *widget, GdkEventCrossing *event,
+static gboolean _lib_navigation_leave_notify_callback(GtkWidget *widget,
+                                                      GdkEventCrossing *event,
                                                       gpointer user_data)
 {
   return TRUE;
