@@ -38,14 +38,15 @@ DT_MODULE(1)
 /** this module stores recently used image collection queries and displays
  * them as one-click buttons to the user. */
 
-static int _conf_get_max_items()
+static int _conf_get_max_saved_items()
 {
-  return (dt_conf_get_int("plugins/lighttable/recentcollect/max_items"));
+  return (MAX(dt_conf_get_int("plugins/lighttable/recentcollect/max_items"),
+              dt_conf_get_int("plugins/lighttable/collect/history_max")));
 }
 
-static int _conf_get_num_items()
+static int _conf_get_max_shown_items()
 {
-  return (dt_conf_get_int("plugins/lighttable/recentcollect/num_items"));
+  return (dt_conf_get_int("plugins/lighttable/recentcollect/max_items"));
 }
 
 typedef struct dt_lib_recentcollect_item_t
@@ -67,10 +68,9 @@ const char *name(dt_lib_module_t *self)
   return _("recently used collections");
 }
 
-const char **views(dt_lib_module_t *self)
+dt_view_type_flags_t views(dt_lib_module_t *self)
 {
-  static const char *v[] = { "lighttable", "map", NULL };
-  return v;
+  return DT_VIEW_LIGHTTABLE | DT_VIEW_MAP;
 }
 
 uint32_t container(dt_lib_module_t *self)
@@ -160,14 +160,16 @@ static void _button_pressed(GtkButton *button, gpointer user_data)
   if(!found) return;
 
   char confname[200];
-  snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", linenumber);
+  snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history_pos%1d", linenumber);
+  const int pos = dt_conf_get_int(confname);
+  snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history%1d", linenumber);
   const char *line = dt_conf_get_string_const(confname);
   if(line)
   {
+    // we store the wanted offset which will be set by thumbtable on collection_change signal
+    dt_conf_set_int("plugins/lighttable/collect/history_next_pos", pos);
+
     dt_collection_deserialize(line, FALSE);
-    // position will be updated when the list of recent collections is.
-    // that way it'll also catch cases when this is triggered by a signal,
-    // not only our button press here.
   }
 }
 
@@ -177,87 +179,15 @@ static void _lib_recentcollection_updated(gpointer instance, dt_collection_chang
 {
   dt_lib_module_t *self = (dt_lib_module_t *)user_data;
   dt_lib_recentcollect_t *d = (dt_lib_recentcollect_t *)self->data;
-  dt_thumbtable_t *table = dt_ui_thumbtable(darktable.gui->ui);
-  // serialize, check for recently used
-  char confname[200] = { 0 };
 
-  char buf[4096];
-  if(dt_collection_serialize(buf, sizeof(buf), FALSE)) return;
-
-  // is the current position, i.e. the one to be stored with the old collection (pos0, pos1-to-be)
-  uint32_t curr_pos = table->offset;
-  uint32_t new_pos = -1;
-
-  if(!d->inited)
-  {
-    new_pos = dt_conf_get_int("plugins/lighttable/recentcollect/pos0");
-    d->inited = 1;
-    dt_thumbtable_set_offset(table, new_pos, TRUE);
-  }
-  else if(curr_pos != -1)
-  {
-    dt_conf_set_int("plugins/lighttable/recentcollect/pos0", curr_pos);
-  }
-
-  int n = -1;
-  for(int k = 0; k < CLAMPS(_conf_get_num_items(), 0, _conf_get_max_items()); k++)
-  {
-    // is it already in the current list?
-    snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", k);
-    const char *line = dt_conf_get_string_const(confname);
-    if(!line) continue;
-    if(!strcmp(line, buf))
-    {
-      snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/pos%1d", k);
-      new_pos = dt_conf_get_int(confname);
-      n = k;
-      break;
-    }
-  }
-  if(n < 0)
-  {
-    const int num_items = CLAMPS(_conf_get_num_items(), 0, _conf_get_max_items());
-
-    if(num_items < _conf_get_max_items())
-    {
-      // new, unused entry
-      n = num_items;
-      dt_conf_set_int("plugins/lighttable/recentcollect/num_items", num_items + 1);
-    }
-    else
-    {
-      // kill least recently used entry:
-      n = num_items - 1;
-    }
-  }
-  if(n >= 0 && n < _conf_get_max_items())
-  {
-    // sort n to the top
-    for(int k = n; k > 0; k--)
-    {
-      snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", k - 1);
-      const gchar *line1 = dt_conf_get_string_const(confname);
-      snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/pos%1d", k - 1);
-      uint32_t pos1 = dt_conf_get_int(confname);
-      if(line1 && line1[0] != '\0')
-      {
-        snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", k);
-        dt_conf_set_string(confname, line1);
-        snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/pos%1d", k);
-        dt_conf_set_int(confname, pos1);
-      }
-    }
-    dt_conf_set_string("plugins/lighttable/recentcollect/line0", buf);
-    dt_conf_set_int("plugins/lighttable/recentcollect/pos0",
-                    (new_pos != -1 ? new_pos : (curr_pos != -1 ? curr_pos : 0)));
-  }
   // update button descriptions:
+  char confname[200] = { 0 };
   GList *current = d->items;
   for(int k = 0; current; k++)
   {
     char str[2048] = { 0 };
     dt_lib_recentcollect_item_t *item = (dt_lib_recentcollect_item_t *)current->data;
-    snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", k);
+    snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history%1d", k);
     const char *line2 = dt_conf_get_string_const(confname);
     if(line2 && line2[0] != '\0') pretty_print(line2, str, sizeof(str));
     gtk_widget_set_tooltip_text(item->button, str);
@@ -276,15 +206,17 @@ static void _lib_recentcollection_updated(gpointer instance, dt_collection_chang
   }
 
   current = d->items;
-  for(int k = 0; k < CLAMPS(_conf_get_num_items(), 0, _conf_get_max_items()); k++)
+  for(int k = 0; k < CLAMPS(_conf_get_max_shown_items(), 0, _conf_get_max_saved_items()); k++)
   {
     dt_lib_recentcollect_item_t *item = (dt_lib_recentcollect_item_t *)current->data;
-    gtk_widget_set_no_show_all(item->button, FALSE);
-    gtk_widget_set_visible(item->button, TRUE);
+    const gchar *line = gtk_button_get_label(GTK_BUTTON(item->button));
+    if(line && line[0] != '\0')
+    {
+      gtk_widget_set_no_show_all(item->button, FALSE);
+      gtk_widget_set_visible(item->button, TRUE);
+    }
     current = g_list_next(current);
   }
-
-  dt_thumbtable_set_offset(table, new_pos, TRUE);
 }
 
 void _menuitem_preferences(GtkMenuItem *menuitem, dt_lib_module_t *self)
@@ -302,13 +234,13 @@ void _menuitem_preferences(GtkMenuItem *menuitem, dt_lib_module_t *self)
 #endif
   gtk_widget_show_all(dialog);
 
-  const int old_nb_items = _conf_get_max_items(); // preserve previous value
+  const int old_nb_items = _conf_get_max_saved_items(); // preserve previous value
 
   if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
   {
     dt_lib_recentcollect_t *d = self->data;
 
-    const int new_nb_items = _conf_get_max_items();
+    const int new_nb_items = _conf_get_max_saved_items();
     const int delta = new_nb_items - old_nb_items;
     if(delta < 0)
     {
@@ -317,9 +249,9 @@ void _menuitem_preferences(GtkMenuItem *menuitem, dt_lib_module_t *self)
       while(current)
       {
         dt_lib_recentcollect_item_t *item = (dt_lib_recentcollect_item_t *)current->data;
-        snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", item->confid);
+        snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history%1d", item->confid);
         dt_conf_set_string(confname, "");
-        snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/pos%1d", item->confid);
+        snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history_pos%1d", item->confid);
         dt_conf_set_int(confname, 0);
         gtk_widget_destroy(item->button);
         free(item);
@@ -327,8 +259,6 @@ void _menuitem_preferences(GtkMenuItem *menuitem, dt_lib_module_t *self)
         current = g_list_next(current);
         d->items = g_list_delete_link(d->items, old);
       }
-      const int num_items = dt_conf_get_int("plugins/lighttable/recentcollect/num_items");
-      dt_conf_set_int("plugins/lighttable/recentcollect/num_items", MIN(num_items, new_nb_items));
     }
     if(delta > 0)
     {
@@ -367,14 +297,13 @@ void set_preferences(void *menu, dt_lib_module_t *self)
 
 void gui_reset(dt_lib_module_t *self)
 {
-  dt_conf_set_int("plugins/lighttable/recentcollect/num_items", 0);
   char confname[200] = { 0 };
 
-  for(int k = 0; k < _conf_get_max_items(); k++)
+  for(int k = 0; k < _conf_get_max_saved_items(); k++)
   {
-    snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/line%1d", k);
+    snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history%1d", k);
     dt_conf_set_string(confname, "");
-    snprintf(confname, sizeof(confname), "plugins/lighttable/recentcollect/pos%1d", k);
+    snprintf(confname, sizeof(confname), "plugins/lighttable/collect/history_pos%1d", k);
     dt_conf_set_int(confname, 0);
   }
   _lib_recentcollection_updated(NULL, DT_COLLECTION_CHANGE_NEW_QUERY, DT_COLLECTION_PROP_UNDEF, NULL, -1, self);
@@ -396,12 +325,14 @@ void gui_init(dt_lib_module_t *self)
   self->data = (void *)d;
 
   GtkWidget *box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-  self->widget = dt_ui_scroll_wrap(box, 50, "plugins/lighttable/recentcollect/windowheight");
+  self->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+  gtk_container_add(GTK_CONTAINER(self->widget),
+                    dt_ui_resize_wrap(box, 50, "plugins/lighttable/recentcollect/windowheight"));
   d->box = box;
   d->inited = 0;
 
   // add buttons in the list, set them all to invisible
-  for(int k = 0; k < _conf_get_max_items(); k++)
+  for(int k = 0; k < _conf_get_max_shown_items(); k++)
   {
     dt_lib_recentcollect_item_t *item = (dt_lib_recentcollect_item_t *)malloc(sizeof(dt_lib_recentcollect_item_t));
     d->items = g_list_append(d->items, item);
@@ -427,7 +358,7 @@ void gui_init(dt_lib_module_t *self)
 void gui_cleanup(dt_lib_module_t *self)
 {
   const int curr_pos = dt_ui_thumbtable(darktable.gui->ui)->offset;
-  dt_conf_set_int("plugins/lighttable/recentcollect/pos0", curr_pos);
+  dt_conf_set_int("plugins/lighttable/collect/history_pos0", curr_pos);
   DT_DEBUG_CONTROL_SIGNAL_DISCONNECT(darktable.signals, G_CALLBACK(_lib_recentcollection_updated), self);
   free(self->data);
   self->data = NULL;
