@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2009-2021 darktable developers.
+    Copyright (C) 2009-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,15 +49,18 @@
     control which view the module should be available in also
     which placement in the panels the module have.
 */
-typedef enum
+typedef enum dt_view_type_flags_t
 {
-  DT_VIEW_LIGHTTABLE = 1,
-  DT_VIEW_DARKROOM = 2,
-  DT_VIEW_TETHERING = 4,
-  DT_VIEW_MAP = 8,
-  DT_VIEW_SLIDESHOW = 16,
-  DT_VIEW_PRINT = 32,
-  DT_VIEW_KNIGHT = 64
+  DT_VIEW_NONE       = 0,
+  DT_VIEW_LIGHTTABLE = 1 << 0,
+  DT_VIEW_DARKROOM   = 1 << 1,
+  DT_VIEW_TETHERING  = 1 << 2,
+  DT_VIEW_MAP        = 1 << 3,
+  DT_VIEW_SLIDESHOW  = 1 << 4,
+  DT_VIEW_PRINT      = 1 << 5,
+  DT_VIEW_KNIGHT     = 1 << 6,
+  DT_VIEW_OTHER      = 1 << 30, // for your own unpublished user view
+  DT_VIEW_ALL        = ~DT_VIEW_NONE,
 } dt_view_type_flags_t;
 
 // flags that a view can set in flags()
@@ -115,10 +118,6 @@ typedef struct dt_mouse_action_t
   gchar name[256];
 } dt_mouse_action_t;
 
-#define DT_VIEW_ALL                                                                              \
-  (DT_VIEW_LIGHTTABLE | DT_VIEW_DARKROOM | DT_VIEW_TETHERING | DT_VIEW_MAP | DT_VIEW_SLIDESHOW | \
-   DT_VIEW_PRINT | DT_VIEW_KNIGHT)
-
 /* maximum zoom factor for the lighttable */
 #define DT_LIGHTTABLE_MAX_ZOOM 25
 
@@ -162,16 +161,22 @@ typedef enum dt_view_image_over_t
 } dt_view_image_over_t;
 
 /** returns an uppercase string of file extension **plus** some flag information **/
-char* dt_view_extend_modes_str(const char * name, const gboolean is_hdr, const gboolean is_bw, const gboolean is_bw_flow);
+char* dt_view_extend_modes_str(const char * name,
+                               const gboolean is_hdr,
+                               const gboolean is_bw,
+                               const gboolean is_bw_flow);
 /** expose an image and return a cair0_surface. */
-dt_view_surface_value_t dt_view_image_get_surface(int imgid, int width, int height, cairo_surface_t **surface,
+dt_view_surface_value_t dt_view_image_get_surface(dt_imgid_t imgid,
+                                                  int32_t width,
+                                                  int32_t height,
+                                                  cairo_surface_t **surface,
                                                   const gboolean quality);
 
 
 /** Set the selection bit to a given value for the specified image */
-void dt_view_set_selection(int imgid, int value);
+void dt_view_set_selection(dt_imgid_t imgid, int value);
 /** toggle selection of given image. */
-void dt_view_toggle_selection(int imgid);
+void dt_view_toggle_selection(dt_imgid_t imgid);
 
 /**
  * holds all relevant data needed to manage the view
@@ -311,8 +316,8 @@ typedef struct dt_view_manager_t
       void (*culling_preview_refresh)(struct dt_view_t *view);
       void (*culling_preview_reload_overlays)(struct dt_view_t *view);
       gboolean (*get_preview_state)(struct dt_view_t *view);
-      void (*set_preview_state)(struct dt_view_t *view, gboolean state, gboolean focus);
-      void (*change_offset)(struct dt_view_t *view, gboolean reset, gint imgid);
+      void (*set_preview_state)(struct dt_view_t *view, gboolean state, gboolean sticky, gboolean focus);
+      void (*change_offset)(struct dt_view_t *view, gboolean reset, dt_imgid_t imgid);
     } lighttable;
 
     /* tethering view proxy object */
@@ -344,7 +349,7 @@ typedef struct dt_view_manager_t
       gboolean (*remove_marker)(const dt_view_t *view, dt_geo_map_display_t type, GObject *marker);
       void (*add_location)(const dt_view_t *view, dt_map_location_data_t *p, const guint posid);
       void (*location_action)(const dt_view_t *view, const int action);
-      void (*drag_set_icon)(const dt_view_t *view, GdkDragContext *context, const int imgid, const int count);
+      void (*drag_set_icon)(const dt_view_t *view, GdkDragContext *context, const dt_imgid_t imgid, const int count);
       gboolean (*redraw)(gpointer user_data);
       gboolean (*display_selected)(gpointer user_data);
     } map;
@@ -369,9 +374,9 @@ void dt_view_manager_cleanup(dt_view_manager_t *vm);
 
 /** return translated name. */
 const char *dt_view_manager_name(dt_view_manager_t *vm);
-/** switch to this module. returns non-null if the module fails to change. */
-int dt_view_manager_switch(dt_view_manager_t *vm, const char *view_name);
-int dt_view_manager_switch_by_view(dt_view_manager_t *vm, const dt_view_t *new_view);
+/** switch to this module. returns TRUE if the module fails to change. */
+gboolean dt_view_manager_switch(dt_view_manager_t *vm, const char *view_name);
+gboolean dt_view_manager_switch_by_view(dt_view_manager_t *vm, const dt_view_t *new_view);
 /** expose current module. */
 void dt_view_manager_expose(dt_view_manager_t *vm, cairo_t *cr, int32_t width, int32_t height,
                             int32_t pointerx, int32_t pointery);
@@ -432,7 +437,7 @@ GtkWidget *dt_view_filter_get_count(const dt_view_manager_t *vm);
 
 // active images functions
 void dt_view_active_images_reset(gboolean raise);
-void dt_view_active_images_add(int imgid, gboolean raise);
+void dt_view_active_images_add(dt_imgid_t imgid, gboolean raise);
 GSList *dt_view_active_images_get();
 
 /** get the lighttable current layout */
@@ -442,7 +447,7 @@ dt_darkroom_layout_t dt_view_darkroom_get_layout(dt_view_manager_t *vm);
 /** get the lighttable full preview state */
 gboolean dt_view_lighttable_preview_state(dt_view_manager_t *vm);
 /** set the lighttable full preview state */
-void dt_view_lighttable_set_preview_state(dt_view_manager_t *vm, gboolean state, gboolean focus);
+void dt_view_lighttable_set_preview_state(dt_view_manager_t *vm, gboolean state, gboolean sticky, gboolean focus);
 /** sets the lighttable image in row zoom */
 void dt_view_lighttable_set_zoom(dt_view_manager_t *vm, gint zoom);
 /** gets the lighttable image in row zoom */
@@ -454,7 +459,7 @@ void dt_view_lighttable_culling_preview_refresh(dt_view_manager_t *vm);
 /** force refresh of culling and/or preview overlays */
 void dt_view_lighttable_culling_preview_reload_overlays(dt_view_manager_t *vm);
 /** sets the offset image (for culling and full preview) */
-void dt_view_lighttable_change_offset(dt_view_manager_t *vm, gboolean reset, gint imgid);
+void dt_view_lighttable_change_offset(dt_view_manager_t *vm, gboolean reset, dt_imgid_t imgid);
 
 /* accel window */
 void dt_view_accels_show(dt_view_manager_t *vm);
@@ -462,7 +467,7 @@ void dt_view_accels_hide(dt_view_manager_t *vm);
 void dt_view_accels_refresh(dt_view_manager_t *vm);
 
 /* audio */
-void dt_view_audio_start(dt_view_manager_t *vm, int imgid);
+void dt_view_audio_start(dt_view_manager_t *vm, dt_imgid_t imgid);
 void dt_view_audio_stop(dt_view_manager_t *vm);
 
 /*
@@ -477,7 +482,7 @@ GObject *dt_view_map_add_marker(const dt_view_manager_t *vm, dt_geo_map_display_
 gboolean dt_view_map_remove_marker(const dt_view_manager_t *vm, dt_geo_map_display_t type, GObject *marker);
 void dt_view_map_add_location(const dt_view_manager_t *vm, dt_map_location_data_t *p, const guint posid);
 void dt_view_map_location_action(const dt_view_manager_t *vm, const int action);
-void dt_view_map_drag_set_icon(const dt_view_manager_t *vm, GdkDragContext *context, const int imgid, const int count);
+void dt_view_map_drag_set_icon(const dt_view_manager_t *vm, GdkDragContext *context, const dt_imgid_t imgid, const int count);
 #endif
 
 /*
