@@ -1,14 +1,7 @@
 #pragma once
 
-#ifdef __MSVCRT_VERSION__
-#undef __MSVCRT_VERSION__
-#endif
-#define __MSVCRT_VERSION__ 0x0700
-
 #define XMD_H
 
-#undef _WIN32_WINNT
-#define _WIN32_WINNT 0x0600
 #include <winsock2.h>
 #include <windows.h>
 #include <psapi.h>
@@ -24,6 +17,9 @@
 #define sleep(n) Sleep(1000 * n)
 #define HAVE_BOOLEAN
 
-// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.sh
+// clang-format off
+// modelines: These editor modelines have been set for all relevant files by tools/update_modelines.py
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
+// clang-format on
+
